/*
 * Copyright (c) 2014-2021 The Linux Foundation. All rights reserved.
 *
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * DOC: reg_services_common.c
 * This file defines regulatory component service functions
 */

#include <wlan_cmn.h>
#include <reg_services_public_struct.h>
#ifdef CONFIG_AFC_SUPPORT
#include <wlan_reg_services_api.h>
#include "reg_opclass.h"
#endif
#include <wlan_objmgr_psoc_obj.h>
#include <qdf_lock.h>
#include "reg_priv_objs.h"
#include "reg_utils.h"
#include "reg_callbacks.h"
#include "reg_services_common.h"
#include <wlan_objmgr_psoc_obj.h>
#include "reg_db.h"
#include "reg_db_parser.h"
#include "reg_build_chan_list.h"
#include <wlan_objmgr_pdev_obj.h>
#include <target_if.h>
#ifdef WLAN_FEATURE_GET_USABLE_CHAN_LIST
#include "wlan_mlme_ucfg_api.h"
#include "wlan_nan_api.h"
#endif

const struct chan_map *channel_map;

#ifdef CONFIG_CHAN_FREQ_API
/* bonded_chan_40mhz_list_freq - List of 40MHz bonnded channel frequencies */
static const struct bonded_channel_freq bonded_chan_40mhz_list_freq[] = {
	{5180, 5200},
	{5220, 5240},
	{5260, 5280},
	{5300, 5320},
	{5500, 5520},
	{5540, 5560},
	{5580, 5600},
	{5620, 5640},
	{5660, 5680},
	{5700, 5720},
	{5745, 5765},
	{5785, 5805},
	{5825, 5845},
	{5865, 5885},
#ifdef CONFIG_BAND_6GHZ
	{5955, 5975},
	{5995, 6015},
	{6035, 6055},
	{6075, 6095},
	{6115, 6135},
	{6155, 6175},
	{6195, 6215},
	{6235, 6255},
	{6275, 6295},
	{6315, 6335},
	{6355, 6375},
	{6395, 6415},
	{6435, 6455},
	{6475, 6495},
	{6515, 6535},
	{6555, 6575},
	{6595, 6615},
	{6635, 6655},
	{6675, 6695},
	{6715, 6735},
	{6755, 6775},
	{6795, 6815},
	{6835, 6855},
	{6875, 6895},
	{6915, 6935},
	{6955, 6975},
	{6995, 7015},
	{7035, 7055},
	{7075, 7095}
#endif /*CONFIG_BAND_6GHZ*/
};

/* bonded_chan_80mhz_list_freq - List of 80MHz bonnded channel frequencies */
static const struct bonded_channel_freq bonded_chan_80mhz_list_freq[] = {
	{5180, 5240},
	{5260, 5320},
	{5500, 5560},
	{5580, 5640},
	{5660, 5720},
	{5745, 5805},
	{5825, 5885},
#ifdef CONFIG_BAND_6GHZ
	{5955, 6015},
	{6035, 6095},
	{6115, 6175},
	{6195, 6255},
	{6275, 6335},
	{6355, 6415},
	{6435, 6495},
	{6515, 6575},
	{6595, 6655},
	{6675, 6735},
	{6755, 6815},
	{6835, 6895},
	{6915, 6975},
	{6995, 7055}
#endif /*CONFIG_BAND_6GHZ*/
};

/* bonded_chan_160mhz_list_freq - List of 160MHz bonnded channel frequencies */
static const struct bonded_channel_freq bonded_chan_160mhz_list_freq[] = {
	{5180, 5320},
	{5500, 5640},
	{5745, 5885},
#ifdef CONFIG_BAND_6GHZ
	{5955, 6095},
	{6115, 6255},
	{6275, 6415},
	{6435, 6575},
	{6595, 6735},
	{6755, 6895},
	{6915, 7055}
#endif /*CONFIG_BAND_6GHZ*/
};

const struct bonded_channel_freq *
reg_get_bonded_chan_entry(qdf_freq_t freq,
			  enum phy_ch_width chwidth)
{
	const struct bonded_channel_freq *bonded_chan_arr;
	uint16_t array_size, i;

	switch (chwidth) {
	case CH_WIDTH_160MHZ:
		bonded_chan_arr = bonded_chan_160mhz_list_freq;
		array_size = QDF_ARRAY_SIZE(bonded_chan_160mhz_list_freq);
		break;
	case CH_WIDTH_80MHZ:
	case CH_WIDTH_80P80MHZ:
		bonded_chan_arr = bonded_chan_80mhz_list_freq;
		array_size = QDF_ARRAY_SIZE(bonded_chan_80mhz_list_freq);
		break;
	case CH_WIDTH_40MHZ:
		bonded_chan_arr = bonded_chan_40mhz_list_freq;
		array_size = QDF_ARRAY_SIZE(bonded_chan_40mhz_list_freq);
		break;
	default:
		reg_debug("Could not find bonded_chan_array for chwidth %d",
			  chwidth);
		return NULL;
	}

	for (i = 0; i < array_size; i++) {
		if ((freq >= bonded_chan_arr[i].start_freq) &&
		    (freq <= bonded_chan_arr[i].end_freq)) {
			return &bonded_chan_arr[i];
		}
	}

	reg_debug("Could not find a bonded pair for freq %d and width %d",
		  freq, chwidth);
	return NULL;
}

#ifdef WLAN_FEATURE_11BE
/* bonded_chan_320mhz_list_freq - List of 320MHz bonnded channel frequencies */
static const struct bonded_channel_freq bonded_chan_320mhz_list_freq[] = {
	{5500, 5800}, /* center freq: 5650 */
#ifdef CONFIG_BAND_6GHZ
	{5955, 6255}, /* center freq: 6105 */
	{6115, 6415}, /* center freq: 6265 */
	{6275, 6575}, /* center freq: 6425 */
	{6435, 6735}, /* center freq: 6585 */
	{6595, 6895}, /* center freq: 6745 */
	{6755, 7055}  /* center freq: 6905 */
#endif /*CONFIG_BAND_6GHZ*/
};
#endif

#endif /*CONFIG_CHAN_FREQ_API*/

enum phy_ch_width get_next_lower_bandwidth(enum phy_ch_width ch_width)
{
	static const enum phy_ch_width get_next_lower_bw[] = {
    /* 80+80 mode not supported in chips that support 320 mode */
#ifdef WLAN_FEATURE_11BE
		[CH_WIDTH_320MHZ] = CH_WIDTH_160MHZ,
#endif
		[CH_WIDTH_80P80MHZ] = CH_WIDTH_160MHZ,
		[CH_WIDTH_160MHZ] = CH_WIDTH_80MHZ,
		[CH_WIDTH_80MHZ] = CH_WIDTH_40MHZ,
		[CH_WIDTH_40MHZ] = CH_WIDTH_20MHZ,
		[CH_WIDTH_20MHZ] = CH_WIDTH_10MHZ,
		[CH_WIDTH_10MHZ] = CH_WIDTH_5MHZ,
		[CH_WIDTH_5MHZ] = CH_WIDTH_INVALID
	};

	return get_next_lower_bw[ch_width];
}

const struct chan_map channel_map_us[NUM_CHANNELS] = {
	[CHAN_ENUM_2412] = {2412, 1, 20, 40},
	[CHAN_ENUM_2417] = {2417, 2, 20, 40},
	[CHAN_ENUM_2422] = {2422, 3, 20, 40},
	[CHAN_ENUM_2427] = {2427, 4, 20, 40},
	[CHAN_ENUM_2432] = {2432, 5, 20, 40},
	[CHAN_ENUM_2437] = {2437, 6, 20, 40},
	[CHAN_ENUM_2442] = {2442, 7, 20, 40},
	[CHAN_ENUM_2447] = {2447, 8, 20, 40},
	[CHAN_ENUM_2452] = {2452, 9, 20, 40},
	[CHAN_ENUM_2457] = {2457, 10, 20, 40},
	[CHAN_ENUM_2462] = {2462, 11, 20, 40},
	[CHAN_ENUM_2467] = {2467, 12, 20, 40},
	[CHAN_ENUM_2472] = {2472, 13, 20, 40},
	[CHAN_ENUM_2484] = {2484, 14, 20, 20},
#ifdef CONFIG_49GHZ_CHAN
	[CHAN_ENUM_4912] = {4912, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4915] = {4915, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4917] = {4917, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4920] = {4920, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4922] = {4922, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4925] = {4925, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4927] = {4927, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4932] = {4932, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4935] = {4935, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4937] = {4937, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4940] = {4940, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4942] = {4942, 1, 5, 5},
	[CHAN_ENUM_4945] = {4945, 11, 10, 10},
	[CHAN_ENUM_4947] = {4947, 2, 5, 5},
	[CHAN_ENUM_4950] = {4950, 20, 10, 20},
	[CHAN_ENUM_4952] = {4952, 3, 5, 5},
	[CHAN_ENUM_4955] = {4955, 21, 10, 20},
	[CHAN_ENUM_4957] = {4957, 4, 5, 5},
	[CHAN_ENUM_4960] = {4960, 22, 10, 20},
	[CHAN_ENUM_4962] = {4962, 5, 5, 5},
	[CHAN_ENUM_4965] = {4965, 23, 10, 20},
	[CHAN_ENUM_4967] = {4967, 6, 5, 5},
	[CHAN_ENUM_4970] = {4970, 24, 10, 20},
	[CHAN_ENUM_4972] = {4972, 7, 5, 5},
	[CHAN_ENUM_4975] = {4975, 25, 10, 20},
	[CHAN_ENUM_4977] = {4977, 8, 5, 5},
	[CHAN_ENUM_4980] = {4980, 26, 10, 20},
	[CHAN_ENUM_4982] = {4982, 9, 5, 5},
	[CHAN_ENUM_4985] = {4985, 19, 10, 10},
	[CHAN_ENUM_4987] = {4987, 10, 5, 5},
	[CHAN_ENUM_5032] = {5032, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5035] = {5035, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5037] = {5037, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5040] = {5040, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5042] = {5042, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5045] = {5045, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5047] = {5047, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5052] = {5052, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5055] = {5055, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5057] = {5057, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5060] = {5060, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5080] = {5080, INVALID_CHANNEL_NUM, 2, 20},
#endif /* CONFIG_49GHZ_CHAN */
	[CHAN_ENUM_5180] = {5180, 36, 2, 160},
	[CHAN_ENUM_5200] = {5200, 40, 2, 160},
	[CHAN_ENUM_5220] = {5220, 44, 2, 160},
	[CHAN_ENUM_5240] = {5240, 48, 2, 160},
	[CHAN_ENUM_5260] = {5260, 52, 2, 160},
	[CHAN_ENUM_5280] = {5280, 56, 2, 160},
	[CHAN_ENUM_5300] = {5300, 60, 2, 160},
	[CHAN_ENUM_5320] = {5320, 64, 2, 160},
	[CHAN_ENUM_5500] = {5500, 100, 2, 240},
	[CHAN_ENUM_5520] = {5520, 104, 2, 240},
	[CHAN_ENUM_5540] = {5540, 108, 2, 240},
	[CHAN_ENUM_5560] = {5560, 112, 2, 240},
	[CHAN_ENUM_5580] = {5580, 116, 2, 240},
	[CHAN_ENUM_5600] = {5600, 120, 2, 240},
	[CHAN_ENUM_5620] = {5620, 124, 2, 240},
	[CHAN_ENUM_5640] = {5640, 128, 2, 240},
	[CHAN_ENUM_5660] = {5660, 132, 2, 240},
	[CHAN_ENUM_5680] = {5680, 136, 2, 240},
	[CHAN_ENUM_5700] = {5700, 140, 2, 240},
	[CHAN_ENUM_5720] = {5720, 144, 2, 240},
	[CHAN_ENUM_5745] = {5745, 149, 2, 160},
	[CHAN_ENUM_5765] = {5765, 153, 2, 160},
	[CHAN_ENUM_5785] = {5785, 157, 2, 160},
	[CHAN_ENUM_5805] = {5805, 161, 2, 160},
	[CHAN_ENUM_5825] = {5825, 165, 2, 160},
	[CHAN_ENUM_5845] = {5845, 169, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5850] = {5850, 170, 2, 160},
	[CHAN_ENUM_5855] = {5855, 171, 2, 160},
	[CHAN_ENUM_5860] = {5860, 172, 2, 160},
#endif
	[CHAN_ENUM_5865] = {5865, 173, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5870] = {5870, 174, 2, 160},
	[CHAN_ENUM_5875] = {5875, 175, 2, 160},
	[CHAN_ENUM_5880] = {5880, 176, 2, 160},
#endif
	[CHAN_ENUM_5885] = {5885, 177, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5890] = {5890, 178, 2, 160},
	[CHAN_ENUM_5895] = {5895, 179, 2, 160},
	[CHAN_ENUM_5900] = {5900, 180, 2, 160},
	[CHAN_ENUM_5905] = {5905, 181, 2, 160},
	[CHAN_ENUM_5910] = {5910, 182, 2, 160},
	[CHAN_ENUM_5915] = {5915, 183, 2, 160},
	[CHAN_ENUM_5920] = {5920, 184, 2, 160},
#endif /* WLAN_FEATURE_DSRC */
#ifdef CONFIG_BAND_6GHZ
	[CHAN_ENUM_5935] = {5935, 2, 2, 20},
	[CHAN_ENUM_5955] = {5955, 1, 2, 320},
	[CHAN_ENUM_5975] = {5975, 5, 2, 320},
	[CHAN_ENUM_5995] = {5995, 9, 2, 320},
	[CHAN_ENUM_6015] = {6015, 13, 2, 320},
	[CHAN_ENUM_6035] = {6035, 17, 2, 320},
	[CHAN_ENUM_6055] = {6055, 21, 2, 320},
	[CHAN_ENUM_6075] = {6075, 25, 2, 320},
	[CHAN_ENUM_6095] = {6095, 29, 2, 320},
	[CHAN_ENUM_6115] = {6115, 33, 2, 320},
	[CHAN_ENUM_6135] = {6135, 37, 2, 320},
	[CHAN_ENUM_6155] = {6155, 41, 2, 320},
	[CHAN_ENUM_6175] = {6175, 45, 2, 320},
	[CHAN_ENUM_6195] = {6195, 49, 2, 320},
	[CHAN_ENUM_6215] = {6215, 53, 2, 320},
	[CHAN_ENUM_6235] = {6235, 57, 2, 320},
	[CHAN_ENUM_6255] = {6255, 61, 2, 320},
	[CHAN_ENUM_6275] = {6275, 65, 2, 320},
	[CHAN_ENUM_6295] = {6295, 69, 2, 320},
	[CHAN_ENUM_6315] = {6315, 73, 2, 320},
	[CHAN_ENUM_6335] = {6335, 77, 2, 320},
	[CHAN_ENUM_6355] = {6355, 81, 2, 320},
	[CHAN_ENUM_6375] = {6375, 85, 2, 320},
	[CHAN_ENUM_6395] = {6395, 89, 2, 320},
	[CHAN_ENUM_6415] = {6415, 93, 2, 320},
	[CHAN_ENUM_6435] = {6435, 97, 2, 320},
	[CHAN_ENUM_6455] = {6455, 101, 2, 320},
	[CHAN_ENUM_6475] = {6475, 105, 2, 320},
	[CHAN_ENUM_6495] = {6495, 109, 2, 320},
	[CHAN_ENUM_6515] = {6515, 113, 2, 320},
	[CHAN_ENUM_6535] = {6535, 117, 2, 320},
	[CHAN_ENUM_6555] = {6555, 121, 2, 320},
	[CHAN_ENUM_6575] = {6575, 125, 2, 320},
	[CHAN_ENUM_6595] = {6595, 129, 2, 320},
	[CHAN_ENUM_6615] = {6615, 133, 2, 320},
	[CHAN_ENUM_6635] = {6635, 137, 2, 320},
	[CHAN_ENUM_6655] = {6655, 141, 2, 320},
	[CHAN_ENUM_6675] = {6675, 145, 2, 320},
	[CHAN_ENUM_6695] = {6695, 149, 2, 320},
	[CHAN_ENUM_6715] = {6715, 153, 2, 320},
	[CHAN_ENUM_6735] = {6735, 157, 2, 320},
	[CHAN_ENUM_6755] = {6755, 161, 2, 320},
	[CHAN_ENUM_6775] = {6775, 165, 2, 320},
	[CHAN_ENUM_6795] = {6795, 169, 2, 320},
	[CHAN_ENUM_6815] = {6815, 173, 2, 320},
	[CHAN_ENUM_6835] = {6835, 177, 2, 320},
	[CHAN_ENUM_6855] = {6855, 181, 2, 320},
	[CHAN_ENUM_6875] = {6875, 185, 2, 320},
	[CHAN_ENUM_6895] = {6895, 189, 2, 320},
	[CHAN_ENUM_6915] = {6915, 193, 2, 320},
	[CHAN_ENUM_6935] = {6935, 197, 2, 320},
	[CHAN_ENUM_6955] = {6955, 201, 2, 320},
	[CHAN_ENUM_6975] = {6975, 205, 2, 320},
	[CHAN_ENUM_6995] = {6995, 209, 2, 320},
	[CHAN_ENUM_7015] = {7015, 213, 2, 320},
	[CHAN_ENUM_7035] = {7035, 217, 2, 320},
	[CHAN_ENUM_7055] = {7055, 221, 2, 320},
	[CHAN_ENUM_7075] = {7075, 225, 2, 160},
	[CHAN_ENUM_7095] = {7095, 229, 2, 160},
	[CHAN_ENUM_7115] = {7115, 233, 2, 160}
#endif /* CONFIG_BAND_6GHZ */
};

const struct chan_map channel_map_eu[NUM_CHANNELS] = {
	[CHAN_ENUM_2412] = {2412, 1, 20, 40},
	[CHAN_ENUM_2417] = {2417, 2, 20, 40},
	[CHAN_ENUM_2422] = {2422, 3, 20, 40},
	[CHAN_ENUM_2427] = {2427, 4, 20, 40},
	[CHAN_ENUM_2432] = {2432, 5, 20, 40},
	[CHAN_ENUM_2437] = {2437, 6, 20, 40},
	[CHAN_ENUM_2442] = {2442, 7, 20, 40},
	[CHAN_ENUM_2447] = {2447, 8, 20, 40},
	[CHAN_ENUM_2452] = {2452, 9, 20, 40},
	[CHAN_ENUM_2457] = {2457, 10, 20, 40},
	[CHAN_ENUM_2462] = {2462, 11, 20, 40},
	[CHAN_ENUM_2467] = {2467, 12, 20, 40},
	[CHAN_ENUM_2472] = {2472, 13, 20, 40},
	[CHAN_ENUM_2484] = {2484, 14, 20, 20},
#ifdef CONFIG_49GHZ_CHAN
	[CHAN_ENUM_4912] = {4912, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4915] = {4915, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4917] = {4917, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4920] = {4920, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4922] = {4922, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4925] = {4925, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4927] = {4927, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4932] = {4932, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4935] = {4935, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4937] = {4937, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4940] = {4940, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4942] = {4942, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4945] = {4945, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4947] = {4947, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4950] = {4950, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4952] = {4952, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4955] = {4955, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4957] = {4957, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4960] = {4960, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4962] = {4962, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4965] = {4965, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4967] = {4967, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4970] = {4970, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4972] = {4972, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4975] = {4975, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4977] = {4977, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4980] = {4980, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4982] = {4982, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4985] = {4985, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4987] = {4987, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5032] = {5032, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5035] = {5035, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5037] = {5037, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5040] = {5040, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5042] = {5042, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5045] = {5045, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5047] = {5047, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5052] = {5052, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5055] = {5055, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5057] = {5057, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5060] = {5060, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5080] = {5080, INVALID_CHANNEL_NUM, 2, 20},
#endif /* CONFIG_49GHZ_CHAN */
	[CHAN_ENUM_5180] = {5180, 36, 2, 160},
	[CHAN_ENUM_5200] = {5200, 40, 2, 160},
	[CHAN_ENUM_5220] = {5220, 44, 2, 160},
	[CHAN_ENUM_5240] = {5240, 48, 2, 160},
	[CHAN_ENUM_5260] = {5260, 52, 2, 160},
	[CHAN_ENUM_5280] = {5280, 56, 2, 160},
	[CHAN_ENUM_5300] = {5300, 60, 2, 160},
	[CHAN_ENUM_5320] = {5320, 64, 2, 160},
	[CHAN_ENUM_5500] = {5500, 100, 2, 240},
	[CHAN_ENUM_5520] = {5520, 104, 2, 240},
	[CHAN_ENUM_5540] = {5540, 108, 2, 240},
	[CHAN_ENUM_5560] = {5560, 112, 2, 240},
	[CHAN_ENUM_5580] = {5580, 116, 2, 240},
	[CHAN_ENUM_5600] = {5600, 120, 2, 240},
	[CHAN_ENUM_5620] = {5620, 124, 2, 240},
	[CHAN_ENUM_5640] = {5640, 128, 2, 240},
	[CHAN_ENUM_5660] = {5660, 132, 2, 240},
	[CHAN_ENUM_5680] = {5680, 136, 2, 240},
	[CHAN_ENUM_5700] = {5700, 140, 2, 240},
	[CHAN_ENUM_5720] = {5720, 144, 2, 240},
	[CHAN_ENUM_5745] = {5745, 149, 2, 160},
	[CHAN_ENUM_5765] = {5765, 153, 2, 160},
	[CHAN_ENUM_5785] = {5785, 157, 2, 160},
	[CHAN_ENUM_5805] = {5805, 161, 2, 160},
	[CHAN_ENUM_5825] = {5825, 165, 2, 160},
	[CHAN_ENUM_5845] = {5845, 169, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5850] = {5850, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5855] = {5855, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5860] = {5860, INVALID_CHANNEL_NUM, 2, 160},
#endif
	[CHAN_ENUM_5865] = {5865, 173, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5870] = {5870, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5875] = {5875, 175, 2, 160},
	[CHAN_ENUM_5880] = {5880, 176, 2, 160},
#endif
	[CHAN_ENUM_5885] = {5885, 177, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5890] = {5890, 178, 2, 160},
	[CHAN_ENUM_5895] = {5895, 179, 2, 160},
	[CHAN_ENUM_5900] = {5900, 180, 2, 160},
	[CHAN_ENUM_5905] = {5905, 181, 2, 160},
	[CHAN_ENUM_5910] = {5910, 182, 2, 160},
	[CHAN_ENUM_5915] = {5915, 183, 2, 160},
	[CHAN_ENUM_5920] = {5920, 184, 2, 160},
#endif /* WLAN_FEATURE_DSRC */
#ifdef CONFIG_BAND_6GHZ
	[CHAN_ENUM_5935] = {5935, 2, 2, 20},
	[CHAN_ENUM_5955] = {5955, 1, 2, 320},
	[CHAN_ENUM_5975] = {5975, 5, 2, 320},
	[CHAN_ENUM_5995] = {5995, 9, 2, 320},
	[CHAN_ENUM_6015] = {6015, 13, 2, 320},
	[CHAN_ENUM_6035] = {6035, 17, 2, 320},
	[CHAN_ENUM_6055] = {6055, 21, 2, 320},
	[CHAN_ENUM_6075] = {6075, 25, 2, 320},
	[CHAN_ENUM_6095] = {6095, 29, 2, 320},
	[CHAN_ENUM_6115] = {6115, 33, 2, 320},
	[CHAN_ENUM_6135] = {6135, 37, 2, 320},
	[CHAN_ENUM_6155] = {6155, 41, 2, 320},
	[CHAN_ENUM_6175] = {6175, 45, 2, 320},
	[CHAN_ENUM_6195] = {6195, 49, 2, 320},
	[CHAN_ENUM_6215] = {6215, 53, 2, 320},
	[CHAN_ENUM_6235] = {6235, 57, 2, 320},
	[CHAN_ENUM_6255] = {6255, 61, 2, 320},
	[CHAN_ENUM_6275] = {6275, 65, 2, 320},
	[CHAN_ENUM_6295] = {6295, 69, 2, 320},
	[CHAN_ENUM_6315] = {6315, 73, 2, 320},
	[CHAN_ENUM_6335] = {6335, 77, 2, 320},
	[CHAN_ENUM_6355] = {6355, 81, 2, 320},
	[CHAN_ENUM_6375] = {6375, 85, 2, 320},
	[CHAN_ENUM_6395] = {6395, 89, 2, 320},
	[CHAN_ENUM_6415] = {6415, 93, 2, 320},
	[CHAN_ENUM_6435] = {6435, 97, 2, 320},
	[CHAN_ENUM_6455] = {6455, 101, 2, 320},
	[CHAN_ENUM_6475] = {6475, 105, 2, 320},
	[CHAN_ENUM_6495] = {6495, 109, 2, 320},
	[CHAN_ENUM_6515] = {6515, 113, 2, 320},
	[CHAN_ENUM_6535] = {6535, 117, 2, 320},
	[CHAN_ENUM_6555] = {6555, 121, 2, 320},
	[CHAN_ENUM_6575] = {6575, 125, 2, 320},
	[CHAN_ENUM_6595] = {6595, 129, 2, 320},
	[CHAN_ENUM_6615] = {6615, 133, 2, 320},
	[CHAN_ENUM_6635] = {6635, 137, 2, 320},
	[CHAN_ENUM_6655] = {6655, 141, 2, 320},
	[CHAN_ENUM_6675] = {6675, 145, 2, 320},
	[CHAN_ENUM_6695] = {6695, 149, 2, 320},
	[CHAN_ENUM_6715] = {6715, 153, 2, 320},
	[CHAN_ENUM_6735] = {6735, 157, 2, 320},
	[CHAN_ENUM_6755] = {6755, 161, 2, 320},
	[CHAN_ENUM_6775] = {6775, 165, 2, 320},
	[CHAN_ENUM_6795] = {6795, 169, 2, 320},
	[CHAN_ENUM_6815] = {6815, 173, 2, 320},
	[CHAN_ENUM_6835] = {6835, 177, 2, 320},
	[CHAN_ENUM_6855] = {6855, 181, 2, 320},
	[CHAN_ENUM_6875] = {6875, 185, 2, 320},
	[CHAN_ENUM_6895] = {6895, 189, 2, 320},
	[CHAN_ENUM_6915] = {6915, 193, 2, 320},
	[CHAN_ENUM_6935] = {6935, 197, 2, 320},
	[CHAN_ENUM_6955] = {6955, 201, 2, 320},
	[CHAN_ENUM_6975] = {6975, 205, 2, 320},
	[CHAN_ENUM_6995] = {6995, 209, 2, 320},
	[CHAN_ENUM_7015] = {7015, 213, 2, 320},
	[CHAN_ENUM_7035] = {7035, 217, 2, 320},
	[CHAN_ENUM_7055] = {7055, 221, 2, 320},
	[CHAN_ENUM_7075] = {7075, 225, 2, 160},
	[CHAN_ENUM_7095] = {7095, 229, 2, 160},
	[CHAN_ENUM_7115] = {7115, 233, 2, 160}
#endif /* CONFIG_BAND_6GHZ */
};

const struct chan_map channel_map_jp[NUM_CHANNELS] = {
	[CHAN_ENUM_2412] = {2412, 1, 20, 40},
	[CHAN_ENUM_2417] = {2417, 2, 20, 40},
	[CHAN_ENUM_2422] = {2422, 3, 20, 40},
	[CHAN_ENUM_2427] = {2427, 4, 20, 40},
	[CHAN_ENUM_2432] = {2432, 5, 20, 40},
	[CHAN_ENUM_2437] = {2437, 6, 20, 40},
	[CHAN_ENUM_2442] = {2442, 7, 20, 40},
	[CHAN_ENUM_2447] = {2447, 8, 20, 40},
	[CHAN_ENUM_2452] = {2452, 9, 20, 40},
	[CHAN_ENUM_2457] = {2457, 10, 20, 40},
	[CHAN_ENUM_2462] = {2462, 11, 20, 40},
	[CHAN_ENUM_2467] = {2467, 12, 20, 40},
	[CHAN_ENUM_2472] = {2472, 13, 20, 40},
	[CHAN_ENUM_2484] = {2484, 14, 20, 20},
#ifdef CONFIG_49GHZ_CHAN
	[CHAN_ENUM_4912] = {4912, 182, 5, 5},
	[CHAN_ENUM_4915] = {4915, 183, 10, 10},
	[CHAN_ENUM_4917] = {4917, 183, 5, 5},
	[CHAN_ENUM_4920] = {4920, 184, 10, 20},
	[CHAN_ENUM_4922] = {4922, 184, 5, 5},
	[CHAN_ENUM_4925] = {4925, 185, 10, 10},
	[CHAN_ENUM_4927] = {4927, 185, 5, 5},
	[CHAN_ENUM_4932] = {4932, 186, 5, 5},
	[CHAN_ENUM_4935] = {4935, 187, 10, 10},
	[CHAN_ENUM_4937] = {4937, 187, 5, 5},
	[CHAN_ENUM_4940] = {4940, 188, 10, 20},
	[CHAN_ENUM_4942] = {4942, 188, 5, 5},
	[CHAN_ENUM_4945] = {4945, 189, 10, 10},
	[CHAN_ENUM_4947] = {4947, 189, 5, 5},
	[CHAN_ENUM_4950] = {4950, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4952] = {4952, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4955] = {4955, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4957] = {4957, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4960] = {4960, 192, 20, 20},
	[CHAN_ENUM_4962] = {4962, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4965] = {4965, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4967] = {4967, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4970] = {4970, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4972] = {4972, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4975] = {4975, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4977] = {4977, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4980] = {4980, 196, 20, 20},
	[CHAN_ENUM_4982] = {4982, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4985] = {4985, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4987] = {4987, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5032] = {5032, 6, 5, 5},
	[CHAN_ENUM_5035] = {5035, 7, 10, 10},
	[CHAN_ENUM_5037] = {5037, 7, 5, 5},
	[CHAN_ENUM_5040] = {5040, 8, 10, 20},
	[CHAN_ENUM_5042] = {5042, 8, 5, 5},
	[CHAN_ENUM_5045] = {5045, 9, 10, 10},
	[CHAN_ENUM_5047] = {5047, 9, 5, 5},
	[CHAN_ENUM_5052] = {5052, 10, 5, 5},
	[CHAN_ENUM_5055] = {5055, 11, 10, 10},
	[CHAN_ENUM_5057] = {5057, 11, 5, 5},
	[CHAN_ENUM_5060] = {5060, 12, 20, 20},
	[CHAN_ENUM_5080] = {5080, 16, 20, 20},
#endif /* CONFIG_49GHZ_CHAN */
	[CHAN_ENUM_5180] = {5180, 36, 2, 160},
	[CHAN_ENUM_5200] = {5200, 40, 2, 160},
	[CHAN_ENUM_5220] = {5220, 44, 2, 160},
	[CHAN_ENUM_5240] = {5240, 48, 2, 160},
	[CHAN_ENUM_5260] = {5260, 52, 2, 160},
	[CHAN_ENUM_5280] = {5280, 56, 2, 160},
	[CHAN_ENUM_5300] = {5300, 60, 2, 160},
	[CHAN_ENUM_5320] = {5320, 64, 2, 160},
	[CHAN_ENUM_5500] = {5500, 100, 2, 240},
	[CHAN_ENUM_5520] = {5520, 104, 2, 240},
	[CHAN_ENUM_5540] = {5540, 108, 2, 240},
	[CHAN_ENUM_5560] = {5560, 112, 2, 240},
	[CHAN_ENUM_5580] = {5580, 116, 2, 240},
	[CHAN_ENUM_5600] = {5600, 120, 2, 240},
	[CHAN_ENUM_5620] = {5620, 124, 2, 240},
	[CHAN_ENUM_5640] = {5640, 128, 2, 240},
	[CHAN_ENUM_5660] = {5660, 132, 2, 240},
	[CHAN_ENUM_5680] = {5680, 136, 2, 240},
	[CHAN_ENUM_5700] = {5700, 140, 2, 240},
	[CHAN_ENUM_5720] = {5720, 144, 2, 240},
	[CHAN_ENUM_5745] = {5745, 149, 2, 160},
	[CHAN_ENUM_5765] = {5765, 153, 2, 160},
	[CHAN_ENUM_5785] = {5785, 157, 2, 160},
	[CHAN_ENUM_5805] = {5805, 161, 2, 160},
	[CHAN_ENUM_5825] = {5825, 165, 2, 160},
	[CHAN_ENUM_5845] = {5845, 169, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5850] = {5850, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5855] = {5855, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5860] = {5860, INVALID_CHANNEL_NUM, 2, 160},
#endif
	[CHAN_ENUM_5865] = {5865, INVALID_CHANNEL_NUM, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5870] = {5870, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5875] = {5875, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5880] = {5880, INVALID_CHANNEL_NUM, 2, 160},
#endif
	[CHAN_ENUM_5885] = {5885, INVALID_CHANNEL_NUM, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5890] = {5890, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5895] = {5895, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5900] = {5900, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5905] = {5905, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5910] = {5910, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5915] = {5915, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5920] = {5920, INVALID_CHANNEL_NUM, 2, 160},
#endif /* WLAN_FEATURE_DSRC */
#ifdef CONFIG_BAND_6GHZ
	[CHAN_ENUM_5935] = {5935, 2, 2, 20},
	[CHAN_ENUM_5955] = {5955, 1, 2, 320},
	[CHAN_ENUM_5975] = {5975, 5, 2, 320},
	[CHAN_ENUM_5995] = {5995, 9, 2, 320},
	[CHAN_ENUM_6015] = {6015, 13, 2, 320},
	[CHAN_ENUM_6035] = {6035, 17, 2, 320},
	[CHAN_ENUM_6055] = {6055, 21, 2, 320},
	[CHAN_ENUM_6075] = {6075, 25, 2, 320},
	[CHAN_ENUM_6095] = {6095, 29, 2, 320},
	[CHAN_ENUM_6115] = {6115, 33, 2, 320},
	[CHAN_ENUM_6135] = {6135, 37, 2, 320},
	[CHAN_ENUM_6155] = {6155, 41, 2, 320},
	[CHAN_ENUM_6175] = {6175, 45, 2, 320},
	[CHAN_ENUM_6195] = {6195, 49, 2, 320},
	[CHAN_ENUM_6215] = {6215, 53, 2, 320},
	[CHAN_ENUM_6235] = {6235, 57, 2, 320},
	[CHAN_ENUM_6255] = {6255, 61, 2, 320},
	[CHAN_ENUM_6275] = {6275, 65, 2, 320},
	[CHAN_ENUM_6295] = {6295, 69, 2, 320},
	[CHAN_ENUM_6315] = {6315, 73, 2, 320},
	[CHAN_ENUM_6335] = {6335, 77, 2, 320},
	[CHAN_ENUM_6355] = {6355, 81, 2, 320},
	[CHAN_ENUM_6375] = {6375, 85, 2, 320},
	[CHAN_ENUM_6395] = {6395, 89, 2, 320},
	[CHAN_ENUM_6415] = {6415, 93, 2, 320},
	[CHAN_ENUM_6435] = {6435, 97, 2, 320},
	[CHAN_ENUM_6455] = {6455, 101, 2, 320},
	[CHAN_ENUM_6475] = {6475, 105, 2, 320},
	[CHAN_ENUM_6495] = {6495, 109, 2, 320},
	[CHAN_ENUM_6515] = {6515, 113, 2, 320},
	[CHAN_ENUM_6535] = {6535, 117, 2, 320},
	[CHAN_ENUM_6555] = {6555, 121, 2, 320},
	[CHAN_ENUM_6575] = {6575, 125, 2, 320},
	[CHAN_ENUM_6595] = {6595, 129, 2, 320},
	[CHAN_ENUM_6615] = {6615, 133, 2, 320},
	[CHAN_ENUM_6635] = {6635, 137, 2, 320},
	[CHAN_ENUM_6655] = {6655, 141, 2, 320},
	[CHAN_ENUM_6675] = {6675, 145, 2, 320},
	[CHAN_ENUM_6695] = {6695, 149, 2, 320},
	[CHAN_ENUM_6715] = {6715, 153, 2, 320},
	[CHAN_ENUM_6735] = {6735, 157, 2, 320},
	[CHAN_ENUM_6755] = {6755, 161, 2, 320},
	[CHAN_ENUM_6775] = {6775, 165, 2, 320},
	[CHAN_ENUM_6795] = {6795, 169, 2, 320},
	[CHAN_ENUM_6815] = {6815, 173, 2, 320},
	[CHAN_ENUM_6835] = {6835, 177, 2, 320},
	[CHAN_ENUM_6855] = {6855, 181, 2, 320},
	[CHAN_ENUM_6875] = {6875, 185, 2, 320},
	[CHAN_ENUM_6895] = {6895, 189, 2, 320},
	[CHAN_ENUM_6915] = {6915, 193, 2, 320},
	[CHAN_ENUM_6935] = {6935, 197, 2, 320},
	[CHAN_ENUM_6955] = {6955, 201, 2, 320},
	[CHAN_ENUM_6975] = {6975, 205, 2, 320},
	[CHAN_ENUM_6995] = {6995, 209, 2, 320},
	[CHAN_ENUM_7015] = {7015, 213, 2, 320},
	[CHAN_ENUM_7035] = {7035, 217, 2, 320},
	[CHAN_ENUM_7055] = {7055, 221, 2, 320},
	[CHAN_ENUM_7075] = {7075, 225, 2, 160},
	[CHAN_ENUM_7095] = {7095, 229, 2, 160},
	[CHAN_ENUM_7115] = {7115, 233, 2, 160}
#endif /* CONFIG_BAND_6GHZ */
};

const struct chan_map channel_map_global[NUM_CHANNELS] = {
	[CHAN_ENUM_2412] = {2412, 1, 20, 40},
	[CHAN_ENUM_2417] = {2417, 2, 20, 40},
	[CHAN_ENUM_2422] = {2422, 3, 20, 40},
	[CHAN_ENUM_2427] = {2427, 4, 20, 40},
	[CHAN_ENUM_2432] = {2432, 5, 20, 40},
	[CHAN_ENUM_2437] = {2437, 6, 20, 40},
	[CHAN_ENUM_2442] = {2442, 7, 20, 40},
	[CHAN_ENUM_2447] = {2447, 8, 20, 40},
	[CHAN_ENUM_2452] = {2452, 9, 20, 40},
	[CHAN_ENUM_2457] = {2457, 10, 20, 40},
	[CHAN_ENUM_2462] = {2462, 11, 20, 40},
	[CHAN_ENUM_2467] = {2467, 12, 20, 40},
	[CHAN_ENUM_2472] = {2472, 13, 20, 40},
	[CHAN_ENUM_2484] = {2484, 14, 20, 20},
#ifdef CONFIG_49GHZ_CHAN
	[CHAN_ENUM_4912] = {4912, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4915] = {4915, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4917] = {4917, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4920] = {4920, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4922] = {4922, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4925] = {4925, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4927] = {4927, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4932] = {4932, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4935] = {4935, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4937] = {4937, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4940] = {4940, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4942] = {4942, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4945] = {4945, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4947] = {4947, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4950] = {4950, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4952] = {4952, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4955] = {4955, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4957] = {4957, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4960] = {4960, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4962] = {4962, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4965] = {4965, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4967] = {4967, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4970] = {4970, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4972] = {4972, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4975] = {4975, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4977] = {4977, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4980] = {4980, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4982] = {4982, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4985] = {4985, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4987] = {4987, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5032] = {5032, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5035] = {5035, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5037] = {5037, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5040] = {5040, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5042] = {5042, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5045] = {5045, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5047] = {5047, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5052] = {5052, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5055] = {5055, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5057] = {5057, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5060] = {5060, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5080] = {5080, INVALID_CHANNEL_NUM, 2, 20},
#endif /* CONFIG_49GHZ_CHAN */
	[CHAN_ENUM_5180] = {5180, 36, 2, 160},
	[CHAN_ENUM_5200] = {5200, 40, 2, 160},
	[CHAN_ENUM_5220] = {5220, 44, 2, 160},
	[CHAN_ENUM_5240] = {5240, 48, 2, 160},
	[CHAN_ENUM_5260] = {5260, 52, 2, 160},
	[CHAN_ENUM_5280] = {5280, 56, 2, 160},
	[CHAN_ENUM_5300] = {5300, 60, 2, 160},
	[CHAN_ENUM_5320] = {5320, 64, 2, 160},
	[CHAN_ENUM_5500] = {5500, 100, 2, 240},
	[CHAN_ENUM_5520] = {5520, 104, 2, 240},
	[CHAN_ENUM_5540] = {5540, 108, 2, 240},
	[CHAN_ENUM_5560] = {5560, 112, 2, 240},
	[CHAN_ENUM_5580] = {5580, 116, 2, 240},
	[CHAN_ENUM_5600] = {5600, 120, 2, 240},
	[CHAN_ENUM_5620] = {5620, 124, 2, 240},
	[CHAN_ENUM_5640] = {5640, 128, 2, 240},
	[CHAN_ENUM_5660] = {5660, 132, 2, 240},
	[CHAN_ENUM_5680] = {5680, 136, 2, 240},
	[CHAN_ENUM_5700] = {5700, 140, 2, 240},
	[CHAN_ENUM_5720] = {5720, 144, 2, 240},
	[CHAN_ENUM_5745] = {5745, 149, 2, 160},
	[CHAN_ENUM_5765] = {5765, 153, 2, 160},
	[CHAN_ENUM_5785] = {5785, 157, 2, 160},
	[CHAN_ENUM_5805] = {5805, 161, 2, 160},
	[CHAN_ENUM_5825] = {5825, 165, 2, 160},
	[CHAN_ENUM_5845] = {5845, 169, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5850] = {5850, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5855] = {5855, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5860] = {5860, INVALID_CHANNEL_NUM, 2, 160},
#endif
	[CHAN_ENUM_5865] = {5865, 173, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5870] = {5870, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5875] = {5875, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5880] = {5880, INVALID_CHANNEL_NUM, 2, 160},
#endif
	[CHAN_ENUM_5885] = {5885, INVALID_CHANNEL_NUM, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5890] = {5890, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5895] = {5895, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5900] = {5900, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5905] = {5905, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5910] = {5910, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5915] = {5915, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5920] = {5920, INVALID_CHANNEL_NUM, 2, 160},
#endif /* WLAN_FEATURE_DSRC */
#ifdef CONFIG_BAND_6GHZ
	[CHAN_ENUM_5935] = {5935, 2, 2, 20},
	[CHAN_ENUM_5955] = {5955, 1, 2, 320},
	[CHAN_ENUM_5975] = {5975, 5, 2, 320},
	[CHAN_ENUM_5995] = {5995, 9, 2, 320},
	[CHAN_ENUM_6015] = {6015, 13, 2, 320},
	[CHAN_ENUM_6035] = {6035, 17, 2, 320},
	[CHAN_ENUM_6055] = {6055, 21, 2, 320},
	[CHAN_ENUM_6075] = {6075, 25, 2, 320},
	[CHAN_ENUM_6095] = {6095, 29, 2, 320},
	[CHAN_ENUM_6115] = {6115, 33, 2, 320},
	[CHAN_ENUM_6135] = {6135, 37, 2, 320},
	[CHAN_ENUM_6155] = {6155, 41, 2, 320},
	[CHAN_ENUM_6175] = {6175, 45, 2, 320},
	[CHAN_ENUM_6195] = {6195, 49, 2, 320},
	[CHAN_ENUM_6215] = {6215, 53, 2, 320},
	[CHAN_ENUM_6235] = {6235, 57, 2, 320},
	[CHAN_ENUM_6255] = {6255, 61, 2, 320},
	[CHAN_ENUM_6275] = {6275, 65, 2, 320},
	[CHAN_ENUM_6295] = {6295, 69, 2, 320},
	[CHAN_ENUM_6315] = {6315, 73, 2, 320},
	[CHAN_ENUM_6335] = {6335, 77, 2, 320},
	[CHAN_ENUM_6355] = {6355, 81, 2, 320},
	[CHAN_ENUM_6375] = {6375, 85, 2, 320},
	[CHAN_ENUM_6395] = {6395, 89, 2, 320},
	[CHAN_ENUM_6415] = {6415, 93, 2, 320},
	[CHAN_ENUM_6435] = {6435, 97, 2, 320},
	[CHAN_ENUM_6455] = {6455, 101, 2, 320},
	[CHAN_ENUM_6475] = {6475, 105, 2, 320},
	[CHAN_ENUM_6495] = {6495, 109, 2, 320},
	[CHAN_ENUM_6515] = {6515, 113, 2, 320},
	[CHAN_ENUM_6535] = {6535, 117, 2, 320},
	[CHAN_ENUM_6555] = {6555, 121, 2, 320},
	[CHAN_ENUM_6575] = {6575, 125, 2, 320},
	[CHAN_ENUM_6595] = {6595, 129, 2, 320},
	[CHAN_ENUM_6615] = {6615, 133, 2, 320},
	[CHAN_ENUM_6635] = {6635, 137, 2, 320},
	[CHAN_ENUM_6655] = {6655, 141, 2, 320},
	[CHAN_ENUM_6675] = {6675, 145, 2, 320},
	[CHAN_ENUM_6695] = {6695, 149, 2, 320},
	[CHAN_ENUM_6715] = {6715, 153, 2, 320},
	[CHAN_ENUM_6735] = {6735, 157, 2, 320},
	[CHAN_ENUM_6755] = {6755, 161, 2, 320},
	[CHAN_ENUM_6775] = {6775, 165, 2, 320},
	[CHAN_ENUM_6795] = {6795, 169, 2, 320},
	[CHAN_ENUM_6815] = {6815, 173, 2, 320},
	[CHAN_ENUM_6835] = {6835, 177, 2, 320},
	[CHAN_ENUM_6855] = {6855, 181, 2, 320},
	[CHAN_ENUM_6875] = {6875, 185, 2, 320},
	[CHAN_ENUM_6895] = {6895, 189, 2, 320},
	[CHAN_ENUM_6915] = {6915, 193, 2, 320},
	[CHAN_ENUM_6935] = {6935, 197, 2, 320},
	[CHAN_ENUM_6955] = {6955, 201, 2, 320},
	[CHAN_ENUM_6975] = {6975, 205, 2, 320},
	[CHAN_ENUM_6995] = {6995, 209, 2, 320},
	[CHAN_ENUM_7015] = {7015, 213, 2, 320},
	[CHAN_ENUM_7035] = {7035, 217, 2, 320},
	[CHAN_ENUM_7055] = {7055, 221, 2, 320},
	[CHAN_ENUM_7075] = {7075, 225, 2, 160},
	[CHAN_ENUM_7095] = {7095, 229, 2, 160},
	[CHAN_ENUM_7115] = {7115, 233, 2, 160}
#endif /* CONFIG_BAND_6GHZ */
};

const struct chan_map channel_map_china[NUM_CHANNELS] = {
	[CHAN_ENUM_2412] = {2412, 1, 20, 40},
	[CHAN_ENUM_2417] = {2417, 2, 20, 40},
	[CHAN_ENUM_2422] = {2422, 3, 20, 40},
	[CHAN_ENUM_2427] = {2427, 4, 20, 40},
	[CHAN_ENUM_2432] = {2432, 5, 20, 40},
	[CHAN_ENUM_2437] = {2437, 6, 20, 40},
	[CHAN_ENUM_2442] = {2442, 7, 20, 40},
	[CHAN_ENUM_2447] = {2447, 8, 20, 40},
	[CHAN_ENUM_2452] = {2452, 9, 20, 40},
	[CHAN_ENUM_2457] = {2457, 10, 20, 40},
	[CHAN_ENUM_2462] = {2462, 11, 20, 40},
	[CHAN_ENUM_2467] = {2467, 12, 20, 40},
	[CHAN_ENUM_2472] = {2472, 13, 20, 40},
	[CHAN_ENUM_2484] = {2484, 14, 20, 20},
#ifdef CONFIG_49GHZ_CHAN
	[CHAN_ENUM_4912] = {4912, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4915] = {4915, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4917] = {4917, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4920] = {4920, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4922] = {4922, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4925] = {4925, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4927] = {4927, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4932] = {4932, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4935] = {4935, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4937] = {4937, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4940] = {4940, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4942] = {4942, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4945] = {4945, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4947] = {4947, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4950] = {4950, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4952] = {4952, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4955] = {4955, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4957] = {4957, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4960] = {4960, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4962] = {4962, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4965] = {4965, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4967] = {4967, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4970] = {4970, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4972] = {4972, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4975] = {4975, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4977] = {4977, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4980] = {4980, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4982] = {4982, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4985] = {4985, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_4987] = {4987, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5032] = {5032, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5035] = {5035, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5037] = {5037, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5040] = {5040, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5042] = {5042, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5045] = {5045, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5047] = {5047, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5052] = {5052, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5055] = {5055, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5057] = {5057, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5060] = {5060, INVALID_CHANNEL_NUM, 2, 20},
	[CHAN_ENUM_5080] = {5080, INVALID_CHANNEL_NUM, 2, 20},
#endif /* CONFIG_49GHZ_CHAN */
	[CHAN_ENUM_5180] = {5180, 36, 2, 160},
	[CHAN_ENUM_5200] = {5200, 40, 2, 160},
	[CHAN_ENUM_5220] = {5220, 44, 2, 160},
	[CHAN_ENUM_5240] = {5240, 48, 2, 160},
	[CHAN_ENUM_5260] = {5260, 52, 2, 160},
	[CHAN_ENUM_5280] = {5280, 56, 2, 160},
	[CHAN_ENUM_5300] = {5300, 60, 2, 160},
	[CHAN_ENUM_5320] = {5320, 64, 2, 160},
	[CHAN_ENUM_5500] = {5500, 100, 2, 240},
	[CHAN_ENUM_5520] = {5520, 104, 2, 240},
	[CHAN_ENUM_5540] = {5540, 108, 2, 240},
	[CHAN_ENUM_5560] = {5560, 112, 2, 240},
	[CHAN_ENUM_5580] = {5580, 116, 2, 240},
	[CHAN_ENUM_5600] = {5600, 120, 2, 240},
	[CHAN_ENUM_5620] = {5620, 124, 2, 240},
	[CHAN_ENUM_5640] = {5640, 128, 2, 240},
	[CHAN_ENUM_5660] = {5660, 132, 2, 240},
	[CHAN_ENUM_5680] = {5680, 136, 2, 240},
	[CHAN_ENUM_5700] = {5700, 140, 2, 240},
	[CHAN_ENUM_5720] = {5720, 144, 2, 240},
	[CHAN_ENUM_5745] = {5745, 149, 2, 160},
	[CHAN_ENUM_5765] = {5765, 153, 2, 160},
	[CHAN_ENUM_5785] = {5785, 157, 2, 160},
	[CHAN_ENUM_5805] = {5805, 161, 2, 160},
	[CHAN_ENUM_5825] = {5825, 165, 2, 160},
	[CHAN_ENUM_5845] = {5845, 169, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5850] = {5850, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5855] = {5855, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5860] = {5860, INVALID_CHANNEL_NUM, 2, 160},
#endif
	[CHAN_ENUM_5865] = {5865, INVALID_CHANNEL_NUM, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5870] = {5870, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5875] = {5875, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5880] = {5880, INVALID_CHANNEL_NUM, 2, 160},
#endif
	[CHAN_ENUM_5885] = {5885, INVALID_CHANNEL_NUM, 2, 160},
#ifdef WLAN_FEATURE_DSRC
	[CHAN_ENUM_5890] = {5890, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5895] = {5895, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5900] = {5900, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5905] = {5905, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5910] = {5910, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5915] = {5915, INVALID_CHANNEL_NUM, 2, 160},
	[CHAN_ENUM_5920] = {5920, INVALID_CHANNEL_NUM, 2, 160},
#endif /* WLAN_FEATURE_DSRC */
#ifdef CONFIG_BAND_6GHZ
	[CHAN_ENUM_5935] = {5935, 2, 2, 20},
	[CHAN_ENUM_5955] = {5955, 1, 2, 320},
	[CHAN_ENUM_5975] = {5975, 5, 2, 320},
	[CHAN_ENUM_5995] = {5995, 9, 2, 320},
	[CHAN_ENUM_6015] = {6015, 13, 2, 320},
	[CHAN_ENUM_6035] = {6035, 17, 2, 320},
	[CHAN_ENUM_6055] = {6055, 21, 2, 320},
	[CHAN_ENUM_6075] = {6075, 25, 2, 320},
	[CHAN_ENUM_6095] = {6095, 29, 2, 320},
	[CHAN_ENUM_6115] = {6115, 33, 2, 320},
	[CHAN_ENUM_6135] = {6135, 37, 2, 320},
	[CHAN_ENUM_6155] = {6155, 41, 2, 320},
	[CHAN_ENUM_6175] = {6175, 45, 2, 320},
	[CHAN_ENUM_6195] = {6195, 49, 2, 320},
	[CHAN_ENUM_6215] = {6215, 53, 2, 320},
	[CHAN_ENUM_6235] = {6235, 57, 2, 320},
	[CHAN_ENUM_6255] = {6255, 61, 2, 320},
	[CHAN_ENUM_6275] = {6275, 65, 2, 320},
	[CHAN_ENUM_6295] = {6295, 69, 2, 320},
	[CHAN_ENUM_6315] = {6315, 73, 2, 320},
	[CHAN_ENUM_6335] = {6335, 77, 2, 320},
	[CHAN_ENUM_6355] = {6355, 81, 2, 320},
	[CHAN_ENUM_6375] = {6375, 85, 2, 320},
	[CHAN_ENUM_6395] = {6395, 89, 2, 320},
	[CHAN_ENUM_6415] = {6415, 93, 2, 320},
	[CHAN_ENUM_6435] = {6435, 97, 2, 320},
	[CHAN_ENUM_6455] = {6455, 101, 2, 320},
	[CHAN_ENUM_6475] = {6475, 105, 2, 320},
	[CHAN_ENUM_6495] = {6495, 109, 2, 320},
	[CHAN_ENUM_6515] = {6515, 113, 2, 320},
	[CHAN_ENUM_6535] = {6535, 117, 2, 320},
	[CHAN_ENUM_6555] = {6555, 121, 2, 320},
	[CHAN_ENUM_6575] = {6575, 125, 2, 320},
	[CHAN_ENUM_6595] = {6595, 129, 2, 320},
	[CHAN_ENUM_6615] = {6615, 133, 2, 320},
	[CHAN_ENUM_6635] = {6635, 137, 2, 320},
	[CHAN_ENUM_6655] = {6655, 141, 2, 320},
	[CHAN_ENUM_6675] = {6675, 145, 2, 320},
	[CHAN_ENUM_6695] = {6695, 149, 2, 320},
	[CHAN_ENUM_6715] = {6715, 153, 2, 320},
	[CHAN_ENUM_6735] = {6735, 157, 2, 320},
	[CHAN_ENUM_6755] = {6755, 161, 2, 320},
	[CHAN_ENUM_6775] = {6775, 165, 2, 320},
	[CHAN_ENUM_6795] = {6795, 169, 2, 320},
	[CHAN_ENUM_6815] = {6815, 173, 2, 320},
	[CHAN_ENUM_6835] = {6835, 177, 2, 320},
	[CHAN_ENUM_6855] = {6855, 181, 2, 320},
	[CHAN_ENUM_6875] = {6875, 185, 2, 320},
	[CHAN_ENUM_6895] = {6895, 189, 2, 320},
	[CHAN_ENUM_6915] = {6915, 193, 2, 320},
	[CHAN_ENUM_6935] = {6935, 197, 2, 320},
	[CHAN_ENUM_6955] = {6955, 201, 2, 320},
	[CHAN_ENUM_6975] = {6975, 205, 2, 320},
	[CHAN_ENUM_6995] = {6995, 209, 2, 320},
	[CHAN_ENUM_7015] = {7015, 213, 2, 320},
	[CHAN_ENUM_7035] = {7035, 217, 2, 320},
	[CHAN_ENUM_7055] = {7055, 221, 2, 320},
	[CHAN_ENUM_7075] = {7075, 225, 2, 160},
	[CHAN_ENUM_7095] = {7095, 229, 2, 160},
	[CHAN_ENUM_7115] = {7115, 233, 2, 160}
#endif /* CONFIG_BAND_6GHZ */
};

void reg_init_channel_map(enum dfs_reg dfs_region)
{
	switch (dfs_region) {
	case DFS_UNINIT_REGION:
	case DFS_UNDEF_REGION:
		channel_map = channel_map_global;
		break;
	case DFS_FCC_REGION:
		channel_map = channel_map_us;
		break;
	case DFS_ETSI_REGION:
		channel_map = channel_map_eu;
		break;
	case DFS_MKK_REGION:
	case DFS_MKKN_REGION:
		channel_map = channel_map_jp;
		break;
	case DFS_CN_REGION:
		channel_map = channel_map_china;
		break;
	case DFS_KR_REGION:
		channel_map = channel_map_global;
		break;
	}
}

#ifdef WLAN_FEATURE_11BE
uint16_t reg_get_bw_value(enum phy_ch_width bw)
{
	switch (bw) {
	case CH_WIDTH_20MHZ:
		return 20;
	case CH_WIDTH_40MHZ:
		return 40;
	case CH_WIDTH_80MHZ:
		return 80;
	case CH_WIDTH_160MHZ:
		return 160;
	case CH_WIDTH_80P80MHZ:
		return 160;
	case CH_WIDTH_INVALID:
		return 0;
	case CH_WIDTH_5MHZ:
		return 5;
	case CH_WIDTH_10MHZ:
		return 10;
	case CH_WIDTH_320MHZ:
	case CH_WIDTH_MAX:
		return 320;
	default:
		return 0;
	}
}
#else
uint16_t reg_get_bw_value(enum phy_ch_width bw)
{
	switch (bw) {
	case CH_WIDTH_20MHZ:
		return 20;
	case CH_WIDTH_40MHZ:
		return 40;
	case CH_WIDTH_80MHZ:
		return 80;
	case CH_WIDTH_160MHZ:
		return 160;
	case CH_WIDTH_80P80MHZ:
		return 160;
	case CH_WIDTH_INVALID:
		return 0;
	case CH_WIDTH_5MHZ:
		return 5;
	case CH_WIDTH_10MHZ:
		return 10;
	case CH_WIDTH_MAX:
		return 160;
	default:
		return 0;
	}
}
#endif

struct wlan_lmac_if_reg_tx_ops *reg_get_psoc_tx_ops(
		struct wlan_objmgr_psoc *psoc)
{
	struct wlan_lmac_if_tx_ops *tx_ops;

	tx_ops = wlan_psoc_get_lmac_if_txops(psoc);
	if (!tx_ops) {
		reg_err("tx_ops is NULL");
		return NULL;
	}

	return &tx_ops->reg_ops;
}

/**
 * reg_combine_channel_states() - Get minimum of channel state1 and state2
 * @chan_state1: Channel state1
 * @chan_state2: Channel state2
 *
 * Return: Channel state
 */
enum channel_state reg_combine_channel_states(enum channel_state chan_state1,
					      enum channel_state chan_state2)
{
	if ((chan_state1 == CHANNEL_STATE_INVALID) ||
	    (chan_state2 == CHANNEL_STATE_INVALID))
		return CHANNEL_STATE_INVALID;
	else
		return min(chan_state1, chan_state2);
}

QDF_STATUS reg_read_default_country(struct wlan_objmgr_psoc *psoc,
				    uint8_t *country_code)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	if (!country_code) {
		reg_err("country_code is NULL");
		return QDF_STATUS_E_INVAL;
	}

	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_INVAL;
	}

	qdf_mem_copy(country_code, psoc_priv_obj->def_country,
		     REG_ALPHA2_LEN + 1);

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_get_max_5g_bw_from_country_code(uint16_t cc,
					       uint16_t *max_bw_5g)
{
	uint16_t i;
	int num_countries;

	*max_bw_5g = 0;
	reg_get_num_countries(&num_countries);

	for (i = 0; i < num_countries; i++) {
		if (g_all_countries[i].country_code == cc)
			break;
	}

	if (i == num_countries)
		return QDF_STATUS_E_FAILURE;

	*max_bw_5g = g_all_countries[i].max_bw_5g;

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_get_max_5g_bw_from_regdomain(uint16_t regdmn,
					    uint16_t *max_bw_5g)
{
	uint16_t i;
	int num_reg_dmn;

	*max_bw_5g = 0;
	reg_get_num_reg_dmn_pairs(&num_reg_dmn);

	for (i = 0; i < num_reg_dmn; i++) {
		if (g_reg_dmn_pairs[i].reg_dmn_pair_id == regdmn)
			break;
	}

	if (i == num_reg_dmn)
		return QDF_STATUS_E_FAILURE;

	*max_bw_5g = regdomains_5g[g_reg_dmn_pairs[i].dmn_id_5g].max_bw;

	return QDF_STATUS_SUCCESS;
}

void reg_get_current_dfs_region(struct wlan_objmgr_pdev *pdev,
				enum dfs_reg *dfs_reg)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg component pdev priv is NULL");
		return;
	}

	*dfs_reg = pdev_priv_obj->dfs_region;
}

void reg_set_dfs_region(struct wlan_objmgr_pdev *pdev,
			enum dfs_reg dfs_reg)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return;
	}

	pdev_priv_obj->dfs_region = dfs_reg;

	reg_init_channel_map(dfs_reg);
}

uint8_t reg_freq_to_chan(struct wlan_objmgr_pdev *pdev,
			 qdf_freq_t freq)
{
	uint32_t count;
	struct regulatory_channel *chan_list;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	if (freq == 0) {
		reg_err_rl("Invalid freq %d", freq);
		return 0;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return 0;
	}

	chan_list = pdev_priv_obj->mas_chan_list;
	for (count = 0; count < NUM_CHANNELS; count++) {
		if (chan_list[count].center_freq >= freq)
			break;
	}

	if (count == NUM_CHANNELS)
		goto end;

	if (chan_list[count].center_freq == freq)
		return chan_list[count].chan_num;

	if (count == 0)
		goto end;

	if ((chan_list[count - 1].chan_num == INVALID_CHANNEL_NUM) ||
	    (chan_list[count].chan_num == INVALID_CHANNEL_NUM)) {
		reg_err("Frequency %d invalid in current reg domain", freq);
		return 0;
	}

	return (chan_list[count - 1].chan_num +
		(freq - chan_list[count - 1].center_freq) / 5);

end:
	reg_err_rl("invalid frequency %d", freq);
	return 0;
}

static uint16_t reg_compute_chan_to_freq(struct wlan_objmgr_pdev *pdev,
					 uint8_t chan_num,
					 enum channel_enum min_chan_range,
					 enum channel_enum max_chan_range)
{
	uint16_t count;
	struct regulatory_channel *chan_list;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return 0;
	}

	if (min_chan_range < MIN_CHANNEL || max_chan_range > MAX_CHANNEL) {
		reg_err_rl("Channel range is invalid");
		return 0;
	}

	chan_list = pdev_priv_obj->mas_chan_list;

	for (count = min_chan_range; count <= max_chan_range; count++) {
		if ((chan_list[count].state != CHANNEL_STATE_DISABLE) &&
		    !(chan_list[count].chan_flags & REGULATORY_CHAN_DISABLED)) {
			if (REG_IS_49GHZ_FREQ(chan_list[count].center_freq)) {
				if (chan_list[count].chan_num == chan_num)
					break;
				continue;
			} else if ((chan_list[count].chan_num >= chan_num) &&
				   (chan_list[count].chan_num !=
							INVALID_CHANNEL_NUM))
				break;
		}
	}

	if (count == max_chan_range + 1)
		goto end;

	if (chan_list[count].chan_num == chan_num) {
		if (chan_list[count].chan_flags & REGULATORY_CHAN_DISABLED)
			reg_err("Channel %d disabled in current reg domain",
				chan_num);
		return chan_list[count].center_freq;
	}

	if (count == min_chan_range)
		goto end;

	if ((chan_list[count - 1].chan_num == INVALID_CHANNEL_NUM) ||
	    REG_IS_49GHZ_FREQ(chan_list[count - 1].center_freq) ||
	    (chan_list[count].chan_num == INVALID_CHANNEL_NUM)) {
		reg_err("Channel %d invalid in current reg domain",
			chan_num);
		return 0;
	}

	return (chan_list[count - 1].center_freq +
		(chan_num - chan_list[count - 1].chan_num) * 5);

end:

	reg_debug_rl("Invalid channel %d", chan_num);
	return 0;
}

uint16_t reg_legacy_chan_to_freq(struct wlan_objmgr_pdev *pdev,
				 uint8_t chan_num)
{
	uint16_t min_chan_range = MIN_24GHZ_CHANNEL;
	uint16_t max_chan_range = MAX_5GHZ_CHANNEL;

	if (chan_num == 0) {
		reg_err_rl("Invalid channel %d", chan_num);
		return 0;
	}

	return reg_compute_chan_to_freq(pdev, chan_num,
					min_chan_range,
					max_chan_range);
}

QDF_STATUS reg_program_default_cc(struct wlan_objmgr_pdev *pdev,
				  uint16_t regdmn)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	struct cur_regulatory_info *reg_info;
	uint16_t cc = -1;
	uint16_t country_index = -1, regdmn_pair = -1;
	struct wlan_objmgr_psoc *psoc;
	QDF_STATUS err;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!pdev_priv_obj) {
		reg_err("reg soc is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	reg_info = (struct cur_regulatory_info *)qdf_mem_malloc
		(sizeof(struct cur_regulatory_info));
	if (!reg_info)
		return QDF_STATUS_E_NOMEM;

	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc) {
		reg_err("psoc is NULL");
		return QDF_STATUS_E_INVAL;
	}

	reg_info->psoc = psoc;
	reg_info->phy_id = wlan_objmgr_pdev_get_pdev_id(pdev);
	reg_info->num_phy = 1;

	if (regdmn == 0) {
		reg_get_default_country(&regdmn);
		regdmn |= COUNTRY_ERD_FLAG;
	}

	if (regdmn & COUNTRY_ERD_FLAG) {
		cc = regdmn & ~COUNTRY_ERD_FLAG;

		reg_get_rdpair_from_country_code(cc,
						 &country_index,
						 &regdmn_pair);

		err = reg_get_cur_reginfo(reg_info, country_index, regdmn_pair);
		if (err == QDF_STATUS_E_FAILURE) {
			reg_err("Unable to set country code\n");
			qdf_mem_free(reg_info->reg_rules_2g_ptr);
			qdf_mem_free(reg_info->reg_rules_5g_ptr);
			qdf_mem_free(reg_info);
			return QDF_STATUS_E_FAILURE;
		}

		pdev_priv_obj->ctry_code = cc;

	} else {
		reg_get_rdpair_from_regdmn_id(regdmn, &regdmn_pair);

		err = reg_get_cur_reginfo(reg_info, country_index, regdmn_pair);
		if (err == QDF_STATUS_E_FAILURE) {
			reg_err("Unable to set country code\n");
			qdf_mem_free(reg_info->reg_rules_2g_ptr);
			qdf_mem_free(reg_info->reg_rules_5g_ptr);
			qdf_mem_free(reg_info);
			return QDF_STATUS_E_FAILURE;
		}

		pdev_priv_obj->reg_dmn_pair = regdmn;
	}

	reg_info->offload_enabled = false;
	reg_process_master_chan_list(reg_info);

	qdf_mem_free(reg_info->reg_rules_2g_ptr);
	qdf_mem_free(reg_info->reg_rules_5g_ptr);
	qdf_mem_free(reg_info);

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_program_chan_list(struct wlan_objmgr_pdev *pdev,
				 struct cc_regdmn_s *rd)
{
	struct cur_regulatory_info *reg_info;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	uint16_t country_index = -1, regdmn_pair = -1;
	struct wlan_objmgr_psoc *psoc;
	struct wlan_lmac_if_reg_tx_ops *tx_ops;
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;
	uint8_t pdev_id;
	uint8_t phy_id;
	QDF_STATUS err;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!pdev_priv_obj) {
		reg_err(" pdev priv obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc) {
		reg_err("psoc is NULL");
		return QDF_STATUS_E_INVAL;
	}

	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	if (psoc_priv_obj->offload_enabled) {
		if ((rd->flags == ALPHA_IS_SET) && (rd->cc.alpha[2] == 'O'))
			pdev_priv_obj->indoor_chan_enabled = false;
		else
			pdev_priv_obj->indoor_chan_enabled = true;

		pdev_id = wlan_objmgr_pdev_get_pdev_id(pdev);
		tx_ops = reg_get_psoc_tx_ops(psoc);

		if (tx_ops->get_phy_id_from_pdev_id)
			tx_ops->get_phy_id_from_pdev_id(psoc, pdev_id, &phy_id);
		else
			phy_id = pdev_id;

		if (tx_ops->set_user_country_code) {
			psoc_priv_obj->new_init_ctry_pending[phy_id] = true;
			return tx_ops->set_user_country_code(psoc, pdev_id, rd);
		}

		return QDF_STATUS_E_FAILURE;
	}

	reg_info = (struct cur_regulatory_info *)qdf_mem_malloc
		(sizeof(struct cur_regulatory_info));
	if (!reg_info)
		return QDF_STATUS_E_NOMEM;

	reg_info->psoc = psoc;
	reg_info->phy_id = wlan_objmgr_pdev_get_pdev_id(pdev);

	if (rd->flags == CC_IS_SET) {
		reg_get_rdpair_from_country_code(rd->cc.country_code,
						 &country_index,
						 &regdmn_pair);
	} else if (rd->flags == ALPHA_IS_SET) {
		reg_get_rdpair_from_country_iso(rd->cc.alpha,
						&country_index,
						&regdmn_pair);
	} else if (rd->flags == REGDMN_IS_SET) {
		reg_get_rdpair_from_regdmn_id(rd->cc.regdmn.reg_2g_5g_pair_id,
					      &regdmn_pair);
	}

	err = reg_get_cur_reginfo(reg_info, country_index, regdmn_pair);
	if (err == QDF_STATUS_E_FAILURE) {
		reg_err("Unable to set country code\n");
		qdf_mem_free(reg_info->reg_rules_2g_ptr);
		qdf_mem_free(reg_info->reg_rules_5g_ptr);
		qdf_mem_free(reg_info);
		return QDF_STATUS_E_FAILURE;
	}

	reg_info->offload_enabled = false;
	reg_process_master_chan_list(reg_info);

	qdf_mem_free(reg_info->reg_rules_2g_ptr);
	qdf_mem_free(reg_info->reg_rules_5g_ptr);
	qdf_mem_free(reg_info);

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_get_current_cc(struct wlan_objmgr_pdev *pdev,
			      struct cc_regdmn_s *rd)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!pdev_priv_obj) {
		reg_err("reg pdev priv is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	if (rd->flags == CC_IS_SET) {
		rd->cc.country_code = pdev_priv_obj->ctry_code;
	} else if (rd->flags == ALPHA_IS_SET) {
		qdf_mem_copy(rd->cc.alpha, pdev_priv_obj->current_country,
			     sizeof(rd->cc.alpha));
	} else if (rd->flags == REGDMN_IS_SET) {
		rd->cc.regdmn.reg_2g_5g_pair_id = pdev_priv_obj->reg_dmn_pair;
		rd->cc.regdmn.sixg_superdmn_id = pdev_priv_obj->reg_6g_superid;
	}

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_set_regdb_offloaded(struct wlan_objmgr_psoc *psoc, bool val)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	psoc_priv_obj->offload_enabled = val;

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_get_curr_regdomain(struct wlan_objmgr_pdev *pdev,
				  struct cur_regdmn_info *cur_regdmn)
{
	struct wlan_objmgr_psoc *psoc;
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;
	uint16_t index;
	int num_reg_dmn;
	uint8_t phy_id;
	uint8_t pdev_id;
	struct wlan_lmac_if_reg_tx_ops *tx_ops;

	psoc = wlan_pdev_get_psoc(pdev);
	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("soc reg component is NULL");
		return QDF_STATUS_E_INVAL;
	}

	pdev_id = wlan_objmgr_pdev_get_pdev_id(pdev);

	tx_ops = reg_get_psoc_tx_ops(psoc);
	if (tx_ops->get_phy_id_from_pdev_id)
		tx_ops->get_phy_id_from_pdev_id(psoc, pdev_id, &phy_id);
	else
		phy_id = pdev_id;

	cur_regdmn->regdmn_pair_id =
		psoc_priv_obj->mas_chan_params[phy_id].reg_dmn_pair;

	reg_get_num_reg_dmn_pairs(&num_reg_dmn);
	for (index = 0; index < num_reg_dmn; index++) {
		if (g_reg_dmn_pairs[index].reg_dmn_pair_id ==
				cur_regdmn->regdmn_pair_id)
			break;
	}

	if (index == num_reg_dmn) {
		reg_debug_rl("invalid regdomain");
		return QDF_STATUS_E_FAILURE;
	}

	cur_regdmn->dmn_id_2g = g_reg_dmn_pairs[index].dmn_id_2g;
	cur_regdmn->dmn_id_5g = g_reg_dmn_pairs[index].dmn_id_5g;
	cur_regdmn->ctl_2g = regdomains_2g[cur_regdmn->dmn_id_2g].ctl_val;
	cur_regdmn->ctl_5g = regdomains_5g[cur_regdmn->dmn_id_5g].ctl_val;
	cur_regdmn->dfs_region =
		regdomains_5g[cur_regdmn->dmn_id_5g].dfs_region;

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_modify_chan_144(struct wlan_objmgr_pdev *pdev,
			       bool enable_ch_144)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	struct wlan_objmgr_psoc *psoc;
	struct wlan_lmac_if_reg_tx_ops *reg_tx_ops;
	QDF_STATUS status;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_INVAL;
	}

	if (pdev_priv_obj->en_chan_144 == enable_ch_144) {
		reg_info("chan 144 is already  %d", enable_ch_144);
		return QDF_STATUS_SUCCESS;
	}

	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc) {
		reg_err("psoc is NULL");
		return QDF_STATUS_E_INVAL;
	}

	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_INVAL;
	}

	reg_debug("setting chan 144: %d", enable_ch_144);
	pdev_priv_obj->en_chan_144 = enable_ch_144;

	reg_compute_pdev_current_chan_list(pdev_priv_obj);

	reg_tx_ops = reg_get_psoc_tx_ops(psoc);
	if (reg_tx_ops->fill_umac_legacy_chanlist)
		reg_tx_ops->fill_umac_legacy_chanlist(pdev,
				pdev_priv_obj->cur_chan_list);

	status = reg_send_scheduler_msg_sb(psoc, pdev);

	return status;
}

bool reg_get_en_chan_144(struct wlan_objmgr_pdev *pdev)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return false;
	}

	return pdev_priv_obj->en_chan_144;
}

struct wlan_psoc_host_hal_reg_capabilities_ext *reg_get_hal_reg_cap(
						struct wlan_objmgr_psoc *psoc)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return NULL;
	}

	return psoc_priv_obj->reg_cap;
}

QDF_STATUS reg_set_hal_reg_cap(
		struct wlan_objmgr_psoc *psoc,
		struct wlan_psoc_host_hal_reg_capabilities_ext *reg_cap,
		uint16_t phy_cnt)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	if (phy_cnt > PSOC_MAX_PHY_REG_CAP) {
		reg_err("phy cnt:%d is more than %d", phy_cnt,
			PSOC_MAX_PHY_REG_CAP);
		return QDF_STATUS_E_FAILURE;
	}

	qdf_mem_copy(psoc_priv_obj->reg_cap, reg_cap,
		     phy_cnt *
		     sizeof(struct wlan_psoc_host_hal_reg_capabilities_ext));

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_update_hal_reg_cap(struct wlan_objmgr_psoc *psoc,
				  uint64_t wireless_modes, uint8_t phy_id)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	if (!psoc) {
		reg_err("psoc is null");
		return QDF_STATUS_E_FAILURE;
	}

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	psoc_priv_obj->reg_cap[phy_id].wireless_modes |= wireless_modes;

	return QDF_STATUS_SUCCESS;
}

bool reg_chan_in_range(struct regulatory_channel *chan_list,
		       qdf_freq_t low_freq_2g, qdf_freq_t high_freq_2g,
		       qdf_freq_t low_freq_5g, qdf_freq_t high_freq_5g,
		       enum channel_enum ch_enum)
{
	uint32_t low_limit_2g = NUM_CHANNELS;
	uint32_t high_limit_2g = NUM_CHANNELS;
	uint32_t low_limit_5g = NUM_CHANNELS;
	uint32_t high_limit_5g = NUM_CHANNELS;
	bool chan_in_range;
	enum channel_enum chan_enum;
	uint16_t min_bw;
	qdf_freq_t center_freq;

	for (chan_enum = 0; chan_enum < NUM_CHANNELS; chan_enum++) {
		min_bw = chan_list[chan_enum].min_bw;
		center_freq = chan_list[chan_enum].center_freq;

		if ((center_freq - min_bw / 2) >= low_freq_2g) {
			low_limit_2g = chan_enum;
			break;
		}
	}

	for (chan_enum = 0; chan_enum < NUM_CHANNELS; chan_enum++) {
		min_bw = chan_list[chan_enum].min_bw;
		center_freq = chan_list[chan_enum].center_freq;

		if ((center_freq - min_bw / 2) >= low_freq_5g) {
			low_limit_5g = chan_enum;
			break;
		}
	}

	for (chan_enum = NUM_CHANNELS - 1; chan_enum >= 0; chan_enum--) {
		min_bw = chan_list[chan_enum].min_bw;
		center_freq = chan_list[chan_enum].center_freq;

		if (center_freq + min_bw / 2 <= high_freq_2g) {
			high_limit_2g = chan_enum;
			break;
		}
		if (chan_enum == 0)
			break;
	}

	for (chan_enum = NUM_CHANNELS - 1; chan_enum >= 0; chan_enum--) {
		min_bw = chan_list[chan_enum].min_bw;
		center_freq = chan_list[chan_enum].center_freq;

		if (center_freq + min_bw / 2 <= high_freq_5g) {
			high_limit_5g = chan_enum;
			break;
		}
		if (chan_enum == 0)
			break;
	}

	chan_in_range = false;
	if  ((low_limit_2g <= ch_enum) &&
	     (high_limit_2g >= ch_enum) &&
	     (low_limit_2g != NUM_CHANNELS) &&
	     (high_limit_2g != NUM_CHANNELS))
		chan_in_range = true;
	if  ((low_limit_5g <= ch_enum) &&
	     (high_limit_5g >= ch_enum) &&
	     (low_limit_5g != NUM_CHANNELS) &&
	     (high_limit_5g != NUM_CHANNELS))
		chan_in_range = true;

	if (chan_in_range)
		return true;
	else
		return false;
}

bool reg_is_24ghz_ch_freq(uint32_t freq)
{
	return REG_IS_24GHZ_CH_FREQ(freq);
}

bool reg_is_5ghz_ch_freq(uint32_t freq)
{
	return REG_IS_5GHZ_FREQ(freq);
}

/**
 * BAND_2G_PRESENT() - Check if REG_BAND_2G is set in the band_mask
 * @band_mask: Bitmask for bands
 *
 * Return: True if REG_BAND_2G is set in the band_mask, else false
 */
static inline bool BAND_2G_PRESENT(uint8_t band_mask)
{
	return !!(band_mask & (BIT(REG_BAND_2G)));
}

/**
 * BAND_5G_PRESENT() - Check if REG_BAND_5G is set in the band_mask
 * @band_mask: Bitmask for bands
 *
 * Return: True if REG_BAND_5G is set in the band_mask, else false
 */
static inline bool BAND_5G_PRESENT(uint8_t band_mask)
{
	return !!(band_mask & (BIT(REG_BAND_5G)));
}

/**
 * reg_is_freq_in_between() - Check whether freq falls within low_freq and
 * high_freq, inclusively.
 * @low_freq - Low frequency.
 * @high_freq - High frequency.
 * @freq - Frequency to be checked.
 *
 * Return: True if freq falls within low_freq and high_freq, else false.
 */
static bool reg_is_freq_in_between(qdf_freq_t low_freq, qdf_freq_t high_freq,
				   qdf_freq_t freq)
{
	return (low_freq <= freq && freq <= high_freq);
}

static bool reg_is_ranges_overlap(qdf_freq_t low_freq, qdf_freq_t high_freq,
				  qdf_freq_t start_edge_freq,
				  qdf_freq_t end_edge_freq)
{
	return (reg_is_freq_in_between(start_edge_freq,
				       end_edge_freq,
				       low_freq) ||
		reg_is_freq_in_between(start_edge_freq,
				       end_edge_freq,
				       high_freq) ||
		reg_is_freq_in_between(low_freq,
				       high_freq,
				       start_edge_freq) ||
		reg_is_freq_in_between(low_freq,
				       high_freq,
				       end_edge_freq));
}

bool reg_is_range_overlap_2g(qdf_freq_t low_freq, qdf_freq_t high_freq)
{
	return reg_is_ranges_overlap(low_freq, high_freq,
				     TWO_GIG_STARTING_EDGE_FREQ,
				     TWO_GIG_ENDING_EDGE_FREQ);
}

bool reg_is_range_overlap_5g(qdf_freq_t low_freq, qdf_freq_t high_freq)
{
	return reg_is_ranges_overlap(low_freq, high_freq,
				     FIVE_GIG_STARTING_EDGE_FREQ,
				     FIVE_GIG_ENDING_EDGE_FREQ);
}

bool reg_is_freq_indoor(struct wlan_objmgr_pdev *pdev, qdf_freq_t freq)
{
	struct regulatory_channel *cur_chan_list;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	enum channel_enum chan_enum;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return false;
	}

	chan_enum = reg_get_chan_enum_for_freq(freq);

	if (chan_enum == INVALID_CHANNEL) {
		reg_err_rl("Invalid chan enum %d", chan_enum);
		return false;
	}

	cur_chan_list = pdev_priv_obj->cur_chan_list;

	return (cur_chan_list[chan_enum].chan_flags &
		REGULATORY_CHAN_INDOOR_ONLY);
}

#ifdef CONFIG_REG_CLIENT
bool reg_is_freq_indoor_in_secondary_list(struct wlan_objmgr_pdev *pdev,
					  qdf_freq_t freq)
{
	struct regulatory_channel *secondary_cur_chan_list;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	enum channel_enum chan_enum;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return false;
	}

	chan_enum = reg_get_chan_enum_for_freq(freq);

	if (chan_enum == INVALID_CHANNEL) {
		reg_err_rl("Invalid chan enum %d", chan_enum);
		return false;
	}

	secondary_cur_chan_list = pdev_priv_obj->secondary_cur_chan_list;

	return (secondary_cur_chan_list[chan_enum].chan_flags &
		REGULATORY_CHAN_INDOOR_ONLY);
}
#endif

#ifdef CONFIG_BAND_6GHZ
bool reg_is_6ghz_chan_freq(uint16_t freq)
{
	return REG_IS_6GHZ_FREQ(freq);
}

#ifdef CONFIG_6G_FREQ_OVERLAP
bool reg_is_range_overlap_6g(qdf_freq_t low_freq, qdf_freq_t high_freq)
{
	return reg_is_ranges_overlap(low_freq, high_freq,
				     SIX_GIG_STARTING_EDGE_FREQ,
				     SIX_GIG_ENDING_EDGE_FREQ);
}

bool reg_is_range_only6g(qdf_freq_t low_freq, qdf_freq_t high_freq)
{
	if (low_freq >= high_freq) {
		reg_err_rl("Low freq is greater than or equal to high freq");
		return false;
	}

	if (reg_is_range_overlap_6g(low_freq, high_freq) &&
	    !reg_is_range_overlap_5g(low_freq, high_freq)) {
		reg_debug_rl("The device is 6G only");
		return true;
	}

	reg_debug_rl("The device is not 6G only");

	return false;
}
#endif

uint16_t reg_min_6ghz_chan_freq(void)
{
	return REG_MIN_6GHZ_CHAN_FREQ;
}

uint16_t reg_max_6ghz_chan_freq(void)
{
	return REG_MAX_6GHZ_CHAN_FREQ;
}

bool reg_is_6ghz_psc_chan_freq(uint16_t freq)
{
	if (!REG_IS_6GHZ_FREQ(freq)) {
		reg_debug(" Channel frequency is not a 6GHz frequency");
		return false;
	}

	if (!(((freq - SIX_GHZ_NON_ORPHAN_START_FREQ) + FREQ_LEFT_SHIFT) %
	      (FREQ_TO_CHAN_SCALE * NUM_80MHZ_BAND_IN_6G))) {
		return true;
	}

	reg_debug_rl("Channel freq %d MHz is not a 6GHz PSC frequency", freq);

	return false;
}

bool reg_is_6g_freq_indoor(struct wlan_objmgr_pdev *pdev, qdf_freq_t freq)
{
	return (REG_IS_6GHZ_FREQ(freq) && reg_is_freq_indoor(pdev, freq));
}

/**
 * reg_get_max_psd() - Get max PSD.
 * @freq: Channel frequency.
 * @bw: Channel bandwidth.
 * @reg_ap: Regulatory 6G AP type.
 * @reg_client: Regulatory 6G client type.
 * @tx_power: Pointer to tx-power.
 *
 * Return: Return QDF_STATUS_SUCCESS, if PSD is filled for 6G TPE IE
 * else return QDF_STATUS_E_FAILURE.
 */
static QDF_STATUS reg_get_max_psd(qdf_freq_t freq,
				  uint16_t bw,
				  enum reg_6g_ap_type reg_ap,
				  enum reg_6g_client_type reg_client,
				  uint8_t *tx_power)
{
	if (reg_ap == REG_INDOOR_AP) {
		switch (reg_client) {
		case REG_DEFAULT_CLIENT:
			*tx_power = REG_PSD_MAX_TXPOWER_FOR_DEFAULT_CLIENT;
			return QDF_STATUS_SUCCESS;
		case REG_SUBORDINATE_CLIENT:
			*tx_power = REG_PSD_MAX_TXPOWER_FOR_SUBORDINATE_CLIENT;
			return QDF_STATUS_SUCCESS;
		default:
			reg_err_rl("Invalid client type");
			return QDF_STATUS_E_FAILURE;
		}
	}

	return QDF_STATUS_E_FAILURE;
}

/**
 * reg_get_max_txpower_for_eirp() - Get max EIRP.
 * @pdev: Pointer to pdev.
 * @freq: Channel frequency.
 * @bw: Channel bandwidth.
 * @reg_ap: Regulatory 6G AP type.
 * @reg_client: Regulatory client type.
 * @tx_power: Pointer to tx-power.
 *
 * Return: Return QDF_STATUS_SUCCESS, if EIRP is filled for 6G TPE IE
 * else return QDF_STATUS_E_FAILURE.
 */
static QDF_STATUS reg_get_max_eirp(struct wlan_objmgr_pdev *pdev,
				   qdf_freq_t freq,
				   uint16_t bw,
				   enum reg_6g_ap_type reg_ap,
				   enum reg_6g_client_type reg_client,
				   uint8_t *tx_power)
{
	if (reg_ap == REG_INDOOR_AP) {
		switch (reg_client) {
		case REG_DEFAULT_CLIENT:
			*tx_power = reg_get_channel_reg_power_for_freq(pdev,
								       freq);
			return QDF_STATUS_SUCCESS;
		case REG_SUBORDINATE_CLIENT:
			*tx_power = REG_EIRP_MAX_TXPOWER_FOR_SUBORDINATE_CLIENT;
			return QDF_STATUS_SUCCESS;
		default:
			reg_err_rl("Invalid client type");
			return QDF_STATUS_E_FAILURE;
		}
	}

	return QDF_STATUS_E_FAILURE;
}

QDF_STATUS reg_get_max_txpower_for_6g_tpe(struct wlan_objmgr_pdev *pdev,
					  qdf_freq_t freq, uint8_t bw,
					  enum reg_6g_ap_type reg_ap,
					  enum reg_6g_client_type reg_client,
					  bool is_psd,
					  uint8_t *tx_power)
{
	if (!REG_IS_6GHZ_FREQ(freq)) {
		reg_err_rl("%d is not a 6G channel frequency", freq);
		return QDF_STATUS_E_FAILURE;
	}

	/*
	 * For now, there is support only for Indoor AP and we have only
	 * LPI power values.
	 */
	if (is_psd)
		return reg_get_max_psd(freq, bw, reg_ap, reg_client, tx_power);

	return reg_get_max_eirp(pdev, freq, bw, reg_ap, reg_client, tx_power);
}

/**
 * BAND_6G_PRESENT() - Check if REG_BAND_6G is set in the band_mask
 * @band_mask: Bitmask for bands
 *
 * Return: True if REG_BAND_6G is set in the band_mask, else false
 */
static inline bool BAND_6G_PRESENT(uint8_t band_mask)
{
	return !!(band_mask & (BIT(REG_BAND_6G)));
}
#else
static inline bool BAND_6G_PRESENT(uint8_t band_mask)
{
	return false;
}
#endif /* CONFIG_BAND_6GHZ */

/**
 * reg_get_band_from_cur_chan_list() - Get channel list and number of channels
 * @pdev: pdev ptr
 * @band_mask: Input bitmap with band set
 * @channel_list: Pointer to Channel List
 * @cur_chan_list: Pointer to primary current channel list for non-beaconing
 * entites (STA, p2p client) and secondary channel list for beaconing entities
 * (SAP, p2p GO)
 *
 * Get the given channel list and number of channels from the current channel
 * list based on input band bitmap.
 *
 * Return: Number of channels, else 0 to indicate error
 */
static uint16_t
reg_get_band_from_cur_chan_list(struct wlan_objmgr_pdev *pdev,
				uint8_t band_mask,
				struct regulatory_channel *channel_list,
				struct regulatory_channel *cur_chan_list)
{
	uint16_t i, num_channels = 0;

	if (BAND_2G_PRESENT(band_mask)) {
		for (i = MIN_24GHZ_CHANNEL; i <= MAX_24GHZ_CHANNEL; i++) {
			if ((cur_chan_list[i].state != CHANNEL_STATE_DISABLE) &&
			    !(cur_chan_list[i].chan_flags &
			      REGULATORY_CHAN_DISABLED)) {
				channel_list[num_channels] = cur_chan_list[i];
				num_channels++;
			}
		}
	}
	if (BAND_5G_PRESENT(band_mask)) {
		for (i = BAND_5GHZ_START_CHANNEL; i <= MAX_5GHZ_CHANNEL; i++) {
			if ((cur_chan_list[i].state != CHANNEL_STATE_DISABLE) &&
			    !(cur_chan_list[i].chan_flags &
			      REGULATORY_CHAN_DISABLED)) {
				channel_list[num_channels] = cur_chan_list[i];
				num_channels++;
			}
		}
	}
	if (BAND_6G_PRESENT(band_mask)) {
		for (i = MIN_6GHZ_CHANNEL; i <= MAX_6GHZ_CHANNEL; i++) {
			if ((cur_chan_list[i].state != CHANNEL_STATE_DISABLE) &&
			    !(cur_chan_list[i].chan_flags &
			      REGULATORY_CHAN_DISABLED)) {
				channel_list[num_channels] = cur_chan_list[i];
				num_channels++;
			}
		}
	}

	if (!num_channels) {
		reg_err("Failed to retrieve the channel list");
		return 0;
	}

	return num_channels;
}

uint16_t
reg_get_band_channel_list(struct wlan_objmgr_pdev *pdev,
			  uint8_t band_mask,
			  struct regulatory_channel *channel_list)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return 0;
	}

	return reg_get_band_from_cur_chan_list(pdev, band_mask, channel_list,
					       pdev_priv_obj->cur_chan_list);
}

#ifdef CONFIG_REG_CLIENT
uint16_t
reg_get_secondary_band_channel_list(struct wlan_objmgr_pdev *pdev,
				    uint8_t band_mask,
				    struct regulatory_channel *channel_list)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return 0;
	}

	return reg_get_band_from_cur_chan_list(
				pdev, band_mask, channel_list,
				pdev_priv_obj->secondary_cur_chan_list);
}
#endif

qdf_freq_t reg_chan_band_to_freq(struct wlan_objmgr_pdev *pdev,
				 uint8_t chan_num,
				 uint8_t band_mask)
{
	enum channel_enum min_chan, max_chan;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	uint16_t freq;

	if (chan_num == 0) {
		reg_err_rl("Invalid channel %d", chan_num);
		return 0;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return 0;
	}

	if (BAND_6G_PRESENT(band_mask)) {
		if (BAND_2G_PRESENT(band_mask) ||
		    BAND_5G_PRESENT(band_mask)) {
			reg_err_rl("Incorrect band_mask %x", band_mask);
				return 0;
		}

		/* Handle 6G channel 2 as a special case as it does not follow
		 * the regular increasing order of channel numbers
		 */
		if (chan_num == SIXG_CHAN_2) {
			struct regulatory_channel *mas_chan_list;

			mas_chan_list = pdev_priv_obj->mas_chan_list;
			/* Check if chan 2 is in the master list */
			if ((mas_chan_list[CHAN_ENUM_SIXG_2].state !=
			     CHANNEL_STATE_DISABLE) &&
			    !(mas_chan_list[CHAN_ENUM_SIXG_2].chan_flags &
			     REGULATORY_CHAN_DISABLED))
				return mas_chan_list[CHAN_ENUM_SIXG_2].
								center_freq;
			else
				return 0;
		}

		/* MIN_6GHZ_CHANNEL corresponds to CHAN_ENUM_5935
		 * ( a.k.a SIXG_CHAN_2). Skip it from the search space
		 */
		min_chan = MIN_6GHZ_CHANNEL + 1;
		max_chan = MAX_6GHZ_CHANNEL;
		return reg_compute_chan_to_freq(pdev, chan_num,
						min_chan,
						max_chan);
	} else {
		if (BAND_2G_PRESENT(band_mask)) {
			min_chan = MIN_24GHZ_CHANNEL;
			max_chan = MAX_24GHZ_CHANNEL;
			freq = reg_compute_chan_to_freq(pdev, chan_num,
							min_chan,
							max_chan);
			if (freq != 0)
				return freq;
		}

		if (BAND_5G_PRESENT(band_mask)) {
			min_chan = BAND_5GHZ_START_CHANNEL;
			max_chan = MAX_5GHZ_CHANNEL;

			return reg_compute_chan_to_freq(pdev, chan_num,
							min_chan,
							max_chan);
		}

		reg_err_rl("Incorrect band_mask %x", band_mask);
		return 0;
	}
}

#ifdef CONFIG_49GHZ_CHAN
bool reg_is_49ghz_freq(qdf_freq_t freq)
{
	return REG_IS_49GHZ_FREQ(freq);
}
#endif /* CONFIG_49GHZ_CHAN */

qdf_freq_t reg_ch_num(uint32_t ch_enum)
{
	return REG_CH_NUM(ch_enum);
}

qdf_freq_t reg_ch_to_freq(uint32_t ch_enum)
{
	return REG_CH_TO_FREQ(ch_enum);
}

uint8_t reg_max_5ghz_ch_num(void)
{
	return REG_MAX_5GHZ_CH_NUM;
}

#ifdef CONFIG_CHAN_FREQ_API
qdf_freq_t reg_min_24ghz_chan_freq(void)
{
	return REG_MIN_24GHZ_CH_FREQ;
}

qdf_freq_t reg_max_24ghz_chan_freq(void)
{
	return REG_MAX_24GHZ_CH_FREQ;
}

qdf_freq_t reg_min_5ghz_chan_freq(void)
{
	return REG_MIN_5GHZ_CH_FREQ;
}

qdf_freq_t reg_max_5ghz_chan_freq(void)
{
	return REG_MAX_5GHZ_CH_FREQ;
}
#endif /* CONFIG_CHAN_FREQ_API */

QDF_STATUS reg_enable_dfs_channels(struct wlan_objmgr_pdev *pdev,
				   bool enable)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;
	struct wlan_objmgr_psoc *psoc;
	QDF_STATUS status;
	struct wlan_lmac_if_reg_tx_ops *reg_tx_ops;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_INVAL;
	}

	if (pdev_priv_obj->dfs_enabled == enable) {
		reg_info("dfs_enabled is already set to %d", enable);
		return QDF_STATUS_SUCCESS;
	}

	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc) {
		reg_err("psoc is NULL");
		return QDF_STATUS_E_INVAL;
	}

	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_INVAL;
	}

	reg_info("set dfs_enabled: %d", enable);

	pdev_priv_obj->dfs_enabled = enable;

	reg_compute_pdev_current_chan_list(pdev_priv_obj);

	reg_tx_ops = reg_get_psoc_tx_ops(psoc);

	/* Fill the ic channel list with the updated current channel
	 * chan list.
	 */
	if (reg_tx_ops->fill_umac_legacy_chanlist)
		reg_tx_ops->fill_umac_legacy_chanlist(pdev,
				pdev_priv_obj->cur_chan_list);

	status = reg_send_scheduler_msg_sb(psoc, pdev);

	return status;
}

bool reg_is_regdmn_en302502_applicable(struct wlan_objmgr_pdev *pdev)
{
	struct cur_regdmn_info cur_reg_dmn;
	QDF_STATUS status;

	status = reg_get_curr_regdomain(pdev, &cur_reg_dmn);
	if (status != QDF_STATUS_SUCCESS) {
		reg_err("Failed to get reg domain");
		return false;
	}

	return reg_en302_502_regdmn(cur_reg_dmn.regdmn_pair_id);
}

QDF_STATUS reg_get_phybitmap(struct wlan_objmgr_pdev *pdev,
			     uint16_t *phybitmap)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!pdev_priv_obj) {
		reg_err("reg pdev private obj is NULL");
		return QDF_STATUS_E_FAULT;
	}

	*phybitmap = pdev_priv_obj->phybitmap;

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_modify_pdev_chan_range(struct wlan_objmgr_pdev *pdev)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;
	struct wlan_objmgr_psoc *psoc;
	struct wlan_lmac_if_reg_tx_ops *reg_tx_ops;
	struct wlan_psoc_host_hal_reg_capabilities_ext *reg_cap_ptr;
	uint32_t cnt;
	uint32_t phy_id;
	enum direction dir;
	QDF_STATUS status = QDF_STATUS_SUCCESS;
	struct target_pdev_info *tgt_pdev;

	tgt_pdev = wlan_pdev_get_tgt_if_handle(pdev);
	phy_id = (uint32_t)target_pdev_get_phy_idx(tgt_pdev);
	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_INVAL;
	}

	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc) {
		reg_err("psoc is NULL");
		return QDF_STATUS_E_INVAL;
	}

	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_INVAL;
	}

	reg_cap_ptr = psoc_priv_obj->reg_cap;

	for (cnt = 0; cnt < PSOC_MAX_PHY_REG_CAP; cnt++) {
		if (!reg_cap_ptr) {
			qdf_mem_free(pdev_priv_obj);
			reg_err("reg cap ptr is NULL");
			return QDF_STATUS_E_FAULT;
		}

		if (reg_cap_ptr->phy_id == phy_id)
			break;
		reg_cap_ptr++;
	}

	if (cnt == PSOC_MAX_PHY_REG_CAP) {
		qdf_mem_free(pdev_priv_obj);
		reg_err("extended capabilities not found for pdev");
		return QDF_STATUS_E_FAULT;
	}

	if (psoc_priv_obj->offload_enabled) {
		dir = NORTHBOUND;
	} else {
		dir = SOUTHBOUND;
	}

	pdev_priv_obj->range_2g_low = reg_cap_ptr->low_2ghz_chan;
	pdev_priv_obj->range_2g_high = reg_cap_ptr->high_2ghz_chan;
	pdev_priv_obj->range_5g_low = reg_cap_ptr->low_5ghz_chan;
	pdev_priv_obj->range_5g_high = reg_cap_ptr->high_5ghz_chan;
	pdev_priv_obj->wireless_modes = reg_cap_ptr->wireless_modes;

	reg_compute_pdev_current_chan_list(pdev_priv_obj);

	reg_tx_ops = reg_get_psoc_tx_ops(psoc);

	/* Fill the ic channel list with the updated current channel
	 * chan list.
	 */
	if (reg_tx_ops->fill_umac_legacy_chanlist) {
	    reg_tx_ops->fill_umac_legacy_chanlist(pdev,
						  pdev_priv_obj->cur_chan_list);

	} else {
		if (dir == NORTHBOUND)
			status = reg_send_scheduler_msg_nb(psoc, pdev);
		else
			status = reg_send_scheduler_msg_sb(psoc, pdev);
	}

	return status;
}

QDF_STATUS reg_update_pdev_wireless_modes(struct wlan_objmgr_pdev *pdev,
					  uint32_t wireless_modes)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = wlan_objmgr_pdev_get_comp_private_obj(pdev,
							      WLAN_UMAC_COMP_REGULATORY);

	if (!pdev_priv_obj) {
		reg_err("reg pdev private obj is NULL");
		return QDF_STATUS_E_INVAL;
	}

	pdev_priv_obj->wireless_modes = wireless_modes;

	return QDF_STATUS_SUCCESS;
}

#ifdef DISABLE_UNII_SHARED_BANDS
/**
 * reg_is_reg_unii_band_1_or_reg_unii_band_2a() - Check the input bitmap
 * @unii_5g_bitmap: 5G UNII band bitmap
 *
 * This function checks if either REG_UNII_BAND_1 or REG_UNII_BAND_2A,
 * are present in the 5G UNII band bitmap.
 *
 * Return: Return true if REG_UNII_BAND_1 or REG_UNII_BAND_2A, are present in
 * the UNII 5g bitmap else return false.
 */
static bool
reg_is_reg_unii_band_1_or_reg_unii_band_2a(uint8_t unii_5g_bitmap)
{
	if (!unii_5g_bitmap)
		return false;

	return ((unii_5g_bitmap & (BIT(REG_UNII_BAND_1) |
		 BIT(REG_UNII_BAND_2A))) ==  unii_5g_bitmap);
}

QDF_STATUS reg_disable_chan_coex(struct wlan_objmgr_pdev *pdev,
				 uint8_t unii_5g_bitmap)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	struct wlan_lmac_if_reg_tx_ops *reg_tx_ops;
	struct wlan_objmgr_psoc *psoc;

	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc) {
		reg_err("psoc is NULL");
		return QDF_STATUS_E_INVAL;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err_rl("reg pdev priv obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	if (unii_5g_bitmap &&
	    !reg_is_reg_unii_band_1_or_reg_unii_band_2a(unii_5g_bitmap)) {
		reg_err_rl("Invalid unii_5g_bitmap =  %d", unii_5g_bitmap);
		return QDF_STATUS_E_FAILURE;
	}

	if (pdev_priv_obj->unii_5g_bitmap == unii_5g_bitmap) {
		reg_debug_rl("UNII bitmask for 5G channels is already set  %d",
			    unii_5g_bitmap);
		return QDF_STATUS_SUCCESS;
	}

	reg_debug_rl("Setting UNII bitmask for 5G: %d", unii_5g_bitmap);
	pdev_priv_obj->unii_5g_bitmap = unii_5g_bitmap;

	reg_compute_pdev_current_chan_list(pdev_priv_obj);

	reg_tx_ops = reg_get_psoc_tx_ops(psoc);

	if (reg_tx_ops->fill_umac_legacy_chanlist) {
		reg_tx_ops->fill_umac_legacy_chanlist(pdev,
				pdev_priv_obj->cur_chan_list);
	}

	return QDF_STATUS_SUCCESS;
}
#endif

#ifdef CONFIG_CHAN_FREQ_API
QDF_STATUS reg_get_channel_list_with_power_for_freq(struct wlan_objmgr_pdev
						    *pdev,
						    struct channel_power
						    *ch_list,
						    uint8_t *num_chan)
{
	int i, count;
	struct regulatory_channel *reg_channels;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	if (!num_chan || !ch_list) {
		reg_err("chan_list or num_ch is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	/* set the current channel list */
	reg_channels = pdev_priv_obj->cur_chan_list;

	for (i = 0, count = 0; i < NUM_CHANNELS; i++) {
		if (reg_channels[i].state &&
		    !(reg_channels[i].chan_flags & REGULATORY_CHAN_DISABLED)) {
			ch_list[count].center_freq =
				reg_channels[i].center_freq;
			ch_list[count].chan_num = reg_channels[i].chan_num;
			ch_list[count++].tx_power =
				reg_channels[i].tx_power;
		}
	}

	*num_chan = count;

	return QDF_STATUS_SUCCESS;
}

enum channel_enum reg_get_chan_enum_for_freq(qdf_freq_t freq)
{
	uint32_t count;

	for (count = 0; count < NUM_CHANNELS; count++)
		if (channel_map[count].center_freq == freq)
			return count;

	reg_err_rl("invalid channel center frequency %d", freq);

	return INVALID_CHANNEL;
}

bool
reg_is_freq_present_in_cur_chan_list(struct wlan_objmgr_pdev *pdev,
				     qdf_freq_t freq)
{
	enum channel_enum chan_enum;
	struct regulatory_channel *cur_chan_list;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err_rl("pdev reg obj is NULL");
		return false;
	}

	cur_chan_list = pdev_priv_obj->cur_chan_list;

	for (chan_enum = 0; chan_enum < NUM_CHANNELS; chan_enum++)
		if (cur_chan_list[chan_enum].center_freq == freq)
			if ((cur_chan_list[chan_enum].state !=
			     CHANNEL_STATE_DISABLE) &&
			    !(cur_chan_list[chan_enum].chan_flags &
			      REGULATORY_CHAN_DISABLED))
				return true;

	reg_debug_rl("Channel center frequency %d not found", freq);

	return false;
}

#ifdef WLAN_FEATURE_GET_USABLE_CHAN_LIST
/**
 * is_freq_present_in_resp_list() - is freq present in resp list
 *
 * @pcl_ch: pcl ch
 * @res_msg: Response msg
 * @count: no of usable channels
 *
 * Return: void
 */
static bool
is_freq_present_in_resp_list(uint32_t pcl_ch,
			     struct get_usable_chan_res_params *res_msg,
			     int count)
{
	int i;

	for (i = 0; i < count; i++) {
		if (res_msg[i].freq == pcl_ch)
			return true;
	}
	return false;
}

/**
 * reg_update_usable_chan_resp() - Update response msg
 * @pdev: Pointer to pdev
 * @res_msg: Response msg
 * @pcl_ch: pcl channel
 * @len: calculated pcl len
 * @iface_mode_mask: interface type
 * @band_mask: requested band mask
 * @count: no of usable channels
 *
 * Return: void
 */
static void
reg_update_usable_chan_resp(struct wlan_objmgr_pdev *pdev,
			    struct get_usable_chan_res_params *res_msg,
			    uint32_t *pcl_ch, uint32_t len,
			    uint32_t iface_mode_mask,
			    uint32_t band_mask, int *count)
{
	int i;
	struct ch_params ch_params = {0};
	int index = *count;

	for (i = 0; i < len; i++) {
		/* In case usable channels are required for multiple filter
		 * mask, Some frequencies may present in res_msg . To avoid
		 * frequency duplication, only mode mask is updated for
		 * existing freqency.
		 */
		if (is_freq_present_in_resp_list(pcl_ch[i], res_msg, *count))
			continue;

		if (!(band_mask & 1 << wlan_reg_freq_to_band(pcl_ch[i])))
			continue;

		ch_params.ch_width = CH_WIDTH_MAX;
		reg_set_channel_params_for_freq(
				pdev,
				pcl_ch[i],
				0, &ch_params);
		res_msg[index].freq = (qdf_freq_t)pcl_ch[i];
		res_msg[index].iface_mode_mask |= 1 << iface_mode_mask;
		res_msg[index].bw = ch_params.ch_width;
		if (ch_params.center_freq_seg0)
			res_msg[index].seg0_freq =
					ch_params.center_freq_seg0;
		if (ch_params.center_freq_seg1)
			res_msg[index].seg1_freq =
					ch_params.center_freq_seg1;
		index++;
	}

	*count = index;
}

/**
 * reg_update_conn_chan_list() - Get usable channels with conn filter
 *				 and policy mgr mask
 * @pdev: Pointer to pdev
 * @res_msg: Response msg
 * @policy_mgr_con_mode: policy mgr mode
 * @iftype: interface type
 * @band_mask: requested band mask
 * @count: no of usable channels
 *
 * Return: qdf status
 */
static QDF_STATUS
reg_update_conn_chan_list(struct wlan_objmgr_pdev *pdev,
			  struct get_usable_chan_res_params *res_msg,
			  enum policy_mgr_con_mode mode,
			  uint32_t iftype,
			  uint32_t band_mask,
			  uint32_t *count)
{
	uint32_t *pcl_ch;
	uint8_t *weight_list;
	uint32_t len;
	uint32_t weight_len;
	struct wlan_objmgr_psoc *psoc;
	QDF_STATUS status = QDF_STATUS_SUCCESS;

	pcl_ch = qdf_mem_malloc(NUM_CHANNELS *
			sizeof(uint32_t));

	if (!pcl_ch) {
		reg_err("pcl_ch invalid");
		return QDF_STATUS_E_FAILURE;
	}

	weight_list = qdf_mem_malloc(NUM_CHANNELS *
			sizeof(uint8_t));

	if (!weight_list) {
		reg_err("weight_list invalid");
		qdf_mem_free(pcl_ch);
		return QDF_STATUS_E_FAILURE;
	}

	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc) {
		reg_err("invalid psoc");
		status = QDF_STATUS_E_FAILURE;
		goto err;
	}

	len = NUM_CHANNELS;
	weight_len = NUM_CHANNELS;

	status = policy_mgr_get_pcl(psoc, mode, pcl_ch, &len,
				    weight_list, weight_len);
	if (QDF_IS_STATUS_ERROR(status)) {
		reg_err("get pcl failed for mode: %d", mode);
		goto err;
	}
	reg_update_usable_chan_resp(pdev, res_msg, pcl_ch, len,
				    iftype, band_mask, count);
err:
	qdf_mem_free(pcl_ch);
	qdf_mem_free(weight_list);
	return status;
}

/**
 * reg_get_usable_channel_con_filter() - Get usable channel with con filter mask
 * @pdev: Pointer to pdev
 * @req_msg: Request msg
 * @res_msg: Response msg
 * @chan_list: reg channel list
 * @count: no of usable channels
 *
 * Return: qdf status
 */
static QDF_STATUS
reg_get_usable_channel_con_filter(struct wlan_objmgr_pdev *pdev,
				  struct get_usable_chan_req_params req_msg,
				  struct get_usable_chan_res_params *res_msg,
				  int *count)
{
	QDF_STATUS status = QDF_STATUS_SUCCESS;
	uint32_t iface_mode_mask = req_msg.iface_mode_mask;

	while (iface_mode_mask) {
		if (iface_mode_mask & 1 << IFTYPE_AP) {
			status =
			reg_update_conn_chan_list(pdev, res_msg, PM_SAP_MODE,
						  IFTYPE_AP, req_msg.band_mask,
						  count);
			iface_mode_mask &= ~(1 << IFTYPE_AP);
		} else if (iface_mode_mask & 1 << IFTYPE_STATION) {
			status =
			reg_update_conn_chan_list(pdev, res_msg, PM_STA_MODE,
						  IFTYPE_STATION,
						  req_msg.band_mask, count);
			iface_mode_mask &= ~(1 << IFTYPE_STATION);
		} else if (iface_mode_mask & 1 << IFTYPE_P2P_GO) {
			status =
			reg_update_conn_chan_list(pdev, res_msg, PM_P2P_GO_MODE,
						  IFTYPE_P2P_GO,
						  req_msg.band_mask, count);
			iface_mode_mask &= ~(1 << IFTYPE_P2P_GO);
		} else if (iface_mode_mask & 1 << IFTYPE_P2P_CLIENT) {
			status =
			reg_update_conn_chan_list(pdev, res_msg,
						  PM_P2P_CLIENT_MODE,
						  IFTYPE_P2P_CLIENT,
						  req_msg.band_mask, count);
			iface_mode_mask &= ~(1 << IFTYPE_P2P_CLIENT);
		} else if (iface_mode_mask & 1 << IFTYPE_NAN) {
			status =
			reg_update_conn_chan_list(pdev, res_msg,
						  PM_NAN_DISC_MODE, IFTYPE_NAN,
						  req_msg.band_mask, count);
			iface_mode_mask &= ~(1 << IFTYPE_NAN);
		} else {
			reg_err("invalid mode");
			break;
		}
	}
	return status;
}

/**
 * reg_remove_freq() - Remove invalid freq
 * @res_msg: Response msg
 * @index: index of freq that needs to be removed
 *
 * Return: void
 */
static void
reg_remove_freq(struct get_usable_chan_res_params *res_msg,
		int index)
{
	reg_debug("removing freq %d", res_msg[index].freq);
	qdf_mem_zero(&res_msg[index],
		     sizeof(struct get_usable_chan_res_params));
}

/**
 * reg_skip_invalid_chan_freq() - Remove invalid freq for SAP, P2P GO
 *				  and NAN
 * @pdev: Pointer to pdev
 * @res_msg: Response msg
 * @count: no of usable channels
 * @iface_mode_mask: interface mode mask
 *
 * Return: qdf status
 */
static QDF_STATUS
reg_skip_invalid_chan_freq(struct wlan_objmgr_pdev *pdev,
			   struct get_usable_chan_res_params *res_msg,
			   uint32_t *no_usable_channels,
			   uint32_t iface_mode_mask)
{
	uint32_t chan_enum, iface_mode = 0;
	QDF_STATUS status = QDF_STATUS_SUCCESS;
	bool include_indoor_channel, dfs_master_capable;
	uint8_t enable_srd_chan, srd_mask = 0;
	struct wlan_objmgr_psoc *psoc;
	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc) {
		reg_err("invalid psoc");
		return QDF_STATUS_E_FAILURE;
	}

	status = ucfg_mlme_get_indoor_channel_support(psoc,
						      &include_indoor_channel);
	if (QDF_IS_STATUS_ERROR(status)) {
		reg_err("failed to get indoor channel skip info");
		return QDF_STATUS_E_FAILURE;
	}

	ucfg_mlme_get_etsi_srd_chan_in_master_mode(psoc,
						   &enable_srd_chan);
	if (QDF_IS_STATUS_ERROR(status)) {
		reg_err("failed to get srd chan info");
		return QDF_STATUS_E_FAILURE;
	}

	status = ucfg_mlme_get_dfs_master_capability(psoc, &dfs_master_capable);
	if (QDF_IS_STATUS_ERROR(status)) {
		reg_err("failed to get dfs master capable");
		return status;
	}

	while (iface_mode_mask) {
		if (iface_mode_mask & (1 << IFTYPE_AP)) {
			srd_mask = 1;
			iface_mode = 1 << IFTYPE_AP;
		} else if (iface_mode_mask & (1 << IFTYPE_P2P_GO)) {
			srd_mask = 2;
			iface_mode = 1 << IFTYPE_P2P_GO;
		} else if (iface_mode_mask & (1 << IFTYPE_NAN)) {
			iface_mode = 1 << IFTYPE_NAN;
		} else {
			break;
		}
		for (chan_enum = 0; chan_enum < *no_usable_channels;
		     chan_enum++) {
			if (iface_mode_mask & (1 << IFTYPE_NAN)) {
				if (!wlan_is_nan_allowed_on_freq(pdev,
				     res_msg[chan_enum].freq))
					res_msg[chan_enum].iface_mode_mask &=
						~(iface_mode);
				if (!res_msg[chan_enum].iface_mode_mask)
					reg_remove_freq(res_msg, chan_enum);
			} else {
				if (wlan_reg_is_freq_indoor(
					pdev, res_msg[chan_enum].freq) &&
					!include_indoor_channel) {
					res_msg[chan_enum].iface_mode_mask &=
							~(iface_mode);
					if (!res_msg[chan_enum].iface_mode_mask)
						reg_remove_freq(res_msg,
								chan_enum);
				}

				if (!(enable_srd_chan & srd_mask) &&
				    reg_is_etsi13_srd_chan_for_freq(
					pdev, res_msg[chan_enum].freq)) {
					res_msg[chan_enum].iface_mode_mask &=
						~(iface_mode);
					if (!res_msg[chan_enum].iface_mode_mask)
						reg_remove_freq(res_msg,
								chan_enum);
				}

				if (!dfs_master_capable &&
				    wlan_reg_is_dfs_for_freq(pdev,
				    res_msg[chan_enum].freq)) {
					res_msg[chan_enum].iface_mode_mask &=
						~(iface_mode);
					if (!res_msg[chan_enum].iface_mode_mask)
						reg_remove_freq(res_msg,
								chan_enum);
				}
			}
		}

		iface_mode_mask &= ~iface_mode;
	}

	return status;
}

/**
 * reg_get_usable_channel_no_filter() - Get usable channel with no filter mask
 * @pdev: Pointer to pdev
 * @req_msg: Request msg
 * @res_msg: Response msg
 * @chan_list: reg channel list
 * @count: no of usable channels
 *
 * Return: qdf status
 */
static QDF_STATUS
reg_get_usable_channel_no_filter(struct wlan_objmgr_pdev *pdev,
				 struct get_usable_chan_req_params req_msg,
				 struct get_usable_chan_res_params *res_msg,
				 struct regulatory_channel *chan_list,
				 int *count)
{
	QDF_STATUS status = QDF_STATUS_SUCCESS;

	status =
	reg_skip_invalid_chan_freq(pdev, res_msg,
				   count, req_msg.iface_mode_mask);
	return status;
}

/**
 * reg_get_usable_channel_coex_filter() - Get usable channel with coex filter
 * @pdev: Pointer to pdev
 * @req_msg: Request msg
 * @res_msg: Response msg
 * @chan_list: reg channel list
 * @count: no of usable channels
 *
 * Return: qdf status
 */
static QDF_STATUS
reg_get_usable_channel_coex_filter(struct wlan_objmgr_pdev *pdev,
				   struct get_usable_chan_req_params req_msg,
				   struct get_usable_chan_res_params *res_msg,
				   struct regulatory_channel *chan_list,
				   int *count)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;
	enum channel_enum chan_enum;
	uint32_t i = 0;
	struct ch_avoid_freq_type freq_range;
	struct wlan_objmgr_psoc *psoc;
	QDF_STATUS status = QDF_STATUS_SUCCESS;

	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc) {
		reg_err("invalid psoc");
		return QDF_STATUS_E_FAILURE;
	}

	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_alert("psoc reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}
	for (chan_enum = 0; chan_enum < *count; chan_enum++) {
		for (i = 0; i <
		    psoc_priv_obj->avoid_freq_list.ch_avoid_range_cnt; i++) {
			freq_range =
			psoc_priv_obj->avoid_freq_list.avoid_freq_range[i];

			if (freq_range.start_freq <=
			    chan_list[chan_enum].center_freq &&
			    freq_range.end_freq >=
			    chan_list[chan_enum].center_freq) {
				reg_remove_freq(res_msg, chan_enum);
			}
		}
	}
	if (req_msg.iface_mode_mask & 1 << IFTYPE_AP ||
	    req_msg.iface_mode_mask & 1 << IFTYPE_P2P_GO ||
	    req_msg.iface_mode_mask & 1 << IFTYPE_NAN)
		status =
		reg_skip_invalid_chan_freq(pdev, res_msg, count,
					   req_msg.iface_mode_mask);
	return status;
}

/**
 * reg_calculate_mode_mask() - calculate valid mode mask
 * @iface_mode_mask: interface mode mask
 *
 * Return: Valid mode mask
 */
static uint32_t
reg_calculate_mode_mask(uint32_t iface_mode_mask)
{
	int mode_mask = 0;

	mode_mask = (iface_mode_mask & 1 << IFTYPE_STATION) |
		    (iface_mode_mask & 1 << IFTYPE_AP) |
		    (iface_mode_mask & 1 << IFTYPE_P2P_GO) |
		    (iface_mode_mask & 1 << IFTYPE_P2P_CLIENT) |
		    (iface_mode_mask & 1 << IFTYPE_P2P_DEVICE) |
		    (iface_mode_mask & 1 << IFTYPE_NAN);

	return mode_mask;
}

/**
 * reg_add_usable_channel_to_resp() - Add usable channels to resp structure
 * @pdev: Pointer to pdev
 * @res_msg: Response msg
 * @iface_mode_mask: interface mode mask
 * @chan_list: reg channel list
 * @count: no of usable channels
 *
 * Return: qdf status
 */
static QDF_STATUS
reg_add_usable_channel_to_resp(struct wlan_objmgr_pdev *pdev,
			       struct get_usable_chan_res_params *res_msg,
			       uint32_t iface_mode_mask,
			       struct regulatory_channel *chan_list,
			       int *count)
{
	enum channel_enum chan_enum;
	struct ch_params ch_params = {0};
	QDF_STATUS status = QDF_STATUS_SUCCESS;
	uint32_t mode_mask = 0;

	mode_mask = reg_calculate_mode_mask(iface_mode_mask);

	for (chan_enum = 0; chan_enum < *count &&
	     chan_enum < NUM_CHANNELS; chan_enum++) {
		ch_params.ch_width = CH_WIDTH_MAX;
		reg_set_channel_params_for_freq(
				pdev,
				chan_list[chan_enum].center_freq,
				chan_list[chan_enum].max_bw, &ch_params);

		res_msg[chan_enum].freq = chan_list[chan_enum].center_freq;
		res_msg[chan_enum].iface_mode_mask = mode_mask;
		if (!res_msg[chan_enum].iface_mode_mask) {
			reg_err("invalid iface mask");
			return QDF_STATUS_E_FAILURE;
		}
		res_msg[chan_enum].bw = ch_params.ch_width;
		res_msg[chan_enum].state = chan_list[chan_enum].state;
		if (ch_params.center_freq_seg0)
			res_msg[chan_enum].seg0_freq =
					ch_params.center_freq_seg0;
		if (ch_params.center_freq_seg1)
			res_msg[chan_enum].seg1_freq =
					ch_params.center_freq_seg1;
	}

	return status;
}

QDF_STATUS
wlan_reg_get_usable_channel(struct wlan_objmgr_pdev *pdev,
			    struct get_usable_chan_req_params req_msg,
			    struct get_usable_chan_res_params *res_msg,
			    uint32_t *usable_channels)
{
	struct regulatory_channel *chan_list;
	QDF_STATUS status = QDF_STATUS_SUCCESS;

	chan_list = qdf_mem_malloc(NUM_CHANNELS *
			sizeof(*chan_list));

	if (!chan_list) {
		reg_err("chan_list invalid");
		return QDF_STATUS_E_FAILURE;
	}

	if ((req_msg.filter_mask & 1 << FILTER_CELLULAR_COEX) ||
	    (!(req_msg.filter_mask & 1 << FILTER_CELLULAR_COEX) &&
	     !(req_msg.filter_mask & 1 << FILTER_WLAN_CONCURRENCY))) {
		*usable_channels = reg_get_band_channel_list(pdev,
							     req_msg.band_mask,
							     chan_list);
		status =
		reg_add_usable_channel_to_resp(pdev, res_msg,
					       req_msg.iface_mode_mask,
					       chan_list, usable_channels);
		if (QDF_IS_STATUS_ERROR(status)) {
			qdf_mem_free(chan_list);
			return status;
		}
	}

	if (req_msg.filter_mask & 1 << FILTER_CELLULAR_COEX)
		status =
		reg_get_usable_channel_coex_filter(pdev, req_msg, res_msg,
						   chan_list, usable_channels);

	if (req_msg.filter_mask & 1 << FILTER_WLAN_CONCURRENCY)
		status =
		reg_get_usable_channel_con_filter(pdev, req_msg, res_msg,
						  usable_channels);

	if (!(req_msg.filter_mask & 1 << FILTER_CELLULAR_COEX) &&
	    !(req_msg.filter_mask & 1 << FILTER_WLAN_CONCURRENCY))
		status =
		reg_get_usable_channel_no_filter(pdev, req_msg, res_msg,
						 chan_list, usable_channels);
	reg_debug("usable chan count is %d", *usable_channels);

	qdf_mem_free(chan_list);
	return status;
}
#endif

enum channel_state reg_get_channel_state_for_freq(struct wlan_objmgr_pdev *pdev,
						  qdf_freq_t freq)
{
	enum channel_enum ch_idx;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	ch_idx = reg_get_chan_enum_for_freq(freq);

	if (ch_idx == INVALID_CHANNEL)
		return CHANNEL_STATE_INVALID;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg obj is NULL");
		return CHANNEL_STATE_INVALID;
	}

	return pdev_priv_obj->cur_chan_list[ch_idx].state;
}

static uint32_t reg_get_channel_flags_for_freq(struct wlan_objmgr_pdev *pdev,
					       qdf_freq_t freq)
{
	enum channel_enum chan_enum;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	chan_enum = reg_get_chan_enum_for_freq(freq);

	if (chan_enum == INVALID_CHANNEL) {
		reg_err("chan freq is not valid");
		return REGULATORY_CHAN_INVALID;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg obj is NULL");
		return REGULATORY_CHAN_INVALID;
	}

	return pdev_priv_obj->cur_chan_list[chan_enum].chan_flags;
}

#ifdef CONFIG_REG_CLIENT
enum channel_state reg_get_channel_state_from_secondary_list_for_freq(
						struct wlan_objmgr_pdev *pdev,
						qdf_freq_t freq)
{
	enum channel_enum ch_idx;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	ch_idx = reg_get_chan_enum_for_freq(freq);

	if (ch_idx == INVALID_CHANNEL)
		return CHANNEL_STATE_INVALID;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg obj is NULL");
		return CHANNEL_STATE_INVALID;
	}

	return pdev_priv_obj->secondary_cur_chan_list[ch_idx].state;
}

static uint32_t reg_get_channel_flags_from_secondary_list_for_freq(
						struct wlan_objmgr_pdev *pdev,
						qdf_freq_t freq)
{
	enum channel_enum chan_enum;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	chan_enum = reg_get_chan_enum_for_freq(freq);

	if (chan_enum == INVALID_CHANNEL) {
		reg_err_rl("chan freq %u is not valid", freq);
		return REGULATORY_CHAN_INVALID;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg obj is NULL");
		return REGULATORY_CHAN_INVALID;
	}

	return pdev_priv_obj->secondary_cur_chan_list[chan_enum].chan_flags;
}
#endif

/**
 * reg_get_5g_bonded_chan_array_for_freq()- Return the channel state for a
 * 5G or 6G channel frequency based on the bonded channel.
 * @pdev: Pointer to pdev.
 * @freq: Channel center frequency.
 * @bonded_chan_ptr: Pointer to bonded_channel_freq.
 *
 * Return: Channel State
 */
static enum channel_state
reg_get_5g_bonded_chan_array_for_freq(struct wlan_objmgr_pdev *pdev,
				      uint16_t freq,
				      const struct bonded_channel_freq *
				      bonded_chan_ptr)
{
	uint16_t chan_cfreq;
	enum channel_state chan_state = CHANNEL_STATE_INVALID;
	enum channel_state temp_chan_state;

	if (!bonded_chan_ptr) {
		reg_debug("bonded chan ptr is NULL");
		return chan_state;
	}

	chan_cfreq =  bonded_chan_ptr->start_freq;
	while (chan_cfreq <= bonded_chan_ptr->end_freq) {
		temp_chan_state = reg_get_channel_state_for_freq(pdev,
								 chan_cfreq);
		if (temp_chan_state < chan_state)
			chan_state = temp_chan_state;
		chan_cfreq = chan_cfreq + 20;
	}

	return chan_state;
}

enum channel_state
reg_get_5g_bonded_channel_state_for_freq(struct wlan_objmgr_pdev *pdev,
					 qdf_freq_t freq,
					 enum phy_ch_width bw)
{
	enum channel_enum ch_indx;
	enum channel_state chan_state;
	struct regulatory_channel *reg_channels;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	bool bw_enabled = false;
	const struct bonded_channel_freq *bonded_chan_ptr = NULL;

	if (bw > CH_WIDTH_80P80MHZ) {
		reg_err_rl("bw (%d) passed is not good", bw);
		return CHANNEL_STATE_INVALID;
	}

	chan_state = reg_get_5g_bonded_channel_for_freq(pdev, freq, bw,
							&bonded_chan_ptr);

	if ((chan_state == CHANNEL_STATE_INVALID) ||
	    (chan_state == CHANNEL_STATE_DISABLE))
		return chan_state;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg obj is NULL");
		return CHANNEL_STATE_INVALID;
	}
	reg_channels = pdev_priv_obj->cur_chan_list;

	ch_indx = reg_get_chan_enum_for_freq(freq);
	if (ch_indx == INVALID_CHANNEL)
		return CHANNEL_STATE_INVALID;
	if (bw == CH_WIDTH_5MHZ)
		bw_enabled = true;
	else if (bw == CH_WIDTH_10MHZ)
		bw_enabled = (reg_channels[ch_indx].min_bw <= 10) &&
			(reg_channels[ch_indx].max_bw >= 10);
	else if (bw == CH_WIDTH_20MHZ)
		bw_enabled = (reg_channels[ch_indx].min_bw <= 20) &&
			(reg_channels[ch_indx].max_bw >= 20);
	else if (bw == CH_WIDTH_40MHZ)
		bw_enabled = (reg_channels[ch_indx].min_bw <= 40) &&
			(reg_channels[ch_indx].max_bw >= 40);
	else if (bw == CH_WIDTH_80MHZ)
		bw_enabled = (reg_channels[ch_indx].min_bw <= 80) &&
			(reg_channels[ch_indx].max_bw >= 80);
	else if (bw == CH_WIDTH_160MHZ)
		bw_enabled = (reg_channels[ch_indx].min_bw <= 160) &&
			(reg_channels[ch_indx].max_bw >= 160);
	else if (bw == CH_WIDTH_80P80MHZ)
		bw_enabled = (reg_channels[ch_indx].min_bw <= 80) &&
			(reg_channels[ch_indx].max_bw >= 80);

	if (bw_enabled)
		return chan_state;
	else
		return CHANNEL_STATE_DISABLE;
}

enum channel_state
reg_get_2g_bonded_channel_state_for_freq(struct wlan_objmgr_pdev *pdev,
					 qdf_freq_t oper_ch_freq,
					 qdf_freq_t sec_ch_freq,
					 enum phy_ch_width bw)
{
	enum channel_enum chan_idx;
	enum channel_state chan_state;
	struct regulatory_channel *reg_channels;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	bool bw_enabled = false;
	enum channel_state chan_state2 = CHANNEL_STATE_INVALID;

	if (bw > CH_WIDTH_40MHZ)
		return CHANNEL_STATE_INVALID;

	if (bw == CH_WIDTH_40MHZ) {
		if ((sec_ch_freq + 20 != oper_ch_freq) &&
		    (oper_ch_freq + 20 != sec_ch_freq))
			return CHANNEL_STATE_INVALID;
		chan_state2 = reg_get_channel_state_for_freq(pdev, sec_ch_freq);
		if (chan_state2 == CHANNEL_STATE_INVALID)
			return chan_state2;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return CHANNEL_STATE_INVALID;
	}

	reg_channels = pdev_priv_obj->cur_chan_list;

	chan_state = reg_get_channel_state_for_freq(pdev, oper_ch_freq);
	if (chan_state2 < chan_state)
		chan_state = chan_state2;

	if ((chan_state == CHANNEL_STATE_INVALID) ||
	    (chan_state == CHANNEL_STATE_DISABLE))
		return chan_state;

	chan_idx = reg_get_chan_enum_for_freq(oper_ch_freq);
	if (chan_idx == INVALID_CHANNEL)
		return CHANNEL_STATE_INVALID;
	if (bw == CH_WIDTH_5MHZ)
		bw_enabled = true;
	else if (bw == CH_WIDTH_10MHZ)
		bw_enabled = (reg_channels[chan_idx].min_bw <= 10) &&
			(reg_channels[chan_idx].max_bw >= 10);
	else if (bw == CH_WIDTH_20MHZ)
		bw_enabled = (reg_channels[chan_idx].min_bw <= 20) &&
			(reg_channels[chan_idx].max_bw >= 20);
	else if (bw == CH_WIDTH_40MHZ)
		bw_enabled = (reg_channels[chan_idx].min_bw <= 40) &&
			(reg_channels[chan_idx].max_bw >= 40);

	if (bw_enabled)
		return chan_state;
	else
		return CHANNEL_STATE_DISABLE;

	return CHANNEL_STATE_ENABLE;
}

#ifdef WLAN_FEATURE_11BE
/**
 * reg_get_320_bonded_chan_array() - Fetches a list of bonded channel pointers
 * for the given bonded channel array. If 320 band center is specified,
 * return the bonded channel pointer comprising of given band center else
 * return list of all available bonded channel pair.
 *
 * @pdev: Pointer to struct wlan_objmgr_pdev.
 * @freq: Input frequency in MHZ whose bonded channel pointer must be fetched.
 * @band_center_320: Channel center frequency of 320MHZ channel.
 * @bonded_chan_ar: Array of bonded channel list.
 * @array_size: Size of bonded channel array.
 * @bonded_chan_ptr: Pointer to hold the address of bonded_channel_freq index.
 *
 * Return: number of bonded channel arrays fetched.
 */

#define MAX_NUM_BONDED_PAIR 2
static uint8_t
reg_get_320_bonded_chan_array(struct wlan_objmgr_pdev *pdev,
			      qdf_freq_t freq,
			      qdf_freq_t band_center_320,
			      const struct bonded_channel_freq bonded_chan_ar[],
			      uint16_t array_size,
			      const struct bonded_channel_freq
			      *bonded_chan_ptr[])
{
	int i;
	uint8_t num_bonded_pairs = 0;

	/* Fetch all possible bonded channel pointers for the given freq */
	if (!band_center_320) {
		for (i = 0 ; i < array_size &&
		     num_bonded_pairs < MAX_NUM_BONDED_PAIR; i++) {
			if ((freq >= bonded_chan_ar[i].start_freq) &&
			    (freq <= bonded_chan_ar[i].end_freq)) {
				bonded_chan_ptr[num_bonded_pairs] =
					&bonded_chan_ar[i];
				num_bonded_pairs++;
			}
		}
	} else {
		/* Fetch the bonded channel pointer for the given band_center */
		for (i = 0; i < array_size; i++) {
			if (((bonded_chan_ar[i].start_freq +
			      bonded_chan_ar[i].end_freq) / 2) ==
				band_center_320) {
				bonded_chan_ptr[num_bonded_pairs] =
					&bonded_chan_ar[i];
				num_bonded_pairs++;
				break;
			}
		}
	}
	return num_bonded_pairs;
}

static inline bool reg_is_state_allowed(enum channel_state chan_state)
{
	return !((chan_state == CHANNEL_STATE_INVALID) ||
		 (chan_state == CHANNEL_STATE_DISABLE));
}

/**
 * reg_get_320_bonded_channel_state() - Given a bonded channel
 * pointer and freq, determine if the subchannels of the bonded pair
 * are valid and supported by the current regulatory.
 *
 * @pdev: Pointer to struct wlan_objmgr_pdev.
 * @freq: Frequency in MHZ.
 * @bonded_chan_ptr: Pointer to const struct bonded_channel_freq.
 * @bw: channel bandwidth
 * @out_punc_pat: Output puncturing pattern
 *
 * Return - The channel state of the bonded pair.
 */
#define SUB_CHAN_BW 20 /* 20 MHZ */
#define BW_160MHZ 160
#define  REG_IS_TOT_CHAN_BW_BELOW_160(_x, _y) \
	(reg_is_state_allowed((_x)) && (_y) < BW_160MHZ)
#define REG_IS_PRIMARY_CHAN_NOT_ALLOWED(_x, _y) \
	(!reg_is_state_allowed(reg_get_channel_state_for_freq((_x), (_y))))

static enum channel_state
reg_get_320_bonded_channel_state(struct wlan_objmgr_pdev *pdev,
				 qdf_freq_t freq,
				 const struct bonded_channel_freq
				 *bonded_chan_ptr,
				 enum phy_ch_width bw,
				 uint16_t *out_punc_pat)
{
	enum channel_state chan_state = CHANNEL_STATE_INVALID;
	enum channel_state temp_chan_state;
	uint16_t chan_cfreq;
	uint16_t max_cont_bw, i;

	*out_punc_pat = ALL_SCHANS_PUNC;

	if (!bonded_chan_ptr)
		return chan_state;

	chan_cfreq =  bonded_chan_ptr->start_freq;

	max_cont_bw = 0;
	i = 0;

	while (chan_cfreq <= bonded_chan_ptr->end_freq) {
		temp_chan_state = reg_get_channel_state_for_freq(pdev,
								 chan_cfreq);
		if (reg_is_state_allowed(temp_chan_state)) {
			max_cont_bw += SUB_CHAN_BW;
			*out_punc_pat |= BIT(i);
		}

		if (temp_chan_state < chan_state)
			chan_state = temp_chan_state;

		chan_cfreq = chan_cfreq + SUB_CHAN_BW;
		i++;
	}

	/* After iterating through all the subchannels, if the final channel
	 * state is invalid/disable, it means all our subchannels are not
	 * valid and we could not find a 320 MHZ channel.
	 * If we have found a channel where the max width is:
	 * 1. Less than 160: there is no puncturing needed. Hence return
	 * the chan state as invalid. Or if the primary freq given is not
	 * supported by regulatory, the channel cannot be enabled as a
	 * punctured channel. So return channel state as invalid.
	 * 2. If greater than 160: Mark the invalid channels as punctured.
	 * and return channel state as ENABLE.
	 */
	if (REG_IS_TOT_CHAN_BW_BELOW_160(chan_state, max_cont_bw) ||
	    REG_IS_PRIMARY_CHAN_NOT_ALLOWED(pdev, freq))
		return CHANNEL_STATE_INVALID;
	else
		return CHANNEL_STATE_ENABLE;
}

/**
 * reg_fill_primary_160mhz_centers() - Fill the primary 160MHz segment centers
 * for a 320MHz channel in the given channel param.
 * @pdev: Pointer to struct wlan_objmgr_pdev.
 * @ch_param: channel params to be filled.
 * @freq: Input primary frequency in MHZ.
 *
 * Return: void.
 */
static void
reg_fill_primary_160mhz_centers(struct wlan_objmgr_pdev *pdev,
				struct ch_params *ch_param, qdf_freq_t freq)
{
	const struct bonded_channel_freq *t_bonded_ch_ptr;

	t_bonded_ch_ptr = reg_get_bonded_chan_entry(freq, CH_WIDTH_160MHZ);
	if (t_bonded_ch_ptr) {
		ch_param->mhz_freq_seg0 =
			(t_bonded_ch_ptr->start_freq +
			 t_bonded_ch_ptr->end_freq) / 2;
		ch_param->center_freq_seg0 =
			reg_freq_to_chan(pdev,
					 ch_param->mhz_freq_seg0);
	} else {
		reg_err("Cannot find 160Mhz centers for freq %d", freq);
		ch_param->ch_width = CH_WIDTH_INVALID;
	}
}

/**
 * reg_fill_channel_list_for_320() - Fill 320MHZ channel list. If we
 * are unable to find a channel whose width is greater than 160MHZ and less
 * than 320 with the help of puncturing, using the given freq, set "update_bw"
 * variable to be true, lower the channel width and return to the caller.
 * The caller fetches a channel of reduced mode based on "update_bw" flag.
 *
 * If 320 band center is 0, return all the 320 channels
 * that match the primary frequency else return only channel
 * that matches 320 band center.
 *
 * @pdev: Pointer to struct wlan_objmgr_pdev.
 * @freq: Input frequency in MHZ.
 * @ch_width: Input channel width, if a channel of the given width is not
 * found, reduce the channel width to the next lower mode and pass it to the
 * caller.
 * @band_center_320: Center of 320MHZ channel.
 * @chan_list: Pointer to reg_channel_list to be filled.
 * @update_bw: Flag to hold if bw is updated.
 *
 * Return - None.
 */
static void
reg_fill_channel_list_for_320(struct wlan_objmgr_pdev *pdev,
			      qdf_freq_t freq,
			      enum phy_ch_width *in_ch_width,
			      qdf_freq_t band_center_320,
			      struct reg_channel_list *chan_list,
			      bool *update_bw)
{
	uint8_t num_bonded_pairs, i, num_ch_params;
	enum channel_state chan_state;
	uint16_t array_size = QDF_ARRAY_SIZE(bonded_chan_320mhz_list_freq);
	uint16_t out_punc_pat;
	uint16_t max_reg_bw;
	enum channel_enum chan_enum;
	const struct bonded_channel_freq *bonded_ch_ptr[2] = {NULL, NULL};
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	*update_bw = false;

	chan_enum = reg_get_chan_enum_for_freq(freq);
	if (chan_enum == INVALID_CHANNEL) {
		reg_err("chan freq is not valid");
		return;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return;
	}

	/* Maximum bandwidth of the channel supported by regulatory for
	 * the given freq.
	 */
	max_reg_bw = pdev_priv_obj->cur_chan_list[chan_enum].max_bw;

	/* Regulatory does not support BW greater than 160.
	 * Try finding a channel in a lower mode.
	 */
	if (max_reg_bw <= BW_160MHZ) {
		*in_ch_width =  get_next_lower_bandwidth(*in_ch_width);
		*update_bw = true;
		return;
	}

	num_bonded_pairs =
		reg_get_320_bonded_chan_array(pdev, freq, band_center_320,
					      bonded_chan_320mhz_list_freq,
					      array_size,
					      bonded_ch_ptr);

	if (!num_bonded_pairs) {
		if (band_center_320) {
			reg_debug("No bonded pair for the given band_center\n");
			chan_list->num_ch_params = 0;
		} else {
			/* Could not find a 320 MHZ bonded channel pair,
			 * find a channel of lower BW.
			 */
			*in_ch_width =  get_next_lower_bandwidth(*in_ch_width);
			*update_bw = true;
		}
		return;
	}

	for (i = 0, num_ch_params = 0 ; i < num_bonded_pairs; i++) {
		/* Chan_state to hold the channel state of bonding
		 * pair of channels.
		 */
		chan_state =
		    reg_get_320_bonded_channel_state(pdev, freq,
						     bonded_ch_ptr[i],
						     *in_ch_width,
						     &out_punc_pat);
		if (chan_state == CHANNEL_STATE_ENABLE) {
			struct ch_params *t_chan_param =
			    &chan_list->chan_param[num_ch_params];

			t_chan_param->mhz_freq_seg1 =
				(bonded_ch_ptr[i]->start_freq +
				 bonded_ch_ptr[i]->end_freq) / 2;
			t_chan_param->center_freq_seg1 =
				reg_freq_to_chan(pdev,
						 t_chan_param->mhz_freq_seg1);
			t_chan_param->ch_width = *in_ch_width;
			t_chan_param->reg_punc_pattern = out_punc_pat;

			reg_fill_primary_160mhz_centers(pdev,
							t_chan_param,
							freq);
			num_ch_params++;
			chan_list->num_ch_params = num_ch_params;
		}
	}

	/* The bonded pairs could not create any channels,
	 * lower the bandwidth to find a channel.
	 */
	if (!chan_list->num_ch_params) {
		*in_ch_width =  get_next_lower_bandwidth(*in_ch_width);
		*update_bw = true;
	}
}

/**
 * No subchannels are punctured
 * binary 0:- Punctured 1:- Not-Punctured.
 */
#define NO_SCHANS_PUNC 0xFFFF

/**
 * reg_fill_pre320mhz_channel() - Fill channel params for channel width
 * less than 320.
 * @pdev: Pointer to struct wlan_objmgr_pdev
 * @chan_list: Pointer to struct reg_channel_list
 * @ch_width: Channel width
 * @freq: Center frequency of the primary channel in MHz
 * @sec_ch_2g_freq:  Secondary 2G channel frequency in MHZ
 */
static void
reg_fill_pre320mhz_channel(struct wlan_objmgr_pdev *pdev,
			   struct reg_channel_list *chan_list,
			   enum phy_ch_width ch_width,
			   qdf_freq_t freq,
			   qdf_freq_t sec_ch_2g_freq)
{
	chan_list->num_ch_params = 1;
	chan_list->chan_param[0].ch_width = ch_width;
	chan_list->chan_param[0].reg_punc_pattern = NO_SCHANS_PUNC;
	reg_set_channel_params_for_freq(pdev, freq, sec_ch_2g_freq,
					&chan_list->chan_param[0]);
}

void
reg_fill_channel_list(struct wlan_objmgr_pdev *pdev,
		      qdf_freq_t freq,
		      qdf_freq_t sec_ch_2g_freq,
		      enum phy_ch_width in_ch_width,
		      qdf_freq_t band_center_320,
		      struct reg_channel_list *chan_list)
{
	bool update_bw;

	if (!chan_list) {
		reg_err("channel params is NULL");
		return;
	}

	if (in_ch_width >= CH_WIDTH_MAX)
		in_ch_width = CH_WIDTH_320MHZ;

	if (in_ch_width == CH_WIDTH_320MHZ) {
		update_bw = 0;
		reg_fill_channel_list_for_320(pdev, freq, &in_ch_width,
					      band_center_320, chan_list,
					      &update_bw);
		if (!update_bw)
			return;
	}

	/* A 320 channel is not available (or) user has not requested
	 * for a 320MHZ channel, look for channels in lower modes,
	 * reg_set_5g_channel_params_for_freq() finds for the
	 * next available mode and fills ch_params.
	 */
	reg_fill_pre320mhz_channel(pdev, chan_list, in_ch_width, freq,
				   sec_ch_2g_freq);
}
#endif

#if defined(WLAN_FEATURE_11BE) && defined(CONFIG_REG_CLIENT)
enum channel_state
reg_get_5g_bonded_channel_for_freq(struct wlan_objmgr_pdev *pdev,
				   uint16_t freq,
				   enum phy_ch_width ch_width,
				   const struct bonded_channel_freq
				   **bonded_chan_ptr_ptr)
{
	if (ch_width == CH_WIDTH_20MHZ)
		return reg_get_channel_state_for_freq(pdev, freq);

	if (ch_width == CH_WIDTH_320MHZ) {
		uint8_t num_bonded_pairs;
		uint16_t array_size =
				QDF_ARRAY_SIZE(bonded_chan_320mhz_list_freq);
		const struct bonded_channel_freq *bonded_ch_ptr[2] = {
								NULL, NULL};
		uint16_t punct_pattern;

		/* For now sending band center freq as 0 */
		num_bonded_pairs =
			reg_get_320_bonded_chan_array(
						pdev, freq, 0,
						bonded_chan_320mhz_list_freq,
						array_size, bonded_ch_ptr);
		if (!num_bonded_pairs) {
			reg_info("No 320MHz bonded pair for freq %d", freq);
			return CHANNEL_STATE_INVALID;
		}
		/* Taking only first bonded pair */
		*bonded_chan_ptr_ptr = bonded_ch_ptr[0];

		return reg_get_320_bonded_channel_state(pdev, freq,
							bonded_ch_ptr[0],
							ch_width,
							&punct_pattern);
	} else {
		*bonded_chan_ptr_ptr = reg_get_bonded_chan_entry(freq,
								 ch_width);
		if (!(*bonded_chan_ptr_ptr))
			return CHANNEL_STATE_INVALID;

		return reg_get_5g_bonded_chan_array_for_freq(
							pdev, freq,
							*bonded_chan_ptr_ptr);
	}
}
#else
enum channel_state
reg_get_5g_bonded_channel_for_freq(struct wlan_objmgr_pdev *pdev,
				   uint16_t freq,
				   enum phy_ch_width ch_width,
				   const struct bonded_channel_freq
				   **bonded_chan_ptr_ptr)

{
	if (ch_width == CH_WIDTH_20MHZ)
		return reg_get_channel_state_for_freq(pdev, freq);

	/* Fetch the bonded_chan_ptr for width greater than 20MHZ. */
	*bonded_chan_ptr_ptr = reg_get_bonded_chan_entry(freq, ch_width);

	if (!(*bonded_chan_ptr_ptr)) {
		reg_debug_rl("bonded_chan_ptr_ptr is NULL");
		return CHANNEL_STATE_INVALID;
	}

	return reg_get_5g_bonded_chan_array_for_freq(pdev, freq,
						     *bonded_chan_ptr_ptr);
}
#endif

/**
 * reg_set_5g_channel_params_for_freq()- Set channel parameters like center
 * frequency for a bonded channel state. Also return the maximum bandwidth
 * supported by the channel.
 * @pdev: Pointer to pdev.
 * @freq: Channel center frequency.
 * ch_params: Pointer to ch_params.
 *
 * Return: void
 */
static void reg_set_5g_channel_params_for_freq(struct wlan_objmgr_pdev *pdev,
					       uint16_t freq,
					       struct ch_params *ch_params)
{
	/*
	 * Set channel parameters like center frequency for a bonded channel
	 * state. Also return the maximum bandwidth supported by the channel.
	 */

	enum channel_state chan_state = CHANNEL_STATE_ENABLE;
	enum channel_state chan_state2 = CHANNEL_STATE_ENABLE;
	const struct bonded_channel_freq *bonded_chan_ptr = NULL;
	const struct bonded_channel_freq *bonded_chan_ptr2 = NULL;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	enum channel_enum chan_enum, sec_5g_chan_enum;
	uint16_t max_bw, bw_80, sec_5g_freq_max_bw = 0;

	if (!ch_params) {
		reg_err("ch_params is NULL");
		return;
	}

	chan_enum = reg_get_chan_enum_for_freq(freq);
	if (chan_enum == INVALID_CHANNEL) {
		reg_err("chan freq is not valid");
		return;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return;
	}

	if (ch_params->ch_width >= CH_WIDTH_MAX) {
		if (ch_params->mhz_freq_seg1 != 0)
			ch_params->ch_width = CH_WIDTH_80P80MHZ;
		else
			ch_params->ch_width = CH_WIDTH_160MHZ;
	}

	max_bw = pdev_priv_obj->cur_chan_list[chan_enum].max_bw;
	bw_80 = reg_get_bw_value(CH_WIDTH_80MHZ);

	if (ch_params->ch_width == CH_WIDTH_80P80MHZ) {
		sec_5g_chan_enum =
			reg_get_chan_enum_for_freq(ch_params->mhz_freq_seg1 -
					NEAREST_20MHZ_CHAN_FREQ_OFFSET);
		if (sec_5g_chan_enum == INVALID_CHANNEL) {
			reg_err("secondary channel freq is not valid");
			return;
		}

		sec_5g_freq_max_bw =
			pdev_priv_obj->cur_chan_list[sec_5g_chan_enum].max_bw;
	}

	while (ch_params->ch_width != CH_WIDTH_INVALID) {
		if (ch_params->ch_width == CH_WIDTH_80P80MHZ) {
			if ((max_bw < bw_80) || (sec_5g_freq_max_bw < bw_80))
				goto update_bw;
		} else if (max_bw < reg_get_bw_value(ch_params->ch_width)) {
			goto update_bw;
		}

		bonded_chan_ptr = NULL;
		bonded_chan_ptr2 = NULL;
		chan_state = reg_get_5g_bonded_channel_for_freq(
				pdev, freq, ch_params->ch_width,
				&bonded_chan_ptr);

		chan_state = reg_get_5g_bonded_channel_state_for_freq(
				pdev, freq, ch_params->ch_width);

		if (ch_params->ch_width == CH_WIDTH_80P80MHZ) {
			chan_state2 = reg_get_5g_bonded_channel_state_for_freq(
					pdev, ch_params->mhz_freq_seg1 -
					NEAREST_20MHZ_CHAN_FREQ_OFFSET,
					CH_WIDTH_80MHZ);

			chan_state = reg_combine_channel_states(
					chan_state, chan_state2);
		}

		if ((chan_state != CHANNEL_STATE_ENABLE) &&
		    (chan_state != CHANNEL_STATE_DFS))
			goto update_bw;
		if (ch_params->ch_width <= CH_WIDTH_20MHZ) {
			ch_params->sec_ch_offset = NO_SEC_CH;
			ch_params->mhz_freq_seg0 = freq;
				ch_params->center_freq_seg0 =
				reg_freq_to_chan(pdev,
						 ch_params->mhz_freq_seg0);
			break;
		} else if (ch_params->ch_width >= CH_WIDTH_40MHZ) {
			bonded_chan_ptr2 =
				reg_get_bonded_chan_entry(freq, CH_WIDTH_40MHZ);

			if (!bonded_chan_ptr || !bonded_chan_ptr2)
				goto update_bw;
			if (freq == bonded_chan_ptr2->start_freq)
				ch_params->sec_ch_offset = LOW_PRIMARY_CH;
			else
				ch_params->sec_ch_offset = HIGH_PRIMARY_CH;

			ch_params->mhz_freq_seg0 =
				(bonded_chan_ptr->start_freq +
				 bonded_chan_ptr->end_freq) / 2;
				ch_params->center_freq_seg0 =
				reg_freq_to_chan(pdev,
						 ch_params->mhz_freq_seg0);
			break;
		}
update_bw:
		ch_params->ch_width =
		    get_next_lower_bandwidth(ch_params->ch_width);
	}

	if (ch_params->ch_width == CH_WIDTH_160MHZ) {
		ch_params->mhz_freq_seg1 = ch_params->mhz_freq_seg0;
			ch_params->center_freq_seg1 =
				reg_freq_to_chan(pdev,
						 ch_params->mhz_freq_seg1);

		chan_state = reg_get_5g_bonded_channel_for_freq(
				pdev, freq, CH_WIDTH_80MHZ, &bonded_chan_ptr);
		if (bonded_chan_ptr) {
			ch_params->mhz_freq_seg0 =
				(bonded_chan_ptr->start_freq +
				 bonded_chan_ptr->end_freq) / 2;
				ch_params->center_freq_seg0 =
				reg_freq_to_chan(pdev,
						 ch_params->mhz_freq_seg0);
		}
	}

	/* Overwrite mhz_freq_seg1 to 0 for non 160 and 80+80 width */
	if (!(ch_params->ch_width == CH_WIDTH_160MHZ ||
	      ch_params->ch_width == CH_WIDTH_80P80MHZ)) {
		ch_params->mhz_freq_seg1 = 0;
		ch_params->center_freq_seg1 = 0;
	}
}

void reg_set_2g_channel_params_for_freq(struct wlan_objmgr_pdev *pdev,
					uint16_t oper_freq,
					struct ch_params *ch_params,
					uint16_t sec_ch_2g_freq)
{
	enum channel_state chan_state = CHANNEL_STATE_ENABLE;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	enum channel_enum chan_enum;
	uint16_t max_bw;

	chan_enum = reg_get_chan_enum_for_freq(oper_freq);
	if (chan_enum == INVALID_CHANNEL) {
		reg_err("chan freq is not valid");
		return;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return;
	}

	if (ch_params->ch_width >= CH_WIDTH_MAX)
		ch_params->ch_width = CH_WIDTH_40MHZ;
	if ((reg_get_bw_value(ch_params->ch_width) > 20) && !sec_ch_2g_freq) {
		if (oper_freq >= TWOG_CHAN_1_IN_MHZ && oper_freq <=
				TWOG_CHAN_5_IN_MHZ)
			sec_ch_2g_freq = oper_freq + 20;
		else if (oper_freq >= TWOG_CHAN_6_IN_MHZ && oper_freq <=
				TWOG_CHAN_13_IN_MHZ)
			sec_ch_2g_freq = oper_freq - 20;
	}

	max_bw = pdev_priv_obj->cur_chan_list[chan_enum].max_bw;

	while (ch_params->ch_width != CH_WIDTH_INVALID) {
		if (max_bw < reg_get_bw_value(ch_params->ch_width))
			goto update_bw;

		chan_state =
		reg_get_2g_bonded_channel_state_for_freq(pdev, oper_freq,
							 sec_ch_2g_freq,
							 ch_params->ch_width);
		if ((chan_state == CHANNEL_STATE_ENABLE) ||
		    (chan_state == CHANNEL_STATE_DFS)) {
			if (ch_params->ch_width == CH_WIDTH_40MHZ) {
				if (oper_freq < sec_ch_2g_freq)
					ch_params->sec_ch_offset =
						LOW_PRIMARY_CH;
				else
					ch_params->sec_ch_offset =
						HIGH_PRIMARY_CH;
				ch_params->mhz_freq_seg0 =
					(oper_freq + sec_ch_2g_freq) / 2;
				if (ch_params->mhz_freq_seg0 ==
						TWOG_CHAN_14_IN_MHZ)
					ch_params->center_freq_seg0 = 14;
				else
					ch_params->center_freq_seg0 =
						(ch_params->mhz_freq_seg0 -
						 TWOG_STARTING_FREQ) /
						FREQ_TO_CHAN_SCALE;
			} else {
				ch_params->sec_ch_offset = NO_SEC_CH;
				ch_params->mhz_freq_seg0 = oper_freq;
				if (ch_params->mhz_freq_seg0 ==
						TWOG_CHAN_14_IN_MHZ)
					ch_params->center_freq_seg0 = 14;
				else
					ch_params->center_freq_seg0 =
						(ch_params->mhz_freq_seg0 -
						 TWOG_STARTING_FREQ) /
						FREQ_TO_CHAN_SCALE;
			}
			break;
		}
update_bw:
		ch_params->ch_width =
		    get_next_lower_bandwidth(ch_params->ch_width);
	}
	/* Overwrite mhz_freq_seg1 and center_freq_seg1 to 0 for 2.4 Ghz */
	ch_params->mhz_freq_seg1 = 0;
	ch_params->center_freq_seg1 = 0;
}

#if defined(WLAN_FEATURE_11BE) && defined(CONFIG_REG_CLIENT)
static void reg_copy_ch_params(struct ch_params *ch_params,
			       struct reg_channel_list chan_list)
{
	/* Taking only first set of chan params*/
	if (chan_list.chan_param[0].ch_width != CH_WIDTH_320MHZ)
		reg_info("coud not find ch_params for 320MHz downgrading to %d",
			 chan_list.chan_param[0].ch_width);
	ch_params->center_freq_seg0 = chan_list.chan_param[0].center_freq_seg0;
	ch_params->center_freq_seg1 = chan_list.chan_param[0].center_freq_seg1;
	ch_params->mhz_freq_seg0 = chan_list.chan_param[0].mhz_freq_seg0;
	ch_params->mhz_freq_seg1 = chan_list.chan_param[0].mhz_freq_seg1;
	ch_params->ch_width = chan_list.chan_param[0].ch_width;
	ch_params->sec_ch_offset = chan_list.chan_param[0].sec_ch_offset;
	ch_params->reg_punc_pattern = chan_list.chan_param[0].reg_punc_pattern;
}

void reg_set_channel_params_for_freq(struct wlan_objmgr_pdev *pdev,
				     qdf_freq_t freq,
				     qdf_freq_t sec_ch_2g_freq,
				     struct ch_params *ch_params)
{
	if (reg_is_5ghz_ch_freq(freq) || reg_is_6ghz_chan_freq(freq)) {
		if (ch_params->ch_width == CH_WIDTH_320MHZ) {
			struct reg_channel_list chan_list;

			qdf_mem_zero(&chan_list, sizeof(chan_list));
			/* For now sending center freq as 0 */
			reg_fill_channel_list(pdev, freq, sec_ch_2g_freq,
					      ch_params->ch_width, 0,
					      &chan_list);
			reg_copy_ch_params(ch_params, chan_list);
		} else {
			reg_set_5g_channel_params_for_freq(pdev, freq,
							   ch_params);
		}
	} else if  (reg_is_24ghz_ch_freq(freq)) {
		reg_set_2g_channel_params_for_freq(pdev, freq, ch_params,
						   sec_ch_2g_freq);
	}
}
#else
void reg_set_channel_params_for_freq(struct wlan_objmgr_pdev *pdev,
				     qdf_freq_t freq,
				     qdf_freq_t sec_ch_2g_freq,
				     struct ch_params *ch_params)
{
	if (reg_is_5ghz_ch_freq(freq) || reg_is_6ghz_chan_freq(freq))
		reg_set_5g_channel_params_for_freq(pdev, freq, ch_params);
	else if  (reg_is_24ghz_ch_freq(freq))
		reg_set_2g_channel_params_for_freq(pdev, freq, ch_params,
						   sec_ch_2g_freq);
}
#endif

uint8_t reg_get_channel_reg_power_for_freq(struct wlan_objmgr_pdev *pdev,
					   qdf_freq_t freq)
{
	enum channel_enum chan_enum;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	struct regulatory_channel *reg_channels;

	chan_enum = reg_get_chan_enum_for_freq(freq);

	if (chan_enum == INVALID_CHANNEL) {
		reg_err("channel is invalid");
		return REG_INVALID_TXPOWER;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return REG_INVALID_TXPOWER;
	}

	reg_channels = pdev_priv_obj->cur_chan_list;

	return reg_channels[chan_enum].tx_power;
}

bool reg_is_dfs_for_freq(struct wlan_objmgr_pdev *pdev, qdf_freq_t freq)
{
	uint32_t chan_flags;

	chan_flags = reg_get_channel_flags_for_freq(pdev, freq);

	return chan_flags & REGULATORY_CHAN_RADAR;
}

#ifdef CONFIG_REG_CLIENT
bool reg_is_dfs_in_secondary_list_for_freq(struct wlan_objmgr_pdev *pdev,
					   qdf_freq_t freq)
{
	uint32_t chan_flags;

	chan_flags = reg_get_channel_flags_from_secondary_list_for_freq(pdev,
									freq);

	return chan_flags & REGULATORY_CHAN_RADAR;
}

/**
 * reg_get_psoc_mas_chan_list () - Get psoc master channel list
 * @pdev: pointer to pdev object
 * @psoc: pointer to psoc object
 *
 * Return: psoc master chanel list
 */
static struct regulatory_channel *reg_get_psoc_mas_chan_list(
						struct wlan_objmgr_pdev *pdev,
						struct wlan_objmgr_psoc *psoc)
{
	struct wlan_regulatory_psoc_priv_obj *soc_reg;
	uint8_t pdev_id;
	uint8_t phy_id;
	struct wlan_lmac_if_reg_tx_ops *reg_tx_ops;

	soc_reg = reg_get_psoc_obj(psoc);
	if (!soc_reg) {
		reg_err("reg psoc private obj is NULL");
		return NULL;
	}
	pdev_id = wlan_objmgr_pdev_get_pdev_id(pdev);

	reg_tx_ops = reg_get_psoc_tx_ops(psoc);
	if (reg_tx_ops->get_phy_id_from_pdev_id)
		reg_tx_ops->get_phy_id_from_pdev_id(psoc, pdev_id, &phy_id);
	else
		phy_id = pdev_id;

	return soc_reg->mas_chan_params[phy_id].mas_chan_list;
}
#else
static inline struct regulatory_channel *reg_get_psoc_mas_chan_list(
						struct wlan_objmgr_pdev *pdev,
						struct wlan_objmgr_psoc *psoc)
{
	return NULL;
}
#endif

void reg_update_nol_ch_for_freq(struct wlan_objmgr_pdev *pdev,
				uint16_t *chan_freq_list,
				uint8_t num_chan,
				bool nol_chan)
{
	enum channel_enum chan_enum;
	struct regulatory_channel *mas_chan_list, *psoc_mas_chan_list;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	struct wlan_objmgr_psoc *psoc;
	uint16_t i;

	if (!num_chan || !chan_freq_list) {
		reg_err("chan_freq_list or num_ch is NULL");
		return;
	}

	psoc = wlan_pdev_get_psoc(pdev);

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!pdev_priv_obj) {
		reg_err("reg pdev private obj is NULL");
		return;
	}

	psoc_mas_chan_list = reg_get_psoc_mas_chan_list(pdev, psoc);

	mas_chan_list = pdev_priv_obj->mas_chan_list;
	for (i = 0; i < num_chan; i++) {
		chan_enum = reg_get_chan_enum_for_freq(chan_freq_list[i]);
		if (chan_enum == INVALID_CHANNEL) {
			reg_err("Invalid freq in nol list, freq %d",
				chan_freq_list[i]);
			continue;
		}
		mas_chan_list[chan_enum].nol_chan = nol_chan;
		if (psoc_mas_chan_list)
			psoc_mas_chan_list[chan_enum].nol_chan = nol_chan;
	}

	reg_compute_pdev_current_chan_list(pdev_priv_obj);

	reg_send_scheduler_msg_sb(psoc, pdev);
}

void reg_update_nol_history_ch_for_freq(struct wlan_objmgr_pdev *pdev,
					uint16_t *chan_list,
					uint8_t num_chan,
					bool nol_history_chan)
{
	enum channel_enum chan_enum;
	struct regulatory_channel *mas_chan_list;
	struct regulatory_channel *cur_chan_list;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	uint16_t i;

	if (!num_chan || !chan_list) {
		reg_err("chan_list or num_ch is NULL");
		return;
	}

	pdev_priv_obj = wlan_objmgr_pdev_get_comp_private_obj(
			pdev, WLAN_UMAC_COMP_REGULATORY);

	if (!pdev_priv_obj) {
		reg_err("reg psoc private obj is NULL");
		return;
	}

	mas_chan_list = pdev_priv_obj->mas_chan_list;
	cur_chan_list = pdev_priv_obj->cur_chan_list;

	for (i = 0; i < num_chan; i++) {
		chan_enum = reg_get_chan_enum_for_freq(chan_list[i]);
		if (chan_enum == INVALID_CHANNEL) {
			reg_err("Invalid ch in nol list, chan %d",
				chan_list[i]);
			continue;
		}
		mas_chan_list[chan_enum].nol_history = nol_history_chan;
		cur_chan_list[chan_enum].nol_history = nol_history_chan;
	}
}

static inline bool REG_IS_FREQUENCY_VALID_5G_SBS(qdf_freq_t curfreq,
						 qdf_freq_t newfreq)
{
	return ((curfreq) > (newfreq) ?
	REG_CH_TO_FREQ(reg_get_chan_enum_for_freq(curfreq))
	- REG_CH_TO_FREQ(reg_get_chan_enum_for_freq(newfreq))
	> REG_SBS_SEPARATION_THRESHOLD :
	REG_CH_TO_FREQ(reg_get_chan_enum_for_freq(newfreq))
	- REG_CH_TO_FREQ(reg_get_chan_enum_for_freq(curfreq))
	> REG_SBS_SEPARATION_THRESHOLD);
}

bool reg_is_frequency_valid_5g_sbs(qdf_freq_t curfreq, qdf_freq_t newfreq)
{
	return REG_IS_FREQUENCY_VALID_5G_SBS(curfreq, newfreq);
}

qdf_freq_t reg_min_chan_freq(void)
{
	return channel_map[MIN_24GHZ_CHANNEL].center_freq;
}

qdf_freq_t reg_max_chan_freq(void)
{
	return channel_map[NUM_CHANNELS - 1].center_freq;
}

bool reg_is_same_band_freqs(qdf_freq_t freq1, qdf_freq_t freq2)
{
	return (freq1 && freq2 && ((REG_IS_6GHZ_FREQ(freq1) &&
				    REG_IS_6GHZ_FREQ(freq2)) ||
				   (REG_IS_5GHZ_FREQ(freq1) &&
				    REG_IS_5GHZ_FREQ(freq2)) ||
				   (REG_IS_24GHZ_CH_FREQ(freq1) &&
				    REG_IS_24GHZ_CH_FREQ(freq2))));
}

enum reg_wifi_band reg_freq_to_band(qdf_freq_t freq)
{
	if (REG_IS_24GHZ_CH_FREQ(freq))
		return REG_BAND_2G;
	else if (REG_IS_5GHZ_FREQ(freq) || REG_IS_49GHZ_FREQ(freq))
		return REG_BAND_5G;
	else if (REG_IS_6GHZ_FREQ(freq))
		return REG_BAND_6G;
	return REG_BAND_UNKNOWN;
}

bool reg_is_disable_for_freq(struct wlan_objmgr_pdev *pdev, qdf_freq_t freq)
{
	enum channel_state ch_state;

	ch_state = reg_get_channel_state_for_freq(pdev, freq);

	return ch_state == CHANNEL_STATE_DISABLE;
}

#ifdef CONFIG_REG_CLIENT
bool reg_is_disable_in_secondary_list_for_freq(struct wlan_objmgr_pdev *pdev,
					       qdf_freq_t freq)
{
	enum channel_state ch_state;

	ch_state = reg_get_channel_state_from_secondary_list_for_freq(pdev,
								      freq);

	return ch_state == CHANNEL_STATE_DISABLE;
}
#endif

bool reg_is_passive_for_freq(struct wlan_objmgr_pdev *pdev, qdf_freq_t freq)
{
	uint32_t chan_flags;

	chan_flags = reg_get_channel_flags_for_freq(pdev, freq);

	return chan_flags & REGULATORY_CHAN_NO_IR;
}
#endif /* CONFIG_CHAN_FREQ_API */

uint8_t  reg_get_max_tx_power(struct wlan_objmgr_pdev *pdev)
{
	struct regulatory_channel *cur_chan_list;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	uint8_t i, max_tx_power = 0;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev private obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	cur_chan_list = pdev_priv_obj->cur_chan_list;

	for (i = 0; i < NUM_CHANNELS; i++) {
		if (cur_chan_list[i].state != CHANNEL_STATE_DISABLE &&
		    cur_chan_list[i].chan_flags != REGULATORY_CHAN_DISABLED) {
			if (cur_chan_list[i].tx_power > max_tx_power)
				max_tx_power = cur_chan_list[i].tx_power;
		}
	}

	if (!max_tx_power)
		reg_err_rl("max_tx_power is zero");

	return max_tx_power;
}

QDF_STATUS reg_set_ignore_fw_reg_offload_ind(struct wlan_objmgr_psoc *psoc)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_reg;

	psoc_reg = reg_get_psoc_obj(psoc);
	if (!IS_VALID_PSOC_REG_OBJ(psoc_reg)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_INVAL;
	}

	psoc_reg->ignore_fw_reg_offload_ind = true;
	return QDF_STATUS_SUCCESS;
}

bool reg_get_ignore_fw_reg_offload_ind(struct wlan_objmgr_psoc *psoc)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_reg;

	psoc_reg = reg_get_psoc_obj(psoc);
	if (!IS_VALID_PSOC_REG_OBJ(psoc_reg))
		return false;

	return psoc_reg->ignore_fw_reg_offload_ind;
}

QDF_STATUS reg_set_6ghz_supported(struct wlan_objmgr_psoc *psoc, bool val)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	psoc_priv_obj->six_ghz_supported = val;

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS
reg_set_5dot9_ghz_supported(struct wlan_objmgr_psoc *psoc, bool val)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	psoc_priv_obj->five_dot_nine_ghz_supported = val;

	return QDF_STATUS_SUCCESS;
}

#ifdef CONFIG_REG_CLIENT
bool reg_is_6ghz_supported(struct wlan_objmgr_psoc *psoc)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return  false;
	}

	return psoc_priv_obj->six_ghz_supported;
}
#endif

bool reg_is_5dot9_ghz_supported(struct wlan_objmgr_psoc *psoc)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return  false;
	}

	return psoc_priv_obj->five_dot_nine_ghz_supported;
}

bool reg_is_fcc_regdmn(struct wlan_objmgr_pdev *pdev)
{
	struct cur_regdmn_info cur_reg_dmn;
	QDF_STATUS status;

	status = reg_get_curr_regdomain(pdev, &cur_reg_dmn);
	if (status != QDF_STATUS_SUCCESS) {
		reg_err_rl("Failed to get reg domain");
		return false;
	}

	return reg_fcc_regdmn(cur_reg_dmn.dmn_id_5g);
}

bool reg_is_5dot9_ghz_freq(struct wlan_objmgr_pdev *pdev, qdf_freq_t freq)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return false;
	}

	return (freq >= channel_map_us[MIN_5DOT9_CHANNEL].center_freq &&
		freq <= channel_map_us[MAX_5DOT9_CHANNEL].center_freq);
}

bool reg_is_5dot9_ghz_chan_allowed_master_mode(struct wlan_objmgr_pdev *pdev)
{
	struct wlan_objmgr_psoc *psoc;
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	if (!pdev) {
		reg_alert("pdev is NULL");
		return true;
	}
	psoc = wlan_pdev_get_psoc(pdev);

	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_alert("psoc reg component is NULL");
		return true;
	}

	return psoc_priv_obj->enable_5dot9_ghz_chan_in_master_mode;
}

#ifdef DISABLE_UNII_SHARED_BANDS
QDF_STATUS
reg_get_unii_5g_bitmap(struct wlan_objmgr_pdev *pdev, uint8_t *bitmap)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err_rl("pdev reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}
	*bitmap =  pdev_priv_obj->unii_5g_bitmap;

	return QDF_STATUS_SUCCESS;
}
#endif

#ifdef WLAN_FEATURE_11BE
bool reg_is_phymode_unallowed(enum reg_phymode phy_in, uint32_t phymode_bitmap)
{
	if (!phymode_bitmap)
		return false;

	if (phy_in == REG_PHYMODE_11BE)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11BE;
	else if (phy_in == REG_PHYMODE_11AX)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11AX;
	else if (phy_in == REG_PHYMODE_11AC)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11AC;
	else if (phy_in == REG_PHYMODE_11N)
		return phymode_bitmap & REGULATORY_CHAN_NO11N;
	else if (phy_in == REG_PHYMODE_11G)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11G;
	else if (phy_in == REG_PHYMODE_11A)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11A;
	else if (phy_in == REG_PHYMODE_11B)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11B;
	else
		return true;
}
#else
bool reg_is_phymode_unallowed(enum reg_phymode phy_in, uint32_t phymode_bitmap)
{
	if (!phymode_bitmap)
		return false;

	if (phy_in == REG_PHYMODE_11AX)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11AX;
	else if (phy_in == REG_PHYMODE_11AC)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11AC;
	else if (phy_in == REG_PHYMODE_11N)
		return phymode_bitmap & REGULATORY_CHAN_NO11N;
	else if (phy_in == REG_PHYMODE_11G)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11G;
	else if (phy_in == REG_PHYMODE_11A)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11A;
	else if (phy_in == REG_PHYMODE_11B)
		return phymode_bitmap & REGULATORY_PHYMODE_NO11B;
	else
		return true;
}
#endif

#ifdef CHECK_REG_PHYMODE
enum reg_phymode reg_get_max_phymode(struct wlan_objmgr_pdev *pdev,
				     enum reg_phymode phy_in,
				     qdf_freq_t freq)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	uint32_t phymode_bitmap;
	enum reg_phymode current_phymode = phy_in;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return REG_PHYMODE_INVALID;
	}

	phymode_bitmap = pdev_priv_obj->phybitmap;

	while (1) {
		if (reg_is_phymode_unallowed(current_phymode, phymode_bitmap)) {
			if (current_phymode == REG_PHYMODE_11N) {
				if (REG_IS_24GHZ_CH_FREQ(freq))
					current_phymode = REG_PHYMODE_11G;
				else
					current_phymode = REG_PHYMODE_11A;
			} else if (current_phymode == REG_PHYMODE_11A ||
				   current_phymode == REG_PHYMODE_11B) {
				reg_err("Couldn't find a suitable phymode");
				return REG_PHYMODE_INVALID;
			} else if (current_phymode > REG_PHYMODE_MAX) {
				reg_err("Unknown phymode");
				return REG_PHYMODE_INVALID;
			} else {
				current_phymode--;
			}
		} else {
			return current_phymode;
		}
	}
}
#endif /* CHECK_REG_PHYMODE */

#ifdef CONFIG_REG_CLIENT
enum band_info reg_band_bitmap_to_band_info(uint32_t band_bitmap)
{
	if ((band_bitmap & BIT(REG_BAND_2G)) &&
	    (band_bitmap & BIT(REG_BAND_5G)) &&
	    (band_bitmap & BIT(REG_BAND_6G)))
		return BAND_ALL;
	else if ((band_bitmap & BIT(REG_BAND_5G)) &&
		 (band_bitmap & BIT(REG_BAND_6G)))
		return BAND_5G;
	else if ((band_bitmap & BIT(REG_BAND_2G)) &&
		 (band_bitmap & BIT(REG_BAND_6G)))
		return BAND_2G;
	else if ((band_bitmap & BIT(REG_BAND_2G)) &&
		 (band_bitmap & BIT(REG_BAND_5G)))
		return BAND_ALL;
	else if (band_bitmap & BIT(REG_BAND_2G))
		return BAND_2G;
	else if (band_bitmap & BIT(REG_BAND_5G))
		return BAND_5G;
	else if (band_bitmap & BIT(REG_BAND_6G))
		return BAND_2G;
	else
		return BAND_UNKNOWN;
}
#endif

#if defined(CONFIG_BAND_6GHZ)
QDF_STATUS
reg_set_cur_6g_ap_pwr_type(struct wlan_objmgr_pdev *pdev,
			   enum reg_6g_ap_type reg_cur_6g_ap_pwr_type)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	if (reg_cur_6g_ap_pwr_type > REG_MAX_SUPP_AP_TYPE) {
		reg_err("Unsupported 6G AP power type");
		return QDF_STATUS_E_FAILURE;
	}
	/* should we validate the input reg_cur_6g_ap_type? */
	pdev_priv_obj->reg_cur_6g_ap_pwr_type = reg_cur_6g_ap_pwr_type;
	return QDF_STATUS_SUCCESS;
}

QDF_STATUS
reg_get_cur_6g_ap_pwr_type(struct wlan_objmgr_pdev *pdev,
			   enum reg_6g_ap_type *reg_cur_6g_ap_pwr_type)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	if (pdev_priv_obj->reg_cur_6g_ap_pwr_type >= REG_CURRENT_MAX_AP_TYPE)
		return QDF_STATUS_E_FAILURE;

	*reg_cur_6g_ap_pwr_type = pdev_priv_obj->reg_cur_6g_ap_pwr_type;

	return QDF_STATUS_SUCCESS;
}

/**
 * get_reg_rules_for_pdev() - Get the pointer to the reg rules for the pdev
 * @pdev: Pointer to pdev
 *
 * Return: Pointer to Standard Power regulatory rules
 */
static struct reg_rule_info *
reg_get_reg_rules_for_pdev(struct wlan_objmgr_pdev *pdev)
{
	struct wlan_objmgr_psoc *psoc;
	struct wlan_regulatory_psoc_priv_obj *psoc_reg_priv;
	uint8_t phy_id;
	struct reg_rule_info *psoc_reg_rules;

	psoc = wlan_pdev_get_psoc(pdev);
	psoc_reg_priv = reg_get_psoc_obj(psoc);
	phy_id = wlan_objmgr_pdev_get_pdev_id(pdev);
	psoc_reg_rules = &psoc_reg_priv->mas_chan_params[phy_id].reg_rules;

	return psoc_reg_rules;
}

#ifdef CONFIG_AFC_SUPPORT
/**
 * reg_is_empty_range() - If both left, right frquency edges in the input range
 * are zero then the range is empty, else not.
 * @in_range: Pointer to input range
 *
 * Return: True if the range is empty, else false
 */
static bool reg_is_empty_range(struct freq_range *in_range)
{
	return !in_range->left && !in_range->right;
}

struct freq_range
reg_init_freq_range(qdf_freq_t left, qdf_freq_t right)
{
	struct freq_range out_range;

	out_range.left = left;
	out_range.right = right;

	return out_range;
}

/**
 * reg_assign_vars_with_range_vals() - Assign input variables with the values of
 * the range variable values
 * @in_range: Pointer to input range object
 * @left: Pointer to the first variable to get the value of left frequency edge
 * @right: Pointer to the second variable to get the value of right frequency
 *         edge
 *
 * Return: void
 */
static void
reg_assign_vars_with_range_vals(struct freq_range *in_range,
				qdf_freq_t *left,
				qdf_freq_t *right)
{
	*left = in_range->left;
	*right = in_range->right;
}

/**
 * reg_intersect_ranges() - Intersect two ranges and return the intesected range
 * @first: Pointer to first input range
 * @second: Pointer to second input range
 *
 * Return: Intersected output range
 */
static struct freq_range
reg_intersect_ranges(struct freq_range *first_range,
		     struct freq_range *second_range)
{
	struct freq_range out_range;
	qdf_freq_t l_freq;
	qdf_freq_t r_freq;

	/* validate if the ranges are proper */

	l_freq = QDF_MAX(first_range->left, second_range->left);
	r_freq = QDF_MIN(first_range->right, second_range->right);

	if (l_freq > r_freq) {
		l_freq = 0;
		l_freq = 0;

		reg_debug("Ranges do not overlap first= [%u, %u], second = [%u, %u]",
			  first_range->left,
			  first_range->right,
			  second_range->left,
			  second_range->right);
	}

	out_range.left = l_freq;
	out_range.right = r_freq;

	return out_range;
}

/**
 * reg_act_sp_rule_cb -  A function pointer type that calculate something
 * from the input frequency range
 * @rule_fr: Pointer to frequencey range
 * @arg: Pointer to generic argument (a.k.a. context)
 *
 * Return: Void
 */
typedef void (*reg_act_sp_rule_cb)(struct freq_range *rule_fr,
				   void *arg);

/**
 * reg_iterate_sp_rules() - Iterate through the Standard Power reg rules, for
 * every reg rule call the call back function to take some action or calculate
 * something
 * @pdev: Pointer to pdev
 * @pdev_priv_obj: Pointer to pdev private object
 * @action_on_sp_rule: A function pointer to take some action or calculate
 * something for every sp rule
 * @arg: Pointer to opque object (argument/context)
 *
 * Return: Void
 */
static void reg_iterate_sp_rules(struct wlan_objmgr_pdev *pdev,
				 struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj,
				 reg_act_sp_rule_cb sp_rule_action,
				 void *arg)
{
	struct cur_reg_rule *p_sp_reg_rule;
	struct reg_rule_info *psoc_reg_rules;
	uint8_t n_6g_sp_ap_reg_rules;
	qdf_freq_t low_5g;
	qdf_freq_t high_5g;
	uint8_t i;
	struct freq_range chip_range;

	psoc_reg_rules = reg_get_reg_rules_for_pdev(pdev);
	n_6g_sp_ap_reg_rules = psoc_reg_rules->num_of_6g_ap_reg_rules[REG_STANDARD_POWER_AP];
	p_sp_reg_rule = psoc_reg_rules->reg_rules_6g_ap[REG_STANDARD_POWER_AP];

	low_5g = pdev_priv_obj->range_5g_low;
	high_5g = pdev_priv_obj->range_5g_high;

	chip_range = reg_init_freq_range(low_5g, high_5g);

	reg_debug("chip_range = [%u, %u]", low_5g, high_5g);
	reg_debug("Num_6g_rules = %u", n_6g_sp_ap_reg_rules);

	for (i = 0; i < n_6g_sp_ap_reg_rules; i++) {
		struct freq_range sp_range;
		struct freq_range out_range;

		sp_range = reg_init_freq_range(p_sp_reg_rule->start_freq,
					       p_sp_reg_rule->end_freq);
		reg_debug("Rule:[%u, %u]",
			  p_sp_reg_rule->start_freq,
			  p_sp_reg_rule->end_freq);
		out_range = reg_intersect_ranges(&chip_range, &sp_range);

		if (sp_rule_action)
			sp_rule_action(&out_range, arg);

		p_sp_reg_rule++;
	}
}

/**
 * reg_afc_incr_num_ranges() - Increment the number of frequency ranges
 * @p_range: Pointer to frequency range
 * @num_freq_ranges: Pointer to number of frequency ranges. This needs to be
 * (Actual type: uint8_t *num_freq_ranges)
 * incremented by the function
 *
 * Return: Void
 */
static void reg_afc_incr_num_ranges(struct freq_range *p_range,
				    void *num_freq_ranges)
{
	if (!reg_is_empty_range(p_range))
		(*(uint8_t *)num_freq_ranges)++;
}

/**
 * reg_get_num_sp_freq_ranges() - Find the number of reg rules from the Standard
 * power regulatory rules
 * @pdev: Pointer to pdev
 *
 * Return: number of frequency ranges
 */
static uint8_t reg_get_num_sp_freq_ranges(struct wlan_objmgr_pdev *pdev,
					  struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj)
{
	uint8_t num_freq_ranges;

	num_freq_ranges = 0;
	reg_iterate_sp_rules(pdev,
			     pdev_priv_obj,
			     reg_afc_incr_num_ranges,
			     &num_freq_ranges);

	reg_debug("Num_freq_ranges=%u", num_freq_ranges);
	return num_freq_ranges;
}

/**
 * reg_afc_get_intersected_ranges() - Get the intersected range into range obj
 * @rule_fr: Pointer to the rule for frequency range
 * @arg: Pointer to opaque object (argument/context)
 * (Actual type: struct wlan_afc_freq_range_obj **p_range_obj)
 * incremented by the function
 *
 * Return: Void
 */
static void reg_afc_get_intersected_ranges(struct freq_range *rule_fr,
					   void *arg)
{
	struct wlan_afc_freq_range_obj *p_range;
	struct wlan_afc_freq_range_obj **pp_range;

	pp_range = (struct wlan_afc_freq_range_obj **)arg;
	p_range = *pp_range;

	if (!reg_is_empty_range(rule_fr)) {
		reg_assign_vars_with_range_vals(rule_fr,
						&p_range->lowfreq,
						&p_range->highfreq);
		reg_debug("Range = [%u, %u]", p_range->lowfreq, p_range->highfreq);
		(*pp_range)++;
	}
}

/**
 * reg_cp_freq_ranges() - Copy frequency ranges  from the Standard power
 * regulatory rules
 * @pdev: Pointer to pdev
 * @pdev_priv_obj: Pointer to pdev private object
 * @num_freq_ranges: Number of frequency ranges
 * @p_range_obj: Pointer to range object
 *
 * Return: void
 */
static void reg_cp_freq_ranges(struct wlan_objmgr_pdev *pdev,
			       struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj,
			       uint8_t num_freq_ranges,
			       struct wlan_afc_freq_range_obj *p_range_obj)
{
	struct wlan_afc_freq_range_obj *p_range;

	reg_debug("Num freq ranges = %u", num_freq_ranges);

	p_range = p_range_obj;
	reg_iterate_sp_rules(pdev,
			     pdev_priv_obj,
			     reg_afc_get_intersected_ranges,
			     &p_range);
}

/**
 * reg_get_frange_list_len() - Calculate the length of the list of the
 * frequency ranges
 * @num_freq_ranges: Number of frequency ranges
 *
 * Return: Length of the frequency range list
 */
static uint16_t reg_get_frange_list_len(uint8_t num_freq_ranges)
{
	uint16_t frange_lst_len;

	if (!num_freq_ranges)
		reg_err("AFC:There is no freq ranges");

	frange_lst_len =
		sizeof(struct wlan_afc_frange_list) +
		sizeof(struct wlan_afc_freq_range_obj) * num_freq_ranges;

	return frange_lst_len;
}

/**
 * reg_get_opclasses_array_len() - Calculate the length of the array of
 * opclasses objects
 * @num_opclasses: The number of opclasses
 * @chansize_lst: The array of sizes of channel lists
 *
 * Return: Length of the array of opclass object
 */
static uint16_t reg_get_opclasses_array_len(struct wlan_objmgr_pdev *pdev,
					    uint8_t num_opclasses,
					    uint8_t *chansize_lst)
{
	uint16_t opclasses_arr_len = 0;
	uint16_t i;

	for (i = 0; i < num_opclasses; i++) {
		opclasses_arr_len +=
			sizeof(struct wlan_afc_opclass_obj) +
			sizeof(uint8_t) * chansize_lst[i];
	}

	return opclasses_arr_len;
}

/**
 * reg_get_afc_req_length() - Calculate the length of the AFC partial request
 * @num_opclasses: The number of opclasses
 * @num_freq_ranges: The number of frequency ranges
 * @chansize_lst: The array of sizes of channel lists
 *
 * Return: Length of the partial AFC request
 */
static uint16_t reg_get_afc_req_length(struct wlan_objmgr_pdev *pdev,
				       uint8_t num_opclasses,
				       uint8_t num_freq_ranges,
				       uint8_t *chansize_lst)
{
	uint16_t afc_req_len;
	uint16_t frange_lst_len;
	uint16_t fixed_param_len;
	uint16_t num_opclasses_len;
	uint16_t opclasses_arr_len;

	fixed_param_len = sizeof(struct wlan_afc_host_req_fixed_params);
	frange_lst_len = reg_get_frange_list_len(num_freq_ranges);
	num_opclasses_len = sizeof(struct wlan_afc_num_opclasses);
	opclasses_arr_len = reg_get_opclasses_array_len(pdev,
							num_opclasses,
							chansize_lst);

	afc_req_len =
		fixed_param_len +
		frange_lst_len +
		num_opclasses_len +
		opclasses_arr_len;

	return afc_req_len;
}

/**
 * reg_fill_afc_fixed_params() - Fill the AFC fixed params
 * @p_fixed_params: Pointer to afc fixed params object
 * @afc_req_len: Length of the partial AFC request
 *
 * Return: Void
 */
static inline void
reg_fill_afc_fixed_params(struct wlan_afc_host_req_fixed_params *p_fixed_params,
			  uint16_t afc_req_len)
{
	p_fixed_params->req_length = afc_req_len;
	p_fixed_params->req_id = DEFAULT_REQ_ID;
	p_fixed_params->min_des_power = DEFAULT_MIN_POWER;
}

/**
 * reg_fill_afc_freq_ranges() - Fill the AFC fixed params
 * @pdev: Pointer to pdev
 * @pdev_priv_obj: Pointer to pdev private object
 * @p_frange_lst: Pointer to frequency range list
 * @num_freq_ranges: Number of frequency ranges
 *
 * Return: Void
 */
static inline void
reg_fill_afc_freq_ranges(struct wlan_objmgr_pdev *pdev,
			 struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj,
			 struct wlan_afc_frange_list *p_frange_lst,
			 uint8_t num_freq_ranges)
{
	struct wlan_afc_freq_range_obj *p_range_obj;

	p_frange_lst->num_ranges = num_freq_ranges;

	p_range_obj = &p_frange_lst->range_objs[0];

	reg_cp_freq_ranges(pdev, pdev_priv_obj, num_freq_ranges, p_range_obj);
}

/**
 * reg_fill_afc_opclass_obj() - Fill the opclass object and return pointer to
 *                              next AFC opclass object
 * @p_obj_opclass_obj: Pointer to opclass object
 * @opclass: Operating class
 * @num_chans: Number of channels in the opclass
 * @p_chan_lst: Pointer to channel list
 *
 * Return: Pointer to the next AFC opclass object
 */
static struct wlan_afc_opclass_obj *
reg_fill_afc_opclass_obj(struct wlan_afc_opclass_obj *p_obj_opclass_obj,
			 uint8_t opclass,
			 uint8_t num_chans,
			 uint8_t *p_chan_lst)
{
	uint16_t len_obj;
	uint8_t *out_p;
	uint8_t *src, *dst;
	uint8_t copy_len;

	p_obj_opclass_obj->opclass_num_cfis = num_chans;
	p_obj_opclass_obj->opclass = opclass;
	src = p_chan_lst;
	dst = p_obj_opclass_obj->cfis;
	copy_len = num_chans * sizeof(uint8_t);

	qdf_mem_copy(dst, src, copy_len);

	len_obj = sizeof(struct wlan_afc_opclass_obj) + copy_len;
	out_p = (uint8_t *)p_obj_opclass_obj + len_obj;

	return (struct wlan_afc_opclass_obj *)out_p;
}

/**
 * reg_fill_afc_opclasses_arr() - Fill the array of opclass objects
 * @num_opclasses: The number of opclasses
 * @opclass_lst: The array of Operating classes
 * @chansize_lst: The array of sizes of channel lists
 * @channel_lists: The array of channel lists
 * @p_opclass_obj_arr: Pointer to the first opclass object
 *
 * Return: Void
 */
static inline void
reg_fill_afc_opclasses_arr(struct wlan_objmgr_pdev *pdev,
			   uint8_t num_opclasses,
			   uint8_t *opclass_lst,
			   uint8_t *chansize_lst,
			   uint8_t *channel_lists[],
			   struct wlan_afc_opclass_obj *p_opclass_obj_arr)
{
	uint16_t i;
	struct wlan_afc_opclass_obj *p_opclass_obj;

	p_opclass_obj = p_opclass_obj_arr;

	for (i = 0; i < num_opclasses; i++) {
		p_opclass_obj = reg_fill_afc_opclass_obj(p_opclass_obj,
							 opclass_lst[i],
							 chansize_lst[i],
							 channel_lists[i]);
	}
}

/**
 * reg_next_opcls_ptr() - Get the pointer to the next opclass object
 * @p_cur_opcls_obj: Pointer to the current operating class object
 * @num_cfis: number of center frequencey indices
 *
 * Return: Pointer to next opclss object
 */
static struct wlan_afc_opclass_obj *
reg_next_opcls_ptr(struct wlan_afc_opclass_obj *p_cur_opcls_obj,
		   uint8_t num_cfis)
{
	uint8_t cur_obj_sz;
	uint8_t fixed_opcls_sz;
	struct wlan_afc_opclass_obj *p_next_opcls_obj;
	uint8_t *p_tmp_next;

	fixed_opcls_sz = sizeof(struct wlan_afc_opclass_obj);
	cur_obj_sz = fixed_opcls_sz + num_cfis * sizeof(uint8_t);
	p_tmp_next = (uint8_t *)p_cur_opcls_obj + cur_obj_sz;
	p_next_opcls_obj = (struct wlan_afc_opclass_obj *)p_tmp_next;

	return p_next_opcls_obj;
}

void reg_print_partial_afc_req_info(struct wlan_objmgr_pdev *pdev,
				    struct wlan_afc_host_partial_request *afc_req)
{
	struct wlan_afc_host_req_fixed_params *p_fixed_params;
	struct wlan_afc_frange_list *p_frange_lst;
	struct wlan_afc_num_opclasses *p_num_opclasses;
	uint8_t i;
	uint8_t j;
	uint16_t frange_lst_len;
	uint8_t num_opclasses;
	struct wlan_afc_opclass_obj *p_obj_opclass_arr;
	struct wlan_afc_opclass_obj *p_opclass_obj;
	uint8_t num_freq_ranges;
	uint8_t *p_temp;

	p_fixed_params = &afc_req->fixed_params;
	reg_debug("req_length=%hu", p_fixed_params->req_length);
	reg_debug("req_id=%llu", p_fixed_params->req_id);
	reg_debug("min_des_power=%hd", p_fixed_params->min_des_power);

	p_temp = (uint8_t *)p_fixed_params;
	p_temp += sizeof(*p_fixed_params);
	p_frange_lst = (struct wlan_afc_frange_list *)p_temp;
	reg_debug("num_ranges=%hhu", p_frange_lst->num_ranges);
	for (i = 0; i < p_frange_lst->num_ranges; i++) {
		struct wlan_afc_freq_range_obj *p_range_obj;

		p_range_obj = &p_frange_lst->range_objs[i];
		reg_debug("lowfreq=%hu", p_range_obj->lowfreq);
		reg_debug("highfreq=%hu", p_range_obj->highfreq);
	}

	num_freq_ranges = p_frange_lst->num_ranges;
	frange_lst_len = reg_get_frange_list_len(num_freq_ranges);
	p_temp += frange_lst_len;
	p_num_opclasses = (struct wlan_afc_num_opclasses *)p_temp;
	num_opclasses = p_num_opclasses->num_opclasses;
	reg_debug("num_opclasses=%hhu", num_opclasses);

	p_temp += sizeof(*p_num_opclasses);
	p_obj_opclass_arr = (struct wlan_afc_opclass_obj *)p_temp;
	p_opclass_obj = p_obj_opclass_arr;
	for (i = 0; i < num_opclasses; i++) {
		uint8_t opclass = p_opclass_obj->opclass;
		uint8_t num_cfis = p_opclass_obj->opclass_num_cfis;
		uint8_t *cfis = p_opclass_obj->cfis;

		reg_debug("opclass[%hhu]=%hhu", i, opclass);
		reg_debug("num_cfis[%hhu]=%hhu", i, num_cfis);
		reg_debug("[");
		for (j = 0; j < num_cfis; j++)
			reg_debug("%hhu,", cfis[j]);
		reg_debug("]");

		p_opclass_obj = reg_next_opcls_ptr(p_opclass_obj, num_cfis);
	}
}

/**
 * reg_get_frange_filled_buf() - Allocate and fill the frange buffer and return
 * the buffer. Also return the number of frequence ranges
 * @pdev: Pointer to pdev
 * @pdev_priv_obj: Pointer to pdev private object
 * @num_freq_ranges: Pointer to number of frequency ranges (output param)
 *
 * Return: Pointer to the frange buffer
 */
static struct wlan_afc_frange_list *
reg_get_frange_filled_buf(struct wlan_objmgr_pdev *pdev,
			  struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj,
			  uint8_t *num_freq_ranges)
{
	uint16_t frange_lst_len;
	struct wlan_afc_frange_list *p_frange_lst_local;

	*num_freq_ranges =  reg_get_num_sp_freq_ranges(pdev, pdev_priv_obj);
	frange_lst_len = reg_get_frange_list_len(*num_freq_ranges);

	p_frange_lst_local = qdf_mem_malloc(frange_lst_len);
	if (!p_frange_lst_local)
		return NULL;

	reg_fill_afc_freq_ranges(pdev,
				 pdev_priv_obj,
				 p_frange_lst_local,
				 *num_freq_ranges);
	return p_frange_lst_local;
}

QDF_STATUS
reg_get_partial_afc_req_info(struct wlan_objmgr_pdev *pdev,
			     struct wlan_afc_host_partial_request **afc_req)
{
	/* allocate the memory for the partial request */
	struct wlan_afc_host_partial_request *temp_afc_req;
	struct wlan_afc_host_req_fixed_params *p_fixed_params;
	struct wlan_afc_frange_list *p_frange_lst_local;
	struct wlan_afc_frange_list *p_frange_lst_afc;
	struct wlan_afc_num_opclasses *p_num_opclasses;
	uint16_t afc_req_len;
	uint16_t frange_lst_len;
	uint8_t num_freq_ranges;
	uint8_t num_opclasses;
	struct wlan_afc_opclass_obj *p_obj_opclass_arr;

	uint8_t *opclass_lst;
	uint8_t *chansize_lst;
	uint8_t **channel_lists;
	QDF_STATUS status;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	temp_afc_req = NULL;
	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		status = QDF_STATUS_E_INVAL;
		goto handle_invalid_priv_object;
	}

	p_frange_lst_local = reg_get_frange_filled_buf(pdev,
						       pdev_priv_obj,
						       &num_freq_ranges);
	if (!p_frange_lst_local) {
		reg_err("Frange lst not allocated");
		status = QDF_STATUS_E_NOMEM;
		goto handle_invalid_priv_object;
	}

	status = reg_dmn_get_6g_opclasses_and_channels(pdev,
						       p_frange_lst_local,
						       &num_opclasses,
						       &opclass_lst,
						       &chansize_lst,
						       &channel_lists);
	if (status != QDF_STATUS_SUCCESS) {
		reg_err("Opclasses and chans not allocated");
		status = QDF_STATUS_E_NOMEM;
		goto free_frange_lst_local;
	}

	afc_req_len = reg_get_afc_req_length(pdev,
					     num_opclasses,
					     num_freq_ranges,
					     chansize_lst);

	temp_afc_req = qdf_mem_malloc(afc_req_len);
	if (!temp_afc_req) {
		reg_err("AFC request not allocated");
		status = QDF_STATUS_E_NOMEM;
		goto free_opcls_chan_mem;
	}

	p_fixed_params = &temp_afc_req->fixed_params;
	reg_fill_afc_fixed_params(p_fixed_params, afc_req_len);

	/* frange list is already filled just copy it */
	frange_lst_len = reg_get_frange_list_len(num_freq_ranges);
	p_frange_lst_afc = (struct wlan_afc_frange_list *)&p_fixed_params[1];
	qdf_mem_copy(p_frange_lst_afc, p_frange_lst_local, frange_lst_len);

	p_num_opclasses = (struct wlan_afc_num_opclasses *)
	    ((char *)(p_frange_lst_afc) + frange_lst_len);
	p_num_opclasses->num_opclasses = num_opclasses;

	p_obj_opclass_arr = (struct wlan_afc_opclass_obj *)&p_num_opclasses[1];
	reg_fill_afc_opclasses_arr(pdev,
				   num_opclasses,
				   opclass_lst,
				   chansize_lst,
				   channel_lists,
				   p_obj_opclass_arr);
free_opcls_chan_mem:
	reg_dmn_free_6g_opclasses_and_channels(pdev,
					       num_opclasses,
					       opclass_lst,
					       chansize_lst,
					       channel_lists);

free_frange_lst_local:
	qdf_mem_free(p_frange_lst_local);

handle_invalid_priv_object:
	*afc_req = temp_afc_req;

	return status;
}

void reg_dmn_set_afc_req_id(struct wlan_afc_host_partial_request *afc_req,
			    uint64_t req_id)
{
	struct wlan_afc_host_req_fixed_params *p_fixed_params;

	p_fixed_params = &afc_req->fixed_params;
	p_fixed_params->req_id = req_id;
}

/**
 * reg_send_afc_partial_request() - Send AFC partial request to registered
 * recipient
 * @pdev: Pointer to pdev
 * @afc_req: Pointer to afc partial request
 *
 * Return: void
 */
static
void reg_send_afc_partial_request(struct wlan_objmgr_pdev *pdev,
				  struct wlan_afc_host_partial_request *afc_req)
{
	afc_req_rx_evt_handler cbf;
	void *arg;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return;
	}

	qdf_spin_lock_bh(&pdev_priv_obj->afc_cb_lock);
	cbf = pdev_priv_obj->afc_cb_obj.func;
	if (cbf) {
		arg = pdev_priv_obj->afc_cb_obj.arg;
		cbf(pdev, afc_req, arg);
	}
	qdf_spin_unlock_bh(&pdev_priv_obj->afc_cb_lock);
}

QDF_STATUS reg_afc_start(struct wlan_objmgr_pdev *pdev, uint64_t req_id)
{
	struct wlan_afc_host_partial_request *afc_req;
	QDF_STATUS status;

	status = reg_get_partial_afc_req_info(pdev, &afc_req);
	if (status != QDF_STATUS_SUCCESS) {
		reg_err("Creating AFC Request failed");
		return QDF_STATUS_E_FAILURE;
	}

	reg_dmn_set_afc_req_id(afc_req, req_id);

	reg_print_partial_afc_req_info(pdev, afc_req);

	reg_send_afc_partial_request(pdev, afc_req);

	qdf_mem_free(afc_req);

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_register_afc_req_rx_callback(struct wlan_objmgr_pdev *pdev,
					    afc_req_rx_evt_handler cbf,
					    void *arg)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	qdf_spin_lock_bh(&pdev_priv_obj->afc_cb_lock);
	pdev_priv_obj->afc_cb_obj.func = cbf;
	pdev_priv_obj->afc_cb_obj.arg = arg;
	qdf_spin_unlock_bh(&pdev_priv_obj->afc_cb_lock);
	reg_debug("afc_event_cb: 0x%pK, arg: 0x%pK", cbf, arg);

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_unregister_afc_req_rx_callback(struct wlan_objmgr_pdev *pdev,
					      afc_req_rx_evt_handler cbf)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	qdf_spin_lock_bh(&pdev_priv_obj->afc_cb_lock);
	if (pdev_priv_obj->afc_cb_obj.func == cbf) {
		pdev_priv_obj->afc_cb_obj.func = NULL;
		pdev_priv_obj->afc_cb_obj.arg = NULL;
	} else {
		reg_err("cb function=0x%pK not found", cbf);
	}
	qdf_spin_unlock_bh(&pdev_priv_obj->afc_cb_lock);

	return QDF_STATUS_SUCCESS;
}

#endif /* CONFIG_AFC_SUPPORT */

QDF_STATUS
reg_get_cur_6g_client_type(struct wlan_objmgr_pdev *pdev,
			   enum reg_6g_client_type
			   *reg_cur_6g_client_mobility_type)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	if (pdev_priv_obj->reg_cur_6g_client_mobility_type >=
	    REG_MAX_CLIENT_TYPE)
		return QDF_STATUS_E_FAILURE;

	*reg_cur_6g_client_mobility_type =
	    pdev_priv_obj->reg_cur_6g_client_mobility_type;

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_get_rnr_tpe_usable(struct wlan_objmgr_pdev *pdev,
				  bool *reg_rnr_tpe_usable)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}
	*reg_rnr_tpe_usable = pdev_priv_obj->reg_rnr_tpe_usable;
	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_get_unspecified_ap_usable(struct wlan_objmgr_pdev *pdev,
					 bool *reg_unspecified_ap_usable)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}
	*reg_unspecified_ap_usable = pdev_priv_obj->reg_unspecified_ap_usable;
	return QDF_STATUS_SUCCESS;
}

QDF_STATUS
reg_find_txpower_from_6g_list(qdf_freq_t freq,
			      struct regulatory_channel *chan_list,
			      uint16_t *txpower)
{
	enum channel_enum chan_enum;

	*txpower = 0;

	for (chan_enum = 0; chan_enum < NUM_6GHZ_CHANNELS; chan_enum++) {
		if (chan_list[chan_enum].center_freq == freq) {
			*txpower = chan_list[chan_enum].tx_power;
			return QDF_STATUS_SUCCESS;
		}
	}

	return QDF_STATUS_E_FAILURE;
}

bool reg_is_6g_psd_power(struct wlan_objmgr_pdev *pdev)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	struct regulatory_channel *cur_chan_list;
	enum channel_enum i;

	if (!pdev) {
		reg_err("pdev is NULL");
		return false;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!pdev_priv_obj) {
		reg_err("pdev priv obj is NULL");
		return false;
	}

	cur_chan_list = pdev_priv_obj->cur_chan_list;

	for (i = MIN_6GHZ_CHANNEL; i <= MAX_6GHZ_CHANNEL; i++) {
		if (!(cur_chan_list[i].chan_flags & REGULATORY_CHAN_DISABLED))
			return cur_chan_list[i].psd_flag;
	}

	return false;
}

QDF_STATUS
reg_get_6g_chan_psd_eirp_power(qdf_freq_t freq,
			       struct regulatory_channel *mas_chan_list,
			       uint16_t *eirp_psd_power)
{
	uint16_t i;

	for (i = 0; i < NUM_6GHZ_CHANNELS; i++) {
		if (freq == mas_chan_list[i].center_freq) {
			*eirp_psd_power = mas_chan_list[i].psd_eirp;
			return QDF_STATUS_SUCCESS;
		}
	}

	return QDF_STATUS_E_FAILURE;
}

QDF_STATUS reg_get_6g_chan_ap_power(struct wlan_objmgr_pdev *pdev,
				    qdf_freq_t chan_freq, bool *is_psd,
				    uint16_t *tx_power,
				    uint16_t *eirp_psd_power)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	struct regulatory_channel *master_chan_list;
	enum reg_6g_ap_type ap_pwr_type;
	QDF_STATUS status = QDF_STATUS_SUCCESS;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!pdev_priv_obj) {
		reg_err("pdev priv obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	status = reg_get_cur_6g_ap_pwr_type(pdev, &ap_pwr_type);
	if (!QDF_IS_STATUS_SUCCESS(status))
		return status;

	master_chan_list = pdev_priv_obj->mas_chan_list_6g_ap[ap_pwr_type];

	reg_find_txpower_from_6g_list(chan_freq, master_chan_list,
				      tx_power);

	*is_psd = reg_is_6g_psd_power(pdev);
	if (*is_psd)
		status = reg_get_6g_chan_psd_eirp_power(chan_freq,
							master_chan_list,
							eirp_psd_power);

	return status;
}

QDF_STATUS reg_get_client_power_for_connecting_ap(struct wlan_objmgr_pdev *pdev,
						  enum reg_6g_ap_type ap_type,
						  qdf_freq_t chan_freq,
						  bool *is_psd,
						  uint16_t *tx_power,
						  uint16_t *eirp_psd_power)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	enum reg_6g_client_type client_type;
	struct regulatory_channel *master_chan_list;
	QDF_STATUS status = QDF_STATUS_SUCCESS;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!pdev_priv_obj) {
		reg_err("pdev priv obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	reg_get_cur_6g_client_type(pdev, &client_type);

	master_chan_list =
		pdev_priv_obj->mas_chan_list_6g_client[ap_type][client_type];

	reg_find_txpower_from_6g_list(chan_freq, master_chan_list,
				      tx_power);

	*is_psd = reg_is_6g_psd_power(pdev);
	if (*is_psd)
		status = reg_get_6g_chan_psd_eirp_power(chan_freq,
							master_chan_list,
							eirp_psd_power);

	return status;
}

QDF_STATUS reg_get_client_power_for_6ghz_ap(struct wlan_objmgr_pdev *pdev,
					    enum reg_6g_client_type client_type,
					    qdf_freq_t chan_freq,
					    bool *is_psd, uint16_t *tx_power,
					    uint16_t *eirp_psd_power)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	enum reg_6g_ap_type ap_pwr_type;
	struct regulatory_channel *master_chan_list;
	QDF_STATUS status = QDF_STATUS_SUCCESS;

	status = reg_get_cur_6g_ap_pwr_type(pdev, &ap_pwr_type);
	if (!QDF_IS_STATUS_SUCCESS(status))
		return status;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!pdev_priv_obj) {
		reg_err("pdev priv obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	master_chan_list = pdev_priv_obj->
			mas_chan_list_6g_client[ap_pwr_type][client_type];

	reg_find_txpower_from_6g_list(chan_freq, master_chan_list,
				      tx_power);

	*is_psd = reg_is_6g_psd_power(pdev);
	if (*is_psd)
		status = reg_get_6g_chan_psd_eirp_power(chan_freq,
							master_chan_list,
							eirp_psd_power);

	return status;
}

/**
 * reg_get_num_rules_of_ap_pwr_type() - Get the number of reg rules present
 * for a given ap power type
 * @pdev - Pointer to pdev
 * @ap_pwr_type - AP power type
 *
 * Return: Return the number of reg rules for a given ap power type
 */
static uint8_t
reg_get_num_rules_of_ap_pwr_type(struct wlan_objmgr_pdev *pdev,
				 enum reg_6g_ap_type ap_pwr_type)
{
	struct reg_rule_info *psoc_reg_rules = reg_get_reg_rules_for_pdev(pdev);

	if (ap_pwr_type > REG_MAX_SUPP_AP_TYPE) {
		reg_err("Unsupported 6G AP power type");
		return 0;
	}

	return psoc_reg_rules->num_of_6g_ap_reg_rules[ap_pwr_type];
}

QDF_STATUS reg_set_ap_pwr_and_update_chan_list(struct wlan_objmgr_pdev *pdev,
					       enum reg_6g_ap_type ap_pwr_type)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	QDF_STATUS status;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return QDF_STATUS_E_INVAL;
	}

	if (!reg_get_num_rules_of_ap_pwr_type(pdev, ap_pwr_type))
		return QDF_STATUS_E_FAILURE;

	status = reg_set_cur_6g_ap_pwr_type(pdev, ap_pwr_type);
	if (QDF_IS_STATUS_ERROR(status)) {
		reg_debug("failed to set AP power type to %d", ap_pwr_type);
		return status;
	}

	reg_compute_pdev_current_chan_list(pdev_priv_obj);

	return QDF_STATUS_SUCCESS;
}
#endif

bool reg_is_regdb_offloaded(struct wlan_objmgr_psoc *psoc)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!psoc_priv_obj) {
		reg_err("reg psoc private obj is NULL");
		return false;
	}

	return psoc_priv_obj->offload_enabled;
}

QDF_STATUS
reg_set_ext_tpc_supported(struct wlan_objmgr_psoc *psoc, bool val)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	psoc_priv_obj->is_ext_tpc_supported = val;

	return QDF_STATUS_SUCCESS;
}

bool reg_is_ext_tpc_supported(struct wlan_objmgr_psoc *psoc)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return  false;
	}

	return psoc_priv_obj->is_ext_tpc_supported;
}

#if defined(CONFIG_BAND_6GHZ)
QDF_STATUS
reg_set_lower_6g_edge_ch_supp(struct wlan_objmgr_psoc *psoc, bool val)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	psoc_priv_obj->is_lower_6g_edge_ch_supported = val;

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS
reg_set_disable_upper_6g_edge_ch_supp(struct wlan_objmgr_psoc *psoc, bool val)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	psoc_priv_obj->is_upper_6g_edge_ch_disabled = val;

	return QDF_STATUS_SUCCESS;
}

bool reg_is_lower_6g_edge_ch_supp(struct wlan_objmgr_psoc *psoc)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return  false;
	}

	return psoc_priv_obj->is_lower_6g_edge_ch_supported;
}

bool reg_is_upper_6g_edge_ch_disabled(struct wlan_objmgr_psoc *psoc)
{
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;

	psoc_priv_obj = reg_get_psoc_obj(psoc);

	if (!IS_VALID_PSOC_REG_OBJ(psoc_priv_obj)) {
		reg_err("psoc reg component is NULL");
		return  false;
	}

	return psoc_priv_obj->is_upper_6g_edge_ch_disabled;
}
#endif

#ifdef FEATURE_WLAN_CH_AVOID_EXT
/**
 * reg_process_ch_avoid_freq_ext() - Update extended avoid frequencies in
 * psoc_priv_obj
 * @psoc: Pointer to psoc structure
 * @pdev: pointer to pdev object
 *
 * Return: None
 */
static QDF_STATUS
reg_process_ch_avoid_freq_ext(struct wlan_objmgr_psoc *psoc,
			      struct wlan_objmgr_pdev *pdev)
{
	uint32_t i;
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;
	uint8_t start_channel;
	uint8_t end_channel;
	struct ch_avoid_freq_type *range;
	enum channel_enum ch_loop;
	enum channel_enum start_ch_idx;
	enum channel_enum end_ch_idx;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!pdev_priv_obj) {
		reg_err("reg pdev private obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}
	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!psoc_priv_obj) {
		reg_err("reg psoc private obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	if (pdev_priv_obj->avoid_chan_ext_list.chan_cnt > 0) {
		uint32_t len;

		len = sizeof(pdev_priv_obj->avoid_chan_ext_list.chan_freq_list);
		pdev_priv_obj->avoid_chan_ext_list.chan_cnt = 0;
		qdf_mem_zero(&pdev_priv_obj->avoid_chan_ext_list.chan_freq_list,
			     len);
	}

	for (i = 0; i < psoc_priv_obj->avoid_freq_ext_list.ch_avoid_range_cnt;
		i++) {
		if (pdev_priv_obj->avoid_chan_ext_list.chan_cnt >=
		    NUM_CHANNELS) {
			reg_debug("ext avoid channel list full");
			break;
		}

		start_ch_idx = INVALID_CHANNEL;
		end_ch_idx = INVALID_CHANNEL;
		range = &psoc_priv_obj->avoid_freq_ext_list.avoid_freq_range[i];

		start_channel = reg_freq_to_chan(pdev, range->start_freq);
		end_channel = reg_freq_to_chan(pdev, range->end_freq);
		reg_debug("start: freq %d, ch %d, end: freq %d, ch %d",
			  range->start_freq, start_channel, range->end_freq,
			  end_channel);

		/* do not process frequency bands that are not mapped to
		 * predefined channels
		 */
		if (start_channel == 0 || end_channel == 0)
			continue;

		for (ch_loop = 0; ch_loop < NUM_CHANNELS;
			ch_loop++) {
			if (REG_CH_TO_FREQ(ch_loop) >= range->start_freq) {
				start_ch_idx = ch_loop;
				break;
			}
		}
		for (ch_loop = 0; ch_loop < NUM_CHANNELS;
			ch_loop++) {
			if (REG_CH_TO_FREQ(ch_loop) >= range->end_freq) {
				end_ch_idx = ch_loop;
				if (REG_CH_TO_FREQ(ch_loop) > range->end_freq)
					end_ch_idx--;
				break;
			}
		}

		if (start_ch_idx == INVALID_CHANNEL ||
		    end_ch_idx == INVALID_CHANNEL)
			continue;

		for (ch_loop = start_ch_idx; ch_loop <= end_ch_idx;
			ch_loop++) {
			pdev_priv_obj->avoid_chan_ext_list.chan_freq_list
			[pdev_priv_obj->avoid_chan_ext_list.chan_cnt++] =
			REG_CH_TO_FREQ(ch_loop);

			if (pdev_priv_obj->avoid_chan_ext_list.chan_cnt >=
				NUM_CHANNELS) {
				reg_debug("avoid freq ext list full");
				break;
			}
		}
		/* if start == end for 5G, meanwhile it only have one valid
		 * channel updated, then disable 20M by default around
		 * this center freq. For example input [5805-5805], it
		 * will disable 20Mhz around 5805, then the range change
		 * to [5705-5815], otherwise, not sure about how many width
		 * need to disabled for such case.
		 */
		if ((ch_loop - start_ch_idx) == 1 &&
		    (range->end_freq - range->start_freq == 0) &&
			reg_is_5ghz_ch_freq(range->start_freq)) {
			range->start_freq = range->start_freq - HALF_20MHZ_BW;
			range->end_freq = range->end_freq + HALF_20MHZ_BW;
		}
	}

	return QDF_STATUS_SUCCESS;
}

/**
 * reg_update_avoid_ch_ext() - Updates the current channel list that block out
 * by extended avoid frequency list
 * @psoc: Pointer to psoc structure
 * @object: Pointer to pdev structure
 * @arg: List of arguments
 *
 * Return: None
 */
static void
reg_update_avoid_ch_ext(struct wlan_objmgr_psoc *psoc,
			void *object, void *arg)
{
	struct wlan_objmgr_pdev *pdev = (struct wlan_objmgr_pdev *)object;
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;
	QDF_STATUS status;

	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!psoc_priv_obj) {
		reg_err("reg psoc private obj is NULL");
		return;
	}

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return;
	}

	if (psoc_priv_obj->ch_avoid_ext_ind) {
		status = reg_process_ch_avoid_freq_ext(psoc, pdev);
		if (QDF_IS_STATUS_ERROR(status))
			psoc_priv_obj->ch_avoid_ext_ind = false;
	}

	reg_compute_pdev_current_chan_list(pdev_priv_obj);
	status = reg_send_scheduler_msg_sb(psoc, pdev);

	if (QDF_IS_STATUS_ERROR(status))
		reg_err("channel change msg schedule failed");
}

QDF_STATUS
reg_process_ch_avoid_ext_event(struct wlan_objmgr_psoc *psoc,
			       struct ch_avoid_ind_type *ch_avoid_event)
{
	uint32_t i;
	struct wlan_regulatory_psoc_priv_obj *psoc_priv_obj;
	QDF_STATUS status;
	struct ch_avoid_freq_type *range;

	psoc_priv_obj = reg_get_psoc_obj(psoc);
	if (!psoc_priv_obj) {
		reg_err("reg psoc private obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	reg_debug("freq range count %d", ch_avoid_event->ch_avoid_range_cnt);

	qdf_mem_zero(&psoc_priv_obj->avoid_freq_ext_list,
		     sizeof(struct ch_avoid_ind_type));

	for (i = 0; i < ch_avoid_event->ch_avoid_range_cnt; i++) {
		range = &psoc_priv_obj->avoid_freq_ext_list.avoid_freq_range[i];
		range->start_freq =
			ch_avoid_event->avoid_freq_range[i].start_freq;
		range->end_freq =
			ch_avoid_event->avoid_freq_range[i].end_freq;
	}
	psoc_priv_obj->avoid_freq_ext_list.ch_avoid_range_cnt =
		ch_avoid_event->ch_avoid_range_cnt;

	psoc_priv_obj->ch_avoid_ext_ind = true;

	status = wlan_objmgr_psoc_try_get_ref(psoc, WLAN_REGULATORY_SB_ID);

	if (QDF_IS_STATUS_ERROR(status)) {
		reg_err("error taking psoc ref cnt");
		return status;
	}

	status = wlan_objmgr_iterate_obj_list(psoc, WLAN_PDEV_OP,
					      reg_update_avoid_ch_ext,
					      NULL, 1,
					      WLAN_REGULATORY_SB_ID);

	wlan_objmgr_psoc_release_ref(psoc, WLAN_REGULATORY_SB_ID);

	return status;
}
#endif

#if defined(CONFIG_AFC_SUPPORT) && defined(CONFIG_BAND_6GHZ)
QDF_STATUS reg_send_afc_cmd(struct wlan_objmgr_pdev *pdev,
			    struct reg_afc_resp_rx_ind_info *afc_ind_obj)
{
	uint8_t pdev_id = wlan_objmgr_pdev_get_pdev_id(pdev);
	struct wlan_objmgr_psoc *psoc;
	struct wlan_lmac_if_reg_tx_ops *tx_ops;

	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc) {
		reg_err("psoc is NULL");
		return QDF_STATUS_E_INVAL;
	}

	tx_ops = reg_get_psoc_tx_ops(psoc);
	if (tx_ops->send_afc_ind)
		return tx_ops->send_afc_ind(psoc, pdev_id, afc_ind_obj);

	return QDF_STATUS_E_FAILURE;
}

bool reg_is_afc_power_event_received(struct wlan_objmgr_pdev *pdev)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return false;
	}

	return pdev_priv_obj->is_6g_afc_power_event_received;
}

QDF_STATUS reg_get_afc_req_id(struct wlan_objmgr_pdev *pdev, uint64_t *req_id)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	*req_id = pdev_priv_obj->afc_request_id;

	return QDF_STATUS_SUCCESS;
}

bool reg_is_afc_expiry_event_received(struct wlan_objmgr_pdev *pdev)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);
	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("pdev reg component is NULL");
		return false;
	}

	return pdev_priv_obj->is_6g_afc_expiry_event_received;
}

bool reg_is_noaction_on_afc_pwr_evt(struct wlan_objmgr_pdev *pdev)
{
	struct wlan_regulatory_pdev_priv_obj *pdev_priv_obj;

	pdev_priv_obj = reg_get_pdev_obj(pdev);

	if (!IS_VALID_PDEV_REG_OBJ(pdev_priv_obj)) {
		reg_err("reg pdev priv obj is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	return pdev_priv_obj->is_reg_noaction_on_afc_pwr_evt;
}
#endif
